@@ -1,5 +1,5 @@
 /*
- * $Id: citadel_ldap.h 7898 2009-10-14 19:20:06Z ajc $
+ * $Id: citadel_ldap.h 8574 2010-05-18 21:34:56Z dothebart $
  *
  * Configuration for LDAP authentication.  Most of this stuff gets pulled out of our site config file.
  *
@@ -21,5 +21,5 @@
  */
 
 int CtdlTryUserLDAP(char *username, char *found_dn, int found_dn_size, char *fullname, int fullname_size, uid_t *found_uid);
-int CtdlTryPasswordLDAP(char *user_dn, char *password);
+int CtdlTryPasswordLDAP(char *user_dn, const char *password);
 int Ctdl_LDAP_to_vCard(char *ldap_dn, struct vCard *v);
