@@ -1,23 +1,23 @@
 /* 
- * $Id: citserver.c 8324 2010-02-17 20:03:28Z ajc $
+ * $Id: citserver.c 8605 2010-06-03 13:05:11Z dothebart $
  *
  * Main source module for the Citadel server
  *
- * Copyright (c) 1987-2009 by the citadel.org team
+ * Copyright (c) 1987-2010 by the citadel.org team
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
- *  (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "sysdep.h"
@@ -29,7 +29,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
@@ -70,7 +69,6 @@
 #include "locate_host.h"
 #include "room_ops.h"
 #include "file_ops.h"
-#include "policy.h"
 #include "control.h"
 #include "euidindex.h"
 #include "context.h"
@@ -82,7 +80,6 @@
 
 #include "ctdl_module.h"
 
-
 char *unique_session_numbers;
 int ScheduledShutdown = 0;
 time_t server_startup_time;
@@ -243,7 +240,7 @@ void master_cleanup(int exitcode) {
 	
 	if (restart_server != 0)
 		exit(1);
-	if ((running_as_daemon != 0) && (exitcode == 0))
+	if ((running_as_daemon != 0) && ((exitcode == 0) ))
 		exitcode = CTDLEXIT_SHUTDOWN;
 	exit(exitcode);
 }
@@ -680,7 +677,7 @@ int CtdlAccessCheck(int required_level) {
 		return(-1);
 	}
 
-	if (CC->user.axlevel >= 6) return(0);
+	if (CC->user.axlevel >= AxAideU) return(0);
  	if (required_level >= ac_aide) {
 		cprintf("%d This command requires Aide access.\n",
 			ERROR + HIGHER_ACCESS_REQUIRED);
@@ -971,10 +968,10 @@ void begin_session(CitContext *con)
 	}
 
 	if (!CC->is_local_socket) {
-		CtdlLogPrintf(CTDL_NOTICE, "Session started from %s [%s].\n", con->cs_host, con->cs_addr);
+		CtdlLogPrintf(CTDL_NOTICE, "Session (%s) started from %s (%s).\n", con->ServiceName, con->cs_host, con->cs_addr);
 	}
 	else {
-		CtdlLogPrintf(CTDL_NOTICE, "Session started via local socket.\n");
+		CtdlLogPrintf(CTDL_NOTICE, "Session (%s) started via local socket UID:%d.\n", con->ServiceName, con->cs_UDSclientUID);
 	}
 
 	/* Run any session startup routines registered by loadable modules */
@@ -1122,5 +1119,5 @@ CTDL_MODULE_INIT(citserver)
 		CtdlRegisterProtoHook(cmd_time, "TIME", "fetch the date and time from the server");
 	}
         /* return our Subversion id for the Log */
-	return "$Id: citserver.c 8324 2010-02-17 20:03:28Z ajc $";
+	return "$Id: citserver.c 8605 2010-06-03 13:05:11Z dothebart $";
 }
