@@ -21,19 +21,35 @@ struct CitContext {
 	struct CitContext *prev;	/* Link to previous session in list */
 	struct CitContext *next;	/* Link to next session in the list */
 
-	int state;		/* thread state (see CON_ values below) */
-	int kill_me;		/* Set to nonzero to flag for termination */
-	int client_socket;
 	int cs_pid;		/* session ID */
 	int dont_term;          /* for special activities like artv so we don't get killed */
 	time_t lastcmd;		/* time of last command executed */
 	time_t lastidle;	/* For computing idle time */
+	int state;		/* thread state (see CON_ values below) */
+	int kill_me;		/* Set to nonzero to flag for termination */
+
+	StrBuf *lBuf;
+
+	const char *Pos;        /* Our read position inside of the ReadBuf */
+	StrBuf *ReadBuf;        /* Our block buffered read buffer */
+	StrBuf *MigrateBuf;        /* Our block buffered read buffer */
+
+	const char *sPos;        /* Our read position inside of the ReadBuf */
+	StrBuf *sReadBuf;        /* Our block buffered read buffer */
+	StrBuf *sMigrateBuf;        /* Our block buffered read buffer */
+	int client_socket;
+	int is_local_socket;	/* set to 1 if client is on unix domain sock */
+	/* Redirect this session's output to a memory buffer? */
+	StrBuf *redirect_buffer;		/* the buffer */
+#ifdef HAVE_OPENSSL
+	SSL *ssl;
+	int redirect_ssl;
+#endif
 
 	char curr_user[USERNAME_SIZE];	/* name of current user */
 	int logged_in;		/* logged in */
 	int internal_pgm;	/* authenticated as internal program */
 	int nologin;		/* not allowed to log in */
-	int is_local_socket;	/* set to 1 if client is on unix domain sock */
 	int curr_view;		/* The view type for the current user/room */
 	int is_master;		/* Is this session logged in using the master user? */
 
@@ -41,8 +57,6 @@ struct CitContext {
 	time_t previous_login;	/* Date/time of previous login */
 	char lastcmdname[5];	/* name of last command executed */
 	unsigned cs_flags;	/* miscellaneous flags */
-	void (*h_command_function) (void) ;	/* service command function */
-	void (*h_async_function) (void) ;	/* do async msgs function */
 	int is_async;		/* Nonzero if client accepts async msgs */
 	int async_waiting;	/* Nonzero if there are async msgs waiting */
 	int input_waiting;	/* Nonzero if there is client input waiting */
@@ -63,6 +77,7 @@ struct CitContext {
 	char cs_inet_fn[128];			/* Friendly-name of outbound Internet mail */
 
 	FILE *download_fp;	/* Fields relating to file transfer */
+	size_t download_fp_total;
 	char download_desired_section[128];
 	FILE *upload_fp;
 	char upl_file[256];
@@ -79,14 +94,6 @@ struct CitContext {
 	/* Beginning of cryptography - session nonce */
 	char cs_nonce[NONCE_SIZE];	/* The nonce for this session's next auth transaction */
 
-	/* Redirect this session's output to a memory buffer? */
-	char *redirect_buffer;		/* the buffer */
-	size_t redirect_len;		/* length of data in buffer */
-	size_t redirect_alloc;		/* length of allocated buffer */
-#ifdef HAVE_OPENSSL
-	SSL *ssl;
-	int redirect_ssl;
-#endif
 
 	/* A linked list of all instant messages sent to us. */
 	struct ExpressMessage *FirstExpressMessage;
@@ -109,6 +116,11 @@ struct CitContext {
 	const char *ServiceName;		/* readable purpose of this session */
 	void *openid_data;			/* Data stored by the OpenID module */
 	char *ldap_dn;				/* DN of user when using AUTHMODE_LDAP */
+
+
+	void (*h_command_function) (void) ;	/* service command function */
+	void (*h_async_function) (void) ;	/* do async msgs function */
+	void (*h_greeting_function) (void) ;	/* greeting function for session startup */
 };
 
 typedef struct CitContext CitContext;
@@ -125,6 +137,8 @@ typedef struct CitContext CitContext;
  */
 enum {
 	CON_IDLE,		/* This context is doing nothing */
+	CON_GREETING,		/* This context needs to output its greeting */
+	CON_STARTING,		/* This context is outputting its greeting */
 	CON_READY,		/* This context needs attention */
 	CON_EXECUTING		/* This context is bound to a thread */
 };
@@ -142,9 +156,9 @@ void RemoveContext (struct CitContext *);
 CitContext *CreateNewContext (void);
 void context_cleanup(void);
 void kill_session (int session_to_kill);
-INLINE void become_session(struct CitContext *which_con);
 void InitializeMasterCC(void);
 void dead_session_purge(int force);
+void set_async_waiting(struct CitContext *ccptr);
 
 /* Deprecated, user CtdlBumpNewMailCounter() instead */
 void BumpNewMailCounter(long) __attribute__ ((deprecated));
@@ -156,4 +170,22 @@ int CtdlTerminateOtherSession (int session_num);
 #define TERM_ALLOWED	0x02
 #define TERM_KILLED	0x03
 #define TERM_NOTALLOWED -1
+
+/*
+ * Bind a thread to a context.  (It's inline merely to speed things up.)
+ */
+static INLINE void become_session(CitContext *which_con) {
+/*
+	pid_t tid = syscall(SYS_gettid);
+*/
+	citthread_setspecific(MyConKey, (void *)which_con );
+/*
+	CtdlLogPrintf(CTDL_DEBUG, "[%d]: Now doing %s\n", 
+		      (int) tid, 
+		      ((which_con != NULL) && (which_con->ServiceName != NULL)) ? 
+		      which_con->ServiceName:"");
+*/
+}
+
+
 #endif /* CONTEXT_H */
