@@ -1,9 +1,9 @@
 /*
- * $Id: control.c 7979 2009-10-29 17:40:39Z davew $
+ * $Id: control.c 8303 2010-02-11 23:25:14Z ajc $
  *
  * This module handles states which are global to the entire server.
  *
- * Copyright (c) 1987-2009 by the citadel.org team
+ * Copyright (c) 1987-2010 by the citadel.org team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -77,12 +77,7 @@ long control_highest_user = 0;
  */
 void lock_control(void)
 {
-#ifdef HAVE_FLOCK
-/*
- * TODO: solaris manpages describe this function, but the headers
- * don't show it! 
- */
-
+#if defined(LOCK_EX) && defined(LOCK_NB)
 	if (flock(fileno(control_fp), (LOCK_EX | LOCK_NB))) {
 		CtdlLogPrintf(CTDL_EMERG, "citserver: unable to lock %s.\n", file_citadel_control);
 		CtdlLogPrintf(CTDL_EMERG, "Is another citserver already running?\n");
@@ -238,13 +233,14 @@ void check_control(void)
 }
 
 
-/**
+/*
  * release_control - close our fd on exit
  */
 void release_control(void)
 {
-	if (control_fp != NULL)
+	if (control_fp != NULL) {
 		fclose(control_fp);
+	}
 	control_fp = NULL;
 }
 
@@ -700,7 +696,7 @@ void cmd_conf(char *argbuf)
 		extract_token(confname, argbuf, 1, '|', sizeof confname);
 		unbuffer_output();
 		cprintf("%d %s\n", SEND_LISTING, confname);
-		confptr = CtdlReadMessageBody("000", config.c_maxmsglen, NULL, 0, 0);
+		confptr = CtdlReadMessageBody(HKEY("000"), config.c_maxmsglen, NULL, 0, 0);
 		CtdlPutSysConfig(confname, confptr);
 		free(confptr);
 	}
@@ -720,8 +716,8 @@ void cmd_conf(char *argbuf)
 CTDL_MODULE_INIT(control)
 {
 	if (!threading) {
-		CtdlRegisterProtoHook(cmd_conf, "CONF", "Autoconverted. TODO: document me.");
+		CtdlRegisterProtoHook(cmd_conf, "CONF", "get/set system configuration");
 	}
 	/* return our Subversion id for the Log */
-	return "$Id: control.c 7979 2009-10-29 17:40:39Z davew $";
+	return "$Id: control.c 8303 2010-02-11 23:25:14Z ajc $";
 }
