@@ -1,5 +1,5 @@
 /*
- * $Id: database.c 7940 2009-10-23 16:22:11Z davew $
+ * $Id: database.c 8566 2010-05-11 22:04:15Z dothebart $
  *
  * This is a data store backend for the Citadel server which uses Berkeley DB.
  *
@@ -301,12 +301,23 @@ void open_databases(void)
 	 * Silently try to create the database subdirectory.  If it's
 	 * already there, no problem.
 	 */
-	mkdir(ctdl_data_dir, 0700);
-	chmod(ctdl_data_dir, 0700);
-	chown(ctdl_data_dir, CTDLUID, (-1));
-
+	if ((mkdir(ctdl_data_dir, 0700) != 0) && (errno != EEXIST)){
+		CtdlLogPrintf(CTDL_EMERG, 
+			      "unable to create database directory [%s]: %s", 
+			      ctdl_data_dir, strerror(errno));
+	}
+	if (chmod(ctdl_data_dir, 0700) != 0){
+		CtdlLogPrintf(CTDL_EMERG, 
+			      "unable to set database directory accessrights [%s]: %s", 
+			      ctdl_data_dir, strerror(errno));
+	}
+	if (chown(ctdl_data_dir, CTDLUID, (-1)) != 0){
+		CtdlLogPrintf(CTDL_EMERG, 
+			      "unable to set the owner for [%s]: %s", 
+			      ctdl_data_dir, strerror(errno));
+	}
 	CtdlLogPrintf(CTDL_DEBUG, "bdb(): Setting up DB environment\n");
-	db_env_set_func_yield(sched_yield);
+	db_env_set_func_yield((int (*)(u_long,  u_long))sched_yield);
 	ret = db_env_create(&dbenv, 0);
 	if (ret) {
 		CtdlLogPrintf(CTDL_EMERG, "bdb(): db_env_create: %s\n", db_strerror(ret));
@@ -480,12 +491,11 @@ void cdb_decompress_if_necessary(struct cdbdata *cdb)
 {
 	static int magic = COMPRESS_MAGIC;
 
-	if (cdb == NULL)
-		return;
-	if (cdb->ptr == NULL)
-		return;
-	if (memcmp(cdb->ptr, &magic, sizeof(magic)))
-		return;
+	if ((cdb == NULL) || 
+	    (cdb->ptr == NULL) || 
+	    (cdb->len < sizeof(magic)) ||
+	    (memcmp(cdb->ptr, &magic, sizeof(magic))))
+	    return;
 
 #ifdef HAVE_ZLIB
 	/* At this point we know we're looking at a compressed item. */
@@ -494,8 +504,13 @@ void cdb_decompress_if_necessary(struct cdbdata *cdb)
 	char *uncompressed_data;
 	char *compressed_data;
 	uLongf destLen, sourceLen;
+	size_t cplen;
 
-	memcpy(&zheader, cdb->ptr, sizeof(struct CtdlCompressHeader));
+	memset(&zheader, 0, sizeof(struct CtdlCompressHeader));
+	cplen = sizeof(struct CtdlCompressHeader);
+	if (sizeof(struct CtdlCompressHeader) > cdb->len)
+		cplen = cdb->len;
+	memcpy(&zheader, cdb->ptr, cplen);
 
 	compressed_data = cdb->ptr;
 	compressed_data += sizeof(struct CtdlCompressHeader);
@@ -527,7 +542,7 @@ void cdb_decompress_if_necessary(struct cdbdata *cdb)
  * Store a piece of data.  Returns 0 if the operation was successful.  If a
  * key already exists it should be overwritten.
  */
-int cdb_store(int cdb, void *ckey, int ckeylen, void *cdata, int cdatalen)
+int cdb_store(int cdb, const void *ckey, int ckeylen, void *cdata, int cdatalen)
 {
 
 	DBT dkey, ddata;
@@ -691,7 +706,7 @@ static DBC *localcursor(int cdb)
  * a struct cdbdata which it is the caller's responsibility to free later on
  * using the cdb_free() routine.
  */
-struct cdbdata *cdb_fetch(int cdb, void *key, int keylen)
+struct cdbdata *cdb_fetch(int cdb, const void *key, int keylen)
 {
 
 	struct cdbdata *tempcdb;
