@@ -1,4 +1,4 @@
-/* $Id: database.h 7898 2009-10-14 19:20:06Z ajc $
+/* $Id: database.h 8566 2010-05-11 22:04:15Z dothebart $
  *
  * Copyright (c) 1987-2009 by the citadel.org team
  *
@@ -23,9 +23,9 @@
 
 void open_databases (void);
 void close_databases (void);
-int cdb_store (int cdb, void *key, int keylen, void *data, int datalen);
+int cdb_store (int cdb, const void *key, int keylen, void *data, int datalen);
 int cdb_delete (int cdb, void *key, int keylen);
-struct cdbdata *cdb_fetch (int cdb, void *key, int keylen);
+struct cdbdata *cdb_fetch (int cdb, const void *key, int keylen);
 void cdb_free (struct cdbdata *cdb);
 void cdb_rewind (int cdb);
 struct cdbdata *cdb_next_item (int cdb);
