@@ -1,5 +1,5 @@
 /* 
- * $Id: file_ops.c 8001 2009-11-03 12:15:17Z davew $
+ * $Id: file_ops.c 8586 2010-05-21 17:48:50Z dothebart $
  *
  * Server functions which handle file transfers and room directories.
  *
@@ -15,6 +15,7 @@
 #include <ctype.h>
 #include <string.h>
 #include <sys/stat.h>
+#include <sys/mman.h>
 
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
@@ -34,7 +35,6 @@
 #include "config.h"
 #include "file_ops.h"
 #include "sysdep_decls.h"
-#include "user_ops.h"
 #include "support.h"
 #include "room_ops.h"
 #include "msgbase.h"
@@ -46,6 +46,7 @@
 #endif
 
 #include "ctdl_module.h"
+#include "user_ops.h"
 
 /*
  * network_talking_to()  --  concurrency checker
@@ -236,6 +237,7 @@ void OpenCmdResult(char *filename, const char *mime_type)
 	long filesize;
 
 	fstat(fileno(CC->download_fp), &statbuf);
+	CC->download_fp_total = statbuf.st_size;
 	filesize = (long) statbuf.st_size;
 	modtime = (time_t) statbuf.st_mtime;
 
@@ -468,7 +470,7 @@ void cmd_uimg(char *cmdbuf)
 		}
 	}
 
-	if (CC->user.axlevel >= 6) {
+	if (CC->user.axlevel >= AxAideU) {
 		snprintf(CC->upl_path, sizeof CC->upl_path, 
 				 "%s/%s",
 				 ctdl_image_dir,
@@ -483,7 +485,7 @@ void cmd_uimg(char *cmdbuf)
 	}
 
 	if ((!strcasecmp(basenm, "_floorpic_"))
-	    && (CC->user.axlevel >= 6)) {
+	    && (CC->user.axlevel >= AxAideU)) {
 		which_floor = extract_int(cmdbuf, 2);
 		snprintf(CC->upl_path, sizeof CC->upl_path,
 				 "%s/floor.%d.gif",
@@ -567,6 +569,7 @@ void cmd_ucls(char *cmd)
 {
 	FILE *fp;
 	char upload_notice[512];
+	static int seq = 0;
 
 	if (CC->upload_fp == NULL) {
 		cprintf("%d You don't have an upload file open.\n", ERROR + RESOURCE_NOT_OPEN);
@@ -577,11 +580,31 @@ void cmd_ucls(char *cmd)
 	CC->upload_fp = NULL;
 
 	if ((!strcasecmp(cmd, "1")) && (CC->upload_type != UPL_FILE)) {
-		CC->upload_type = UPL_FILE;
 		cprintf("%d Upload completed.\n", CIT_OK);
 
-		/* FIXME ... here we need to trigger a network run */
+		if (CC->upload_type == UPL_NET) {
+			char final_filename[PATH_MAX];
+		        snprintf(final_filename, sizeof final_filename,
+				"%s/%s.%04lx.%04x",
+				ctdl_netin_dir,
+				CC->net_node,
+				(long)getpid(),
+				++seq
+			);
 
+			if (link(CC->upl_path, final_filename) == 0) {
+				unlink(CC->upl_path);
+			}
+			else {
+				CtdlLogPrintf(CTDL_ALERT, "Cannot link %d to %d: %s\n",
+					CC->upl_path, final_filename, strerror(errno)
+				);
+			}
+
+			/* FIXME ... here we need to trigger a network run */
+		}
+
+		CC->upload_type = UPL_FILE;
 		return;
 	}
 
@@ -633,14 +656,22 @@ void cmd_read(char *cmdbuf)
 		return;
 	}
 
-	if (bytes > 100000) bytes = 100000;
-	buf = malloc(bytes + 1);
-
-	fseek(CC->download_fp, start_pos, 0);
-	actual_bytes = fread(buf, 1, bytes, CC->download_fp);
-	cprintf("%d %d\n", BINARY_FOLLOWS, (int)actual_bytes);
-	client_write(buf, actual_bytes);
-	free(buf);
+	buf = mmap(NULL, 
+		   CC->download_fp_total, 
+		   PROT_READ, 
+		   MAP_PRIVATE,
+		   fileno(CC->download_fp), 
+		   0);
+	
+	actual_bytes = CC->download_fp_total - start_pos;
+	if ((actual_bytes > 0) && (buf != NULL)) {
+		cprintf("%d %d\n", BINARY_FOLLOWS, (int)actual_bytes);
+		client_write(buf + start_pos, actual_bytes);
+	}
+	else {
+		cprintf("%d %s\n", ERROR, strerror(errno));
+	}
+	munmap(buf, CC->download_fp_total);
 }
 
 
@@ -726,6 +757,7 @@ void cmd_ndop(char *cmdbuf)
 	CC->dl_is_net = 1;
 
 	stat(pathname, &statbuf);
+	CC->download_fp_total = statbuf.st_size;
 	cprintf("%d %ld\n", CIT_OK, (long)statbuf.st_size);
 }
 
@@ -750,7 +782,7 @@ void cmd_nuop(char *cmdbuf)
 
 	snprintf(CC->upl_path, sizeof CC->upl_path,
 			 "%s/%s.%04lx.%04x",
-			 ctdl_netin_dir,
+			 ctdl_nettmp_dir,
 			 CC->net_node, 
 			 (long)getpid(), 
 			 ++seq);
@@ -783,19 +815,19 @@ void cmd_nuop(char *cmdbuf)
 CTDL_MODULE_INIT(file_ops)
 {
 	if (!threading) {
-		CtdlRegisterProtoHook(cmd_delf, "DELF", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_movf, "MOVF", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_open, "OPEN", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_clos, "CLOS", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_uopn, "UOPN", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_ucls, "UCLS", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_read, "READ", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_writ, "WRIT", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_oimg, "OIMG", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_ndop, "NDOP", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_nuop, "NUOP", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_uimg, "UIMG", "Autoconverted. TODO: document me.");
+		CtdlRegisterProtoHook(cmd_delf, "DELF", "Delete a file");
+		CtdlRegisterProtoHook(cmd_movf, "MOVF", "Move a file");
+		CtdlRegisterProtoHook(cmd_open, "OPEN", "Open a download file transfer");
+		CtdlRegisterProtoHook(cmd_clos, "CLOS", "Close a download file transfer");
+		CtdlRegisterProtoHook(cmd_uopn, "UOPN", "Open an upload file transfer");
+		CtdlRegisterProtoHook(cmd_ucls, "UCLS", "Close an upload file transfer");
+		CtdlRegisterProtoHook(cmd_read, "READ", "File transfer read operation");
+		CtdlRegisterProtoHook(cmd_writ, "WRIT", "File transfer write operation");
+		CtdlRegisterProtoHook(cmd_ndop, "NDOP", "Open a network spool file for download");
+		CtdlRegisterProtoHook(cmd_nuop, "NUOP", "Open a network spool file for upload");
+		CtdlRegisterProtoHook(cmd_oimg, "OIMG", "Open an image file for download");
+		CtdlRegisterProtoHook(cmd_uimg, "UIMG", "Upload an image file");
 	}
         /* return our Subversion id for the Log */
-	return "$Id: file_ops.c 8001 2009-11-03 12:15:17Z davew $";
+	return "$Id: file_ops.c 8586 2010-05-21 17:48:50Z dothebart $";
 }
