@@ -1,5 +1,5 @@
 /*
- * $Id: housekeeping.c 6432 2008-07-13 09:40:28Z dothebart $
+ * $Id: housekeeping.c 8036 2009-11-10 20:12:04Z davew $
  *
  * This file contains miscellaneous housekeeping tasks.
  *
@@ -46,43 +46,6 @@
 #include "ctdl_module.h"
 #include "threads.h"
 
-/*
- * Terminate idle sessions.  This function pounds through the session table
- * comparing the current time to each session's time-of-last-command.  If an
- * idle session is found it is terminated, then the search restarts at the
- * beginning because the pointer to our place in the list becomes invalid.
- */
-void terminate_idle_sessions(void) {
-	struct CitContext *ccptr;
-	time_t now;
-	int session_to_kill;
-	int killed = 0;
-	int longrunners = 0;
-
-	now = time(NULL);
-	session_to_kill = 0;
-	begin_critical_section(S_SESSION_TABLE);
-	for (ccptr = ContextList; ccptr != NULL; ccptr = ccptr->next) {
-		if (  (ccptr!=CC)
-	   	&& (config.c_sleeping > 0)
-	   	&& (now - (ccptr->lastcmd) > config.c_sleeping) ) {
-			if (!ccptr->dont_term) {
-				ccptr->kill_me = 1;
-				++killed;
-			}
-			else 
-				longrunners ++;
-		}
-	}
-	end_critical_section(S_SESSION_TABLE);
-	if (killed > 0)
-		CtdlLogPrintf(CTDL_INFO, "Terminated %d idle sessions\n", killed);
-	if (longrunners > 0)
-		CtdlLogPrintf(CTDL_INFO, "Didn't terminate %d protected idle sessions;\n", killed);
-}
-
-
-
 void check_sched_shutdown(void) {
 	if ((ScheduledShutdown == 1) && (ContextList == NULL)) {
 		CtdlLogPrintf(CTDL_NOTICE, "Scheduled shutdown initiating.\n");
@@ -117,7 +80,7 @@ void check_ref_counts(void) {
 	}
 
 	cdb_begin_transaction();
-	ForEachRoom(check_ref_counts_backend, (void *)new_refcounts );
+	CtdlForEachRoom(check_ref_counts_backend, (void *)new_refcounts );
 	cdb_end_transaction();
 
 	for (a=0; a<MAXFLOORS; ++a) {
