@@ -1,5 +1,5 @@
 /* 
- * $Id: internet_addressing.h 5149 2007-05-08 17:17:48Z dothebart $
+ * $Id: internet_addressing.h 8446 2010-03-13 18:04:48Z dothebart $
  *
  */
 
@@ -15,7 +15,7 @@ struct internet_address_list {
 
 int fuzzy_match(struct ctdluser *us, char *matchstring);
 void process_rfc822_addr(const char *rfc822, char *user, char *node, char *name);
-char *rfc822_fetch_field(char *rfc822, char *fieldname);
+char *rfc822_fetch_field(const char *rfc822, const char *fieldname);
 
 int IsDirectory(char *addr, int allow_masq_domains);
 void CtdlDirectoryInit(void);
@@ -23,6 +23,8 @@ void CtdlDirectoryAddUser(char *internet_addr, char *citadel_addr);
 void CtdlDirectoryDelUser(char *internet_addr, char *citadel_addr);
 int CtdlDirectoryLookup(char *target, char *internet_addr, size_t targbuflen);
 struct CtdlMessage *convert_internet_message(char *rfc822);
+struct CtdlMessage *convert_internet_message_buf(StrBuf **rfc822);
+
 int CtdlHostAlias(char *fqdn);
 char *harvest_collected_addresses(struct CtdlMessage *msg);
 
