@@ -1,51 +1,31 @@
 /*
- * $Id: serv_bio.c 6159 2008-03-29 01:15:32Z davew $
+ * $Id: serv_bio.c 7956 2009-10-25 00:26:06Z davew $
  *
  * This module implementsserver commands related to the display and
  * manipulation of user "bio" files.
  *
+ *
+ * Copyright (c) 1987-2009 by the citadel.org team
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "sysdep.h"
-#include <stdlib.h>
-#include <unistd.h>
-#include <stdio.h>
-#include <fcntl.h>
-#include <signal.h>
-#include <pwd.h>
-#include <errno.h>
-#include <sys/types.h>
-
-#if TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#else
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
-
-#include <sys/wait.h>
-#include <string.h>
-#include <limits.h>
-#include <libcitadel.h>
-#include "citadel.h"
-#include "server.h"
-#include "citserver.h"
-#include "support.h"
-#include "config.h"
-#include "control.h"
-#include "room_ops.h"
-#include "user_ops.h"
-#include "policy.h"
-#include "database.h"
-#include "msgbase.h"
-#include "citadel_dirs.h"
-
 #include "ctdl_module.h"
 
+
+
 /*
  * enter user bio
  */
@@ -86,7 +66,7 @@ void cmd_rbio(char *cmdbuf)
 	FILE *fp;
 
 	extract_token(buf, cmdbuf, 0, '|', sizeof buf);
-	if (getuser(&ruser, buf) != 0) {
+	if (CtdlGetUser(&ruser, buf) != 0) {
 		cprintf("%d No such user.\n",ERROR + NO_SUCH_USER);
 		return;
 	}
@@ -123,7 +103,7 @@ void cmd_lbio(char *cmdbuf) {
 
 	cprintf("%d\n", LISTING_FOLLOWS);
 	while (fgets(buf, sizeof buf, ls)!=NULL)
-		if (getuserbynumber(&usbuf,atol(buf))==0)
+		if (CtdlGetUserByNumber(&usbuf,atol(buf))==0)
 			cprintf("%s\n", usbuf.fullname);
 	pclose(ls);
 	cprintf("000\n");
@@ -141,7 +121,7 @@ CTDL_MODULE_INIT(bio)
 	        CtdlRegisterProtoHook(cmd_lbio, "LBIO", "List users with bios");
 	}
 	/* return our Subversion id for the Log */
-        return "$Id: serv_bio.c 6159 2008-03-29 01:15:32Z davew $";
+        return "$Id: serv_bio.c 7956 2009-10-25 00:26:06Z davew $";
 }
 
 
