@@ -1,5 +1,5 @@
 /* 
- * $Id: serv_calendar.c 8035 2009-11-10 20:00:25Z davew $ 
+ * $Id: serv_calendar.c 8550 2010-05-03 16:41:45Z ajc $ 
  *
  * This module implements iCalendar object processing and the Calendar>
  * room on a Citadel server.  It handles iCalendar objects using the
@@ -1706,7 +1706,7 @@ void ical_putics_grabtzids(icalparameter *param, void *data)
 	HashList *keys = (HashList *) data;
 	
 	if ( (keys) && (tzid) && (!IsEmptyStr(tzid)) ) {
-		Put(keys, tzid, strlen(tzid), strdup(tzid), generic_free_handler);
+		Put(keys, tzid, strlen(tzid), strdup(tzid), NULL);
 	}
 }
 
@@ -1741,7 +1741,7 @@ void ical_putics(void)
 	}
 
 	cprintf("%d Transmit data now\n", SEND_LISTING);
-	calstream = CtdlReadMessageBody("000", config.c_maxmsglen, NULL, 0, 0);
+	calstream = CtdlReadMessageBody(HKEY("000"), config.c_maxmsglen, NULL, 0, 0);
 	if (calstream == NULL) {
 		return;
 	}
@@ -2599,5 +2599,5 @@ CTDL_MODULE_INIT(calendar)
 	}
 	
 	/* return our Subversion id for the Log */
-	return "$Id: serv_calendar.c 8035 2009-11-10 20:00:25Z davew $";
+	return "$Id: serv_calendar.c 8550 2010-05-03 16:41:45Z ajc $";
 }
