@@ -1,5 +1,5 @@
 /*
- * $Id: serv_virus.c 7921 2009-10-17 12:21:17Z davew $
+ * $Id: serv_virus.c 8525 2010-04-24 09:33:15Z dothebart $
  *
  * This module allows Citadel to use clamd to filter incoming messages
  * arriving via SMTP.  For more information on clamd, visit
@@ -58,7 +58,6 @@
 #include "config.h"
 #include "control.h"
 #include "user_ops.h"
-#include "policy.h"
 #include "database.h"
 #include "msgbase.h"
 #include "internet_addressing.h"
@@ -83,8 +82,8 @@ int clamd(struct CtdlMessage *msg) {
         char portbuf[SIZ];
 	int is_virus = 0;
 	int clamhost;
-	char *msgtext;
-	size_t msglen;
+	StrBuf *msgtext;
+	CitContext *CCC;
 
 	/* Don't care if you're logged in.  You can still spread viruses.
 	 */
@@ -117,14 +116,18 @@ int clamd(struct CtdlMessage *msg) {
 		 */
 		return(0);
 	}
+	CCC=CC;
+	CCC->sReadBuf = NewStrBuf();
+	CCC->sMigrateBuf = NewStrBuf();
+	CCC->sPos = NULL;
 
 	/* Command */
 	CtdlLogPrintf(CTDL_DEBUG, "Transmitting STREAM command\n");
 	sprintf(buf, "STREAM\r\n");
-	sock_write(sock, buf, strlen(buf));
+	sock_write(&sock, buf, strlen(buf));
 
 	CtdlLogPrintf(CTDL_DEBUG, "Waiting for PORT number\n");
-        if (sock_getln(sock, buf, sizeof buf) < 0) {
+        if (sock_getln(&sock, buf, sizeof buf) < 0) {
                 goto bail;
         }
 
@@ -143,6 +146,8 @@ int clamd(struct CtdlMessage *msg) {
 		/* If the service isn't running, just pass the mail
 		 * through.  Potentially throwing away mails isn't good.
 		 */
+		FreeStrBuf(&CCC->sReadBuf);
+		FreeStrBuf(&CCC->sMigrateBuf);
 		return(0);
         }
 	else {
@@ -152,27 +157,23 @@ int clamd(struct CtdlMessage *msg) {
 
 
 	/* Message */
-	CC->redirect_buffer = malloc(SIZ);
-	CC->redirect_len = 0;
-	CC->redirect_alloc = SIZ;
+	CC->redirect_buffer = NewStrBufPlain(NULL, SIZ);
 	CtdlOutputPreLoadedMsg(msg, MT_RFC822, HEADERS_ALL, 0, 1, 0);
 	msgtext = CC->redirect_buffer;
-	msglen = CC->redirect_len;
 	CC->redirect_buffer = NULL;
-	CC->redirect_len = 0;
-	CC->redirect_alloc = 0;
 
-	sock_write(streamsock, msgtext, msglen);
-	free(msgtext);
+	sock_write(&streamsock, SKEY(msgtext));
+	FreeStrBuf(&msgtext);
 
 	/* Close the streamsocket connection; this tells clamd
 	 * that we're done.
 	 */
-	close(streamsock);
+	if (streamsock != -1)
+		close(streamsock);
 	
 	/* Response */
 	CtdlLogPrintf(CTDL_DEBUG, "Awaiting response\n");
-        if (sock_getln(sock, buf, sizeof buf) < 0) {
+        if (sock_getln(&sock, buf, sizeof buf) < 0) {
                 goto bail;
         }
         CtdlLogPrintf(CTDL_DEBUG, "<%s\n", buf);
@@ -188,6 +189,8 @@ int clamd(struct CtdlMessage *msg) {
 	}
 
 bail:	close(sock);
+	FreeStrBuf(&CCC->sReadBuf);
+	FreeStrBuf(&CCC->sMigrateBuf);
 	return(is_virus);
 }
 
@@ -201,5 +204,5 @@ CTDL_MODULE_INIT(virus)
 	}
 	
 	/* return our Subversion id for the Log */
-        return "$Id: serv_virus.c 7921 2009-10-17 12:21:17Z davew $";
+        return "$Id: serv_virus.c 8525 2010-04-24 09:33:15Z dothebart $";
 }
