@@ -1,4 +1,4 @@
-/* $Id: serv_crypto.c 7914 2009-10-15 16:29:43Z ajc $
+/* $Id: serv_crypto.c 8494 2010-03-25 23:12:09Z dothebart $
  *
  * Copyright (c) 1987-2009 by the citadel.org team
  *
@@ -260,17 +260,23 @@ void init_ssl(void)
 					*/
 
 					X509_NAME_add_entry_by_txt(name, "O",
-						MBSTRING_ASC, config.c_humannode, -1, -1, 0);
+								   MBSTRING_ASC, 
+								   (unsigned char*) config.c_humannode,
+								   -1, -1, 0);
 
 					X509_NAME_add_entry_by_txt(name, "OU",
-						MBSTRING_ASC, "Citadel server", -1, -1, 0);
+								   MBSTRING_ASC, 
+								   (unsigned const char*)"Citadel server",
+								   -1, -1, 0);
 
 					/* X509_NAME_add_entry_by_txt(name, "CN",
 						MBSTRING_ASC, config.c_fqdn, -1, -1, 0);
 					*/
 
-					X509_NAME_add_entry_by_txt(name, "CN",
-						MBSTRING_ASC, "*", -1, -1, 0);
+					X509_NAME_add_entry_by_txt(name, 
+								   "CN",
+								   MBSTRING_ASC, 
+								   (const unsigned char *)"*", -1, -1, 0);
 				
 					X509_REQ_set_subject_name(req, name);
 
@@ -387,7 +393,7 @@ void init_ssl(void)
 /*
  * client_write_ssl() Send binary data to the client encrypted.
  */
-void client_write_ssl(char *buf, int nbytes)
+void client_write_ssl(const char *buf, int nbytes)
 {
 	int retval;
 	int nremain;
@@ -425,62 +431,167 @@ void client_write_ssl(char *buf, int nbytes)
 
 
 /*
- * client_read_ssl() - read data from the encrypted layer.
+ * read data from the encrypted layer.
  */
-int client_read_ssl(char *buf, int bytes, int timeout)
+int client_read_sslbuffer(StrBuf *buf, int timeout)
 {
-#if 0
-	fd_set rfds;
-	struct timeval tv;
-	int retval;
-	int s;
-#endif
-	int len, rlen;
+	char sbuf[16384]; /* OpenSSL communicates in 16k blocks, so let's speak its native tongue. */
+	int rlen;
 	char junk[1];
+	SSL *pssl = CC->ssl;
 
-	len = 0;
-	while (len < bytes) {
-#if 0
-		/*
-		 * This code is disabled because we don't need it when
-		 * using blocking reads (which we are). -IO
-		 */
-		FD_ZERO(&rfds);
-		s = BIO_get_fd(CC->ssl->rbio, NULL);
-		FD_SET(s, &rfds);
-		tv.tv_sec = timeout;
-		tv.tv_usec = 0;
+	if (pssl == NULL) return(-1);
 
-		retval = select(s + 1, &rfds, NULL, NULL, &tv);
-
-		if (FD_ISSET(s, &rfds) == 0) {
-			return (0);
-		}
-
-#endif
-		if (SSL_want_read(CC->ssl)) {
-			if ((SSL_write(CC->ssl, junk, 0)) < 1) {
-				CtdlLogPrintf(CTDL_DEBUG, "SSL_write in client_read: %s\n", ERR_reason_error_string(ERR_get_error()));
+	while (1) {
+		if (SSL_want_read(pssl)) {
+			if ((SSL_write(pssl, junk, 0)) < 1) {
+				CtdlLogPrintf(CTDL_DEBUG, "SSL_write in client_read\n");
 			}
 		}
-		rlen = SSL_read(CC->ssl, &buf[len], bytes - len);
+		rlen = SSL_read(pssl, sbuf, sizeof(sbuf));
 		if (rlen < 1) {
 			long errval;
 
-			errval = SSL_get_error(CC->ssl, rlen);
-			if (errval == SSL_ERROR_WANT_READ ||
-			    errval == SSL_ERROR_WANT_WRITE) {
+			errval = SSL_get_error(pssl, rlen);
+			if (errval == SSL_ERROR_WANT_READ || errval == SSL_ERROR_WANT_WRITE) {
 				sleep(1);
 				continue;
 			}
 			CtdlLogPrintf(CTDL_DEBUG, "SSL_read got error %ld\n", errval);
 			endtls();
-			return (client_read_to
-				(&buf[len], bytes - len, timeout));
+			return (-1);
 		}
-		len += rlen;
+		StrBufAppendBufPlain(buf, sbuf, rlen, 0);
+		return rlen;
 	}
-	return (1);
+	return (0);
+}
+
+int client_readline_sslbuffer(StrBuf *Line, StrBuf *IOBuf, const char **Pos, int timeout)
+{
+	CitContext *CCC = CC;
+	const char *pos = NULL;
+	const char *pLF;
+	int len, rlen, retlen;
+	int nSuccessLess = 0;
+	const char *pch = NULL;
+	
+	retlen = 0;
+	if ((Line == NULL) ||
+	    (Pos == NULL) ||
+	    (IOBuf == NULL))
+	{
+		if (Pos != NULL)
+			*Pos = NULL;
+//		*Error = ErrRBLF_PreConditionFailed;
+		return -1;
+	}
+
+	pos = *Pos;
+	if ((StrLength(IOBuf) > 0) && 
+	    (pos != NULL) && 
+	    (pos < ChrPtr(IOBuf) + StrLength(IOBuf))) 
+	{
+		pch = pos;
+		pch = strchr(pch, '\n');
+		
+		if (pch == NULL) {
+			StrBufAppendBufPlain(Line, pos, 
+					     StrLength(IOBuf) - (pos - ChrPtr(IOBuf)), 0);
+			FlushStrBuf(IOBuf);
+			pos = *Pos = NULL;
+		}
+		else {
+			int n = 0;
+			if ((pch > ChrPtr(IOBuf)) && 
+			    (*(pch - 1) == '\r')) {
+				n = 1;
+			}
+			StrBufAppendBufPlain(Line, pos, 
+					     (pch - pos - n), 0);
+
+			if (StrLength(IOBuf) <= (pch - ChrPtr(IOBuf) + 1)) {
+				FlushStrBuf(IOBuf);
+				pos = *Pos = NULL;
+			}
+			else 
+				*Pos = pch + 1;
+			return StrLength(Line);
+		}
+	}
+
+	pLF = NULL;
+	while ((nSuccessLess < timeout) && 
+	       (pLF == NULL) &&
+	       (CCC->ssl != NULL)) {
+
+		rlen = client_read_sslbuffer(IOBuf, timeout);
+		if (rlen < 1) {
+//			*Error = strerror(errno);
+//			close(*fd);
+//			*fd = -1;
+			return -1;
+		}
+		else if (rlen > 0) {
+			pLF = strchr(ChrPtr(IOBuf), '\n');
+		}
+	}
+	*Pos = NULL;
+	if (pLF != NULL) {
+		pos = ChrPtr(IOBuf);
+		len = pLF - pos;
+		if (len > 0 && (*(pLF - 1) == '\r') )
+			len --;
+		StrBufAppendBufPlain(Line, pos, len, 0);
+		if (pLF + 1 >= ChrPtr(IOBuf) + StrLength(IOBuf))
+		{
+			FlushStrBuf(IOBuf);
+		}
+		else 
+			*Pos = pLF + 1;
+		return StrLength(Line);
+	}
+//	*Error = ErrRBLF_NotEnoughSentFromServer;
+	return -1;
+}
+
+
+
+int client_read_sslblob(StrBuf *Target, long bytes, int timeout)
+{
+	long bufremain;
+	long baselen;
+	int retval = 0;
+	CitContext *CCC = CC;
+
+	baselen = StrLength(Target);
+
+	if (CCC->Pos == NULL)
+		CCC->Pos = ChrPtr(CCC->ReadBuf);
+	bufremain = StrLength(CCC->ReadBuf) - 
+		(CCC->Pos - ChrPtr(CCC->ReadBuf));
+
+	if (bytes < bufremain)
+		bufremain = bytes;
+	StrBufAppendBufPlain(Target, CCC->Pos, bufremain, 0);
+	StrBufCutLeft(CCC->ReadBuf, bufremain);
+	CCC->Pos = NULL;
+
+	if (bytes > bufremain) 
+	{
+		while ((StrLength(CCC->ReadBuf) + StrLength(Target) < bytes + baselen) &&
+		       (retval >= 0))
+			retval = client_read_sslbuffer(CCC->ReadBuf, timeout);
+		if (retval >= 0) {
+			StrBufAppendBuf(Target, CCC->ReadBuf, 0); /* todo: Buf > bytes? */
+			return 1;
+		}
+		else {
+			return -1;
+		}
+	}
+	else 
+		return 1;
 }
 
 
