@@ -1,4 +1,4 @@
-/* $Id: serv_crypto.h 5351 2007-08-03 22:12:16Z davew $ */
+/* $Id: serv_crypto.h 8470 2010-03-17 22:52:36Z dothebart $ */
 
 /*
  * Number of days for which self-signed certs are valid.
@@ -14,8 +14,10 @@
 #ifdef HAVE_OPENSSL
 void destruct_ssl(void);
 void init_ssl(void);
-void client_write_ssl (char *buf, int nbytes);
-int client_read_ssl (char *buf, int bytes, int timeout);
+void client_write_ssl (const char *buf, int nbytes);
+int client_read_sslbuffer(StrBuf *buf, int timeout);
+int client_readline_sslbuffer(StrBuf *Target, StrBuf *Buffer, const char **Pos, int timeout);
+int client_read_sslblob(StrBuf *Target, long want_len, int timeout);
 void cmd_stls(char *params);
 void cmd_gtls(char *params);
 void endtls(void);
