@@ -1,5 +1,5 @@
 /*
- * $Id: serv_expire.c 8039 2009-11-10 21:22:20Z davew $
+ * $Id: serv_expire.c 8610 2010-06-07 22:06:50Z dothebart $
  *
  * This module handles the expiry of old messages and the purging of old users.
  *
@@ -831,7 +831,7 @@ int PurgeStaleOpenIDassociations(void) {
 			memcpy(&usernum, cdboi->ptr, sizeof(long));
 			if (CtdlGetUserByNumber(&usbuf, usernum) != 0) {
 				deleteme = strdup(cdboi->ptr + sizeof(long)),
-				Put(keys, deleteme, strlen(deleteme), deleteme, generic_free_handler);
+				Put(keys, deleteme, strlen(deleteme), deleteme, NULL);
 			}
 		}
 		cdb_free(cdboi);
@@ -864,10 +864,9 @@ void *purge_databases(void *args)
         struct tm tm;
 	struct CitContext purgerCC;
 
-	CtdlLogPrintf(CTDL_DEBUG, "Auto-purger_thread() initializing\n");
-
 	CtdlFillSystemContext(&purgerCC, "purger");
 	citthread_setspecific(MyConKey, (void *)&purgerCC );
+	CtdlLogPrintf(CTDL_DEBUG, "Auto-purger_thread() initializing\n");
 
         while (!CtdlThreadCheckStop()) {
                 /* Do the auto-purge if the current hour equals the purge hour,
@@ -945,6 +944,7 @@ void *purge_databases(void *args)
                 	CtdlLogPrintf(CTDL_INFO, "Auto-purger: STOPPED.\n");
 
         }
+	CtdlClearSystemContext();
         return NULL;
 }
 /*****************************************************************************/
@@ -1052,9 +1052,12 @@ CTDL_MODULE_INIT(expire)
 	{
 		/* CtdlRegisterProtoHook(cmd_fsck, "FSCK", "Check message ref counts"); */
 		CtdlRegisterProtoHook(cmd_tdap, "TDAP", "Manually initiate auto-purger");
+
+		CtdlRegisterProtoHook(cmd_gpex, "GPEX", "Autoconverted. TODO: document me.");
+		CtdlRegisterProtoHook(cmd_spex, "SPEX", "Autoconverted. TODO: document me.");
 	}
 	else
 		CtdlThreadCreate("Auto Purger", CTDLTHREAD_BIGSTACK, purge_databases, NULL);
 	/* return our Subversion id for the Log */
-	return "$Id: serv_expire.c 8039 2009-11-10 21:22:20Z davew $";
+	return "$Id: serv_expire.c 8610 2010-06-07 22:06:50Z dothebart $";
 }
