@@ -1,5 +1,5 @@
 /*
- * $Id: ft_wordbreaker.c 7914 2009-10-15 16:29:43Z ajc $
+ * $Id: ft_wordbreaker.c 8378 2010-02-25 22:43:57Z dothebart $
  *
  * Default wordbreaker module for full text indexing.
  *
@@ -191,15 +191,15 @@ int intcmp(const void *rec1, const void *rec2) {
 }
 
 
-void wordbreaker(char *text, int *num_tokens, int **tokens) {
+void wordbreaker(const char *text, int *num_tokens, int **tokens) {
 
 	int wb_num_tokens = 0;
 	int wb_num_alloc = 0;
 	int *wb_tokens = NULL;
 
-	char *ptr;
-	char *word_start;
-	char *word_end;
+	const char *ptr;
+	const char *word_start;
+	const char *word_end;
 	char ch;
 	int word_len;
 	char word[256];
