@@ -1,5 +1,5 @@
 /*
- * $Id: ft_wordbreaker.h 7914 2009-10-15 16:29:43Z ajc $
+ * $Id: ft_wordbreaker.h 8378 2010-02-25 22:43:57Z dothebart $
  *
  *
  * Copyright (c) 2005-2009 by the citadel.org team
@@ -33,7 +33,7 @@
 #define WB_MIN			4	// nothing with 3 or less chars
 #define WB_MAX			40
 
-void wordbreaker(char *text, int *num_tokens, int **tokens);
+void wordbreaker(const char *text, int *num_tokens, int **tokens);
 
 void initialize_noise_words(void);
 void noise_word_cleanup(void);
