@@ -1,5 +1,5 @@
 /*
- * $Id: serv_fulltext.c 8025 2009-11-10 03:58:32Z ajc $
+ * $Id: serv_fulltext.c 8610 2010-06-07 22:06:50Z dothebart $
  *
  * This module handles fulltext indexing of the message base.
  *
@@ -123,7 +123,8 @@ void ft_index_message(long msgnum, int op) {
 	int *tokens = NULL;
 	int i, j;
 	struct cdbdata *cdb_bucket;
-	char *msgtext;
+	StrBuf *msgtext;
+	char *txt;
 	int tok;
 	struct CtdlMessage *msg = NULL;
 
@@ -146,18 +147,15 @@ void ft_index_message(long msgnum, int op) {
 	/* Output the message as text before indexing it, so we don't end up
 	 * indexing a bunch of encoded base64, etc.
 	 */
-	CC->redirect_buffer = malloc(SIZ);
-	CC->redirect_len = 0;
-	CC->redirect_alloc = SIZ;
+	CC->redirect_buffer = NewStrBufPlain(NULL, SIZ);
 	CtdlOutputPreLoadedMsg(msg, MT_CITADEL, HEADERS_ALL, 0, 1, 0);
 	CtdlFreeMessage(msg);
 	msgtext = CC->redirect_buffer;
 	CC->redirect_buffer = NULL;
-	CC->redirect_len = 0;
-	CC->redirect_alloc = 0;
 	CtdlLogPrintf(CTDL_DEBUG, "Wordbreaking message %ld...\n", msgnum);
-	wordbreaker(msgtext, &num_tokens, &tokens);
-	free(msgtext);
+	txt = SmashStrBuf(&msgtext);
+	wordbreaker(txt, &num_tokens, &tokens);
+	free(txt);
 
 	CtdlLogPrintf(CTDL_DEBUG, "Indexing message %ld [%d tokens]\n", msgnum, num_tokens);
 	if (num_tokens > 0) {
@@ -376,10 +374,10 @@ void do_fulltext_indexing(void) {
 void *indexer_thread(void *arg) {
 	struct CitContext indexerCC;
 
-	CtdlLogPrintf(CTDL_DEBUG, "indexer_thread() initializing\n");
 
 	CtdlFillSystemContext(&indexerCC, "indexer");
 	citthread_setspecific(MyConKey, (void *)&indexerCC );
+	CtdlLogPrintf(CTDL_DEBUG, "indexer_thread() initializing\n");
 
 	while (!CtdlThreadCheckStop()) {
 		do_fulltext_indexing();
@@ -387,6 +385,7 @@ void *indexer_thread(void *arg) {
 	}
 
 	CtdlLogPrintf(CTDL_DEBUG, "indexer_thread() exiting\n");
+	CtdlClearSystemContext();
 	return NULL;
 }
 
@@ -397,7 +396,7 @@ void *indexer_thread(void *arg) {
  * (This one does the "all of these words" search.)
  * Caller is responsible for freeing the message list.
  */
-void ft_search(int *fts_num_msgs, long **fts_msgs, char *search_string) {
+void ft_search(int *fts_num_msgs, long **fts_msgs, const char *search_string) {
 	int num_tokens = 0;
 	int *tokens = NULL;
 	int i, j;
@@ -537,5 +536,5 @@ CTDL_MODULE_INIT(fulltext)
 		CtdlThreadCreate("Indexer", CTDLTHREAD_BIGSTACK, indexer_thread, NULL);
 	}
 	/* return our Subversion id for the Log */
-	return "$Id: serv_fulltext.c 8025 2009-11-10 03:58:32Z ajc $";
+	return "$Id: serv_fulltext.c 8610 2010-06-07 22:06:50Z dothebart $";
 }
