@@ -1,5 +1,5 @@
 /*
- * $Id: serv_fulltext.h 7914 2009-10-15 16:29:43Z ajc $
+ * $Id: serv_fulltext.h 8378 2010-02-25 22:43:57Z dothebart $
  *
  *
  * Copyright (c) 2005-2009 by the citadel.org team
@@ -20,5 +20,5 @@
  */
 
 void ft_index_message(long msgnum, int op);
-void ft_search(int *fts_num_msgs, long **fts_msgs, char *search_string);
+void ft_search(int *fts_num_msgs, long **fts_msgs, const char *search_string);
 void *indexer_thread(void *arg);
