@@ -1,5 +1,5 @@
 /*
- * $Id: imap_fetch.h 7914 2009-10-15 16:29:43Z ajc $
+ * $Id: imap_fetch.h 8439 2010-03-11 20:33:17Z dothebart $
  *
  *
  * Copyright (c) 2001-2009 by the citadel.org team
@@ -19,8 +19,8 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-void imap_pick_range(char *range, int is_uid);
-void imap_fetch(int num_parms, char *parms[]);
-void imap_uidfetch(int num_parms, char *parms[]);
+void imap_pick_range(const char *range, int is_uid);
+void imap_fetch(int num_parms, ConstStr *Params);
+void imap_uidfetch(int num_parms, ConstStr *Params);
 void imap_fetch_flags(int seq);
-int imap_extract_data_items(char **argv, char *items);
+int imap_extract_data_items(citimap_command *Cmd);
