@@ -1,5 +1,5 @@
 /*
- * $Id: imap_list.h 5351 2007-08-03 22:12:16Z davew $
+ * $Id: imap_list.h 8378 2010-02-25 22:43:57Z dothebart $
  */
 
 /*
@@ -8,4 +8,4 @@
  */
 #define MAX_PATTERNS 20
 
-void imap_list(int num_parms, char *parms[]);
+void imap_list(int num_parms, ConstStr *Params);
