@@ -1,5 +1,5 @@
 /*
- * $Id: imap_metadata.h 7914 2009-10-15 16:29:43Z ajc $
+ * $Id: imap_metadata.h 8378 2010-02-25 22:43:57Z dothebart $
  *
  *
  * Copyright (c) 2007-2009 by the citadel.org team
@@ -19,5 +19,5 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-void imap_getmetadata(int num_parms, char *parms[]);
-void imap_setmetadata(int num_parms, char *parms[]);
+void imap_getmetadata(int num_parms, ConstStr *Params);
+void imap_setmetadata(int num_parms, ConstStr *Params);
