@@ -1,5 +1,5 @@
 /*
- * $Id: imap_misc.h 7914 2009-10-15 16:29:43Z ajc $
+ * $Id: imap_misc.h 8378 2010-02-25 22:43:57Z dothebart $
  *
  *
  * Copyright (c) 2001-2009 by the citadel.org team
@@ -19,6 +19,6 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-void imap_copy(int num_parms, char *parms[]);
-void imap_uidcopy(int num_parms, char *parms[]);
-void imap_append(int num_parms, char *parms[]);
+void imap_copy(int num_parms, ConstStr *Params);
+void imap_uidcopy(int num_parms, ConstStr *Params);
+void imap_append(int num_parms, ConstStr *Params);
