@@ -1,5 +1,5 @@
 /*
- * $Id: imap_tools.c 8060 2009-11-18 15:35:41Z ajc $
+ * $Id: imap_tools.c 8716 2010-07-26 06:08:06Z dothebart $
  *
  * Utility functions for the IMAP module.
  *
@@ -31,6 +31,7 @@
 #include "sysdep_decls.h"
 #include "internet_addressing.h"
 #include "imap_tools.h"
+#include "serv_imap.h"
 #include "ctdl_module.h"
 
 #ifndef HAVE_SNPRINTF
@@ -83,7 +84,7 @@ static void string_append_sn(struct string* s, char* p, int len)
 
 static void string_append_c(struct string* s, int c)
 {
-	char buf[5];
+	char UmlChar[5];
 	int len = 0;
 
 	/* Don't do anything if there's no room. */
@@ -101,27 +102,27 @@ static void string_append_c(struct string* s, int c)
 	}
 	else if (c <= 0x7FF)
 	{
-		buf[0] = 0xC0 | (c >> 6);
-		buf[1] = 0x80 | (c & 0x3F);
+		UmlChar[0] = 0xC0 | (c >> 6);
+		UmlChar[1] = 0x80 | (c & 0x3F);
 		len = 2;
 	}
 	else if (c <= 0xFFFF)
 	{
-		buf[0] = 0xE0 | (c >> 12);
-		buf[1] = 0x80 | ((c >> 6) & 0x3f);
-		buf[2] = 0x80 | (c & 0x3f);
+		UmlChar[0] = 0xE0 | (c >> 12);
+		UmlChar[1] = 0x80 | ((c >> 6) & 0x3f);
+		UmlChar[2] = 0x80 | (c & 0x3f);
 		len = 3;
 	}
 	else
 	{
-		buf[0] = 0xf0 | c >> 18;
-		buf[1] = 0x80 | ((c >> 12) & 0x3f);
-		buf[2] = 0x80 | ((c >> 6) & 0x3f);
-		buf[3] = 0x80 | (c & 0x3f);
+		UmlChar[0] = 0xf0 | c >> 18;
+		UmlChar[1] = 0x80 | ((c >> 12) & 0x3f);
+		UmlChar[2] = 0x80 | ((c >> 6) & 0x3f);
+		UmlChar[3] = 0x80 | (c & 0x3f);
 		len = 4;
 	}
 
-	string_append_sn(s, buf, len);
+	string_append_sn(s, UmlChar, len);
 }	
 
 /* Reads a UTF8 character from a char*, advancing the pointer. */
@@ -297,10 +298,10 @@ static char* toimap(char* destp, char* destend, char* src)
 /* Convert from an IMAP-safe name back into a Citadel name. Returns the end of the destination. */
 
 static int cfrommap(int c);
-static char* fromimap(char* destp, char* destend, char* src)
+static char* fromimap(char* destp, char* destend, const char* src)
 {
 	struct string dest;
-	unsigned char *p = (unsigned char*) src;
+	unsigned const char *p = (unsigned const char*) src;
 	int v = 0;
 	int i = 0;
 	int state = 0;
@@ -385,7 +386,7 @@ static int cfrommap(int c)
 /* Output a string to the IMAP client, either as a literal or quoted.
  * (We do a literal if it has any double-quotes or backslashes.) */
 
-void imap_strout(char *buf)
+void plain_imap_strout(char *buf)
 {
 	int i;
 	int is_literal = 0;
@@ -409,9 +410,203 @@ void imap_strout(char *buf)
 	}
 }
 
+/* Output a string to the IMAP client, either as a literal or quoted.
+ * (We do a literal if it has any double-quotes or backslashes.) */
+
+void imap_strout(ConstStr *args)
+{
+	int i;
+	int is_literal = 0;
+	
+	if ((args == NULL) || (args->len == 0))
+	{	/* yeah, we handle this */
+		cprintf("NIL");
+		return;
+	}
+
+	for (i = 0; i < args->len; ++i) {
+		if ((args->Key[i] == '\"') || (args->Key[i] == '\\'))
+			is_literal = 1;
+	}
+
+	if (is_literal) {
+		cprintf("{%ld}\r\n%s", args->len, args->Key);
+	} else {
+		cprintf("\"%s\"", args->Key);
+	}
+}
+
+
+
+
 /* Break a command down into tokens, unquoting any escaped characters. */
 
-int imap_parameterize(char** args, char* in)
+void MakeStringOf(StrBuf *Buf, int skip)
+{
+	int i;
+	citimap_command *Cmd = &IMAP->Cmd;
+
+	for (i=skip; i<Cmd->num_parms; ++i) {
+		StrBufAppendBufPlain(Buf, Cmd->Params[i].Key, Cmd->Params[i].len, 0);
+		if (i < (Cmd->num_parms-1)) StrBufAppendBufPlain(Buf, HKEY(" "), 0);
+	}
+}
+
+
+void TokenCutRight(citimap_command *Cmd, 
+		   ConstStr *CutMe,
+		   int n)
+{
+	const char *CutAt;
+
+	if (CutMe->len < n) {
+		CutAt = CutMe->Key;
+		CutMe->len = 0;
+	}
+	else {
+		CutAt = CutMe->Key + CutMe->len - n;
+		CutMe->len -= n;
+	}
+	StrBufPeek(Cmd->CmdBuf, CutAt, -1, '\0');
+}
+
+void TokenCutLeft(citimap_command *Cmd, 
+		  ConstStr *CutMe,
+		  int n)
+{
+	if (CutMe->len < n) {
+		CutMe->Key += CutMe->len;
+		CutMe->len = 0;
+	}
+	else {
+		CutMe->Key += n;
+		CutMe->len -= n;
+	}
+}
+
+
+
+int CmdAdjust(citimap_command *Cmd, 
+	      int nArgs,
+	      int Realloc)
+{
+	ConstStr *Params;
+	if (nArgs > Cmd->avail_parms) {
+		Params = (ConstStr*) malloc(sizeof(ConstStr) * nArgs);
+		if (Realloc) {
+			memcpy(Params, 
+			       Cmd->Params, 
+			       sizeof(ConstStr) * Cmd->avail_parms);
+
+			memset(Cmd->Params + 
+			       sizeof(ConstStr) * Cmd->avail_parms,
+			       0, 
+			       sizeof(ConstStr) * nArgs - 
+			       sizeof(ConstStr) * Cmd->avail_parms 
+				);
+		}
+		else {
+			Cmd->num_parms = 0;
+			memset(Params, 0, 
+			       sizeof(ConstStr) * nArgs);
+		}
+		Cmd->avail_parms = nArgs;
+		if (Cmd->Params != NULL)
+			free (Cmd->Params);
+		Cmd->Params = Params;
+	}
+	else {
+		if (!Realloc) {
+			memset(Cmd->Params, 
+			       0,
+			       sizeof(ConstStr) * Cmd->avail_parms);
+			Cmd->num_parms = 0;
+		}
+	}
+	return Cmd->avail_parms;
+}
+
+int imap_parameterize(citimap_command *Cmd)
+{
+	int nArgs;
+	const char *In, *End;
+
+	In = ChrPtr(Cmd->CmdBuf);
+	End = In + StrLength(Cmd->CmdBuf);
+
+	/* we start with 10 chars per arg, maybe we need to realloc later. */
+	nArgs = StrLength(Cmd->CmdBuf) / 10 + 10;
+	nArgs = CmdAdjust(Cmd, nArgs, 0);
+	while (In < End)
+	{
+		/* Skip whitespace. */
+		while (isspace(*In))
+			In++;
+		if (*In == '\0')
+			break;
+
+		/* Found the start of a token. */
+		
+		Cmd->Params[Cmd->num_parms].Key = In;
+
+		/* Read in the token. */
+
+		for (;;)
+		{
+			if (isspace(*In))
+				break;
+			
+			if (*In == '\"')
+			{
+				/* Found a quoted section. */
+
+				Cmd->Params[Cmd->num_parms].Key++; 
+				//In++;
+				for (;;)
+				{
+					In++;
+					if (*In == '\"') {
+						StrBufPeek(Cmd->CmdBuf, In, -1, '\0');
+						break;
+					}
+					else if (*In == '\\')
+						In++;
+
+					if (*In == '\0') {
+						Cmd->Params[Cmd->num_parms].len = 
+							In - Cmd->Params[Cmd->num_parms].Key;
+						Cmd->num_parms++;
+						return Cmd->num_parms;
+					}
+				}
+				break;
+			}
+			else if (*In == '\\')
+			{
+				In++;
+			}
+
+			if (*In == '\0') {
+				Cmd->Params[Cmd->num_parms].len = 
+					In - Cmd->Params[Cmd->num_parms].Key;
+				Cmd->num_parms++;
+				return Cmd->num_parms;
+			}
+			In++;
+		}
+		StrBufPeek(Cmd->CmdBuf, In, -1, '\0');
+		Cmd->Params[Cmd->num_parms].len = 
+			In - Cmd->Params[Cmd->num_parms].Key;
+		if (Cmd->num_parms + 1 >= Cmd->avail_parms) {
+			nArgs = CmdAdjust(Cmd, nArgs * 2, 1);
+		}
+		Cmd->num_parms ++;
+		In++;
+	}
+	return Cmd->num_parms;
+}
+
+int old_imap_parameterize(char** args, char *in)
 {
 	char* out = in;
 	int num = 0;
@@ -519,10 +714,10 @@ void imap_mailboxname(char *buf, int bufsize, struct ctdlroom *qrbuf)
  *
  */
 
-int imap_roomname(char *rbuf, int bufsize, char *foldername)
+int imap_roomname(char *rbuf, int bufsize, const char *foldername)
 {
 	int levels;
-	char floorname[256];
+	char floorname[ROOMNAMELEN*2];
 	char roomname[ROOMNAMELEN];
 	int i;
 	struct floor *fl;
@@ -565,10 +760,12 @@ int imap_roomname(char *rbuf, int bufsize, char *foldername)
 	levels = num_tokens(rbuf, FDELIM);
 	if (levels > 1)
 	{
+		long len;
 		/* Extract the main room name. */
 		
-		extract_token(floorname, rbuf, 0, FDELIM, sizeof floorname);
-		strcpy(roomname, &rbuf[strlen(floorname)+1]);
+		len = extract_token(floorname, rbuf, 0, FDELIM, sizeof floorname);
+		if (len < 0) len = 0;
+		safestrncpy(roomname, &rbuf[len  + 1], sizeof(roomname));
 
 		/* Try and find it on any floor. */
 		
@@ -614,11 +811,11 @@ void imap_ial_out(struct internet_address_list *ialist)
 
 	for (iptr = ialist; iptr != NULL; iptr = iptr->next) {
 		cprintf("(");
-		imap_strout(iptr->ial_name);
+		plain_imap_strout(iptr->ial_name);
 		cprintf(" NIL ");
-		imap_strout(iptr->ial_user);
+		plain_imap_strout(iptr->ial_user);
 		cprintf(" ");
-		imap_strout(iptr->ial_node);
+		plain_imap_strout(iptr->ial_node);
 		cprintf(")");
 	}
 
@@ -633,7 +830,7 @@ void imap_ial_out(struct internet_address_list *ialist)
  * return 1 for a valid message set.  If any other character is found, 
  * return 0.
  */
-int imap_is_message_set(char *buf)
+int imap_is_message_set(const char *buf)
 {
 	int i;
 
@@ -677,19 +874,30 @@ static int do_imap_match(const char *supplied_text, const char *supplied_p)
 {
 	int matched, i;
 	char lcase_text[SIZ], lcase_p[SIZ];
-	char *text = lcase_text;
-	char *p = lcase_p;
-
+	char *text;
+	char *p;
+	
 	/* Copy both strings and lowercase them, in order to
 	 * make this entire operation case-insensitive.
 	 */
-	for (i=0; i<=strlen(supplied_text); ++i)
+	for (i=0; 
+	     ((supplied_text[i] != '\0') && 
+	      (i < sizeof(lcase_text)));
+	     ++i)
 		lcase_text[i] = tolower(supplied_text[i]);
-	for (i=0; i<=strlen(supplied_p); ++i)
-		p[i] = tolower(supplied_p[i]);
+	lcase_text[i] = '\0';
+
+	for (i=0; 
+	     ((supplied_p[i] != '\0') && 
+	      (i < sizeof(lcase_p))); 
+	     ++i)
+		lcase_p[i] = tolower(supplied_p[i]);
+	lcase_p[i] = '\0';
 
 	/* Start matching */
-	for (; *p; text++, p++) {
+	for (p = lcase_p, text = lcase_text; 
+	     !IsEmptyStr(p) && !IsEmptyStr(text); 
+	     text++, p++) {
 		if ((*text == '\0') && (*p != '*') && (*p != '%')) {
 			return WILDMAT_ABORT;
 		}
@@ -719,7 +927,8 @@ star:
 			}
 			return WILDMAT_ABORT;
 		case '%':
-			while (++p, ((*p == '*') || (*p == '%'))) {
+			while (++p, (!IsEmptyStr(p) && ((*p == '*') || (*p == '%'))))
+			{
 				/* Consecutive %'s act just like one, but even
 				 * a single star makes the sequence act like
 				 * one star, instead.
@@ -734,7 +943,7 @@ star:
 				 * Trailing % matches everything
 				 * without a delimiter.
 				 */
-				while (*text) {
+				while (!IsEmptyStr(text)) {
 					if (*text == WILDMAT_DELIM) {
 						return WILDMAT_FALSE;
 					}
@@ -742,7 +951,7 @@ star:
 				}
 				return WILDMAT_TRUE;
 			}
-			while (*text && (*(text - 1) != WILDMAT_DELIM)) {
+			while (!IsEmptyStr(text) && (*(text - 1) != WILDMAT_DELIM)) {
 				if ((matched = do_imap_match(text++, p))
 				   != WILDMAT_FALSE) {
 					return matched;
@@ -752,7 +961,8 @@ star:
 		}
 	}
 
-	return (*text == '\0');
+	if ((*text == '\0') && (*p == '\0')) return WILDMAT_TRUE;
+	else return WILDMAT_FALSE;
 }
 
 
@@ -761,7 +971,7 @@ star:
  * Support function for mailbox pattern name matching in LIST and LSUB
  * Returns nonzero if the supplied mailbox name matches the supplied pattern.
  */
-int imap_mailbox_matches_pattern(char *pattern, char *mailboxname)
+int imap_mailbox_matches_pattern(const char *pattern, char *mailboxname)
 {
 	/* handle just-star case quickly */
 	if ((pattern[0] == '*') && (pattern[1] == '\0')) {
@@ -776,7 +986,7 @@ int imap_mailbox_matches_pattern(char *pattern, char *mailboxname)
  * Compare an IMAP date string (date only, no time) to the date found in
  * a Unix timestamp.
  */
-int imap_datecmp(char *datestr, time_t msgtime) {
+int imap_datecmp(const char *datestr, time_t msgtime) {
 	char daystr[256];
 	char monthstr[256];
 	char yearstr[256];
