@@ -1,5 +1,5 @@
 /*
- * $Id: serv_managesieve.c 7921 2009-10-17 12:21:17Z davew $
+ * $Id: serv_managesieve.c 8574 2010-05-18 21:34:56Z dothebart $
  *
  * This module is an managesieve implementation for the Citadel system.
  * It is compliant with all of the following:
@@ -62,7 +62,6 @@
 #include "config.h"
 #include "control.h"
 #include "user_ops.h"
-#include "policy.h"
 #include "database.h"
 #include "msgbase.h"
 #include "internet_addressing.h"
@@ -244,11 +243,12 @@ void cmd_mgsve_auth(int num_parms, char **parms, struct sdm_userdata *u)
 		if (login_ok == CtdlLoginExistingUser(NULL, username))
 		{
 			char *pass;
+
 			pass = &(auth[strlen(auth)+1]);
 			/* for some reason the php script sends us the username twice. y? */
 			pass = &(pass[strlen(pass)+1]);
 			
-			if (pass_ok == CtdlTryPassword(pass))
+			if (pass_ok == CtdlTryPassword(pass, strlen(pass)))
 			{
 				MGSVE->command_state = mgsve_password;
 				cprintf("OK\r\n");
@@ -523,7 +523,7 @@ void managesieve_command_loop(void) {
 	memset(cmdbuf, 0, sizeof cmdbuf); /* Clear it, just in case */
 	length = client_getln(cmdbuf, sizeof cmdbuf);
 	if (length >= 1) {
-		num_parms = imap_parameterize(parms, cmdbuf);
+		num_parms = old_imap_parameterize(parms, cmdbuf);
 		if (num_parms == 0) return;
 		length = strlen(parms[0]);
 	}
@@ -614,7 +614,7 @@ CTDL_MODULE_INIT(managesieve)
 	}
 	
 	/* return our Subversion id for the Log */
-	return "$Id: serv_managesieve.c 7921 2009-10-17 12:21:17Z davew $";
+	return "$Id: serv_managesieve.c 8574 2010-05-18 21:34:56Z dothebart $";
 }
 
 
