@@ -1,14 +1,14 @@
 /*
- * $Id: serv_migrate.c 8054 2009-11-16 22:02:11Z dothebart $
- *
- * Copyright (c) 2000-2009 by the citadel.org development team
+ * $Id: serv_migrate.c 8430 2010-03-05 21:19:07Z ajc $
  *
  * This module dumps and/or loads the Citadel database in XML format.
  *
+ * Copyright (c) 1987-2010 by the citadel.org team
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3, or (at your option)
- * any later version.
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,9 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
- * 
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "sysdep.h"
@@ -797,7 +795,7 @@ void migr_xml_end(void *data, const char *el) {
 				}
 			}
 			if (msgcount > 0) {
-				CtdlSaveMsgPointersInRoom(FRname, msglist, msgcount, 0, NULL);
+				CtdlSaveMsgPointersInRoom(FRname, msglist, msgcount, 0, NULL, 1);
 			}
 			free(msglist);
 			msglist = NULL;
@@ -990,5 +988,5 @@ CTDL_MODULE_INIT(migrate)
 	}
 	
 	/* return our Subversion id for the Log */
-	return "$Id: serv_migrate.c 8054 2009-11-16 22:02:11Z dothebart $";
+	return "$Id: serv_migrate.c 8430 2010-03-05 21:19:07Z ajc $";
 }
