@@ -1,10 +1,10 @@
 /*
- * $Id: serv_network.c 8056 2009-11-17 04:27:39Z ajc $ 
+ * $Id: serv_network.c 8675 2010-07-11 22:09:08Z dothebart $ 
  *
  * This module handles shared rooms, inter-Citadel mail, and outbound
  * mailing list processing.
  *
- * Copyright (c) 2000-2009 by the citadel.org team
+ * Copyright (c) 2000-2010 by the citadel.org team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -68,7 +68,6 @@
 #include "support.h"
 #include "config.h"
 #include "user_ops.h"
-#include "policy.h"
 #include "database.h"
 #include "msgbase.h"
 #include "internet_addressing.h"
@@ -463,6 +462,7 @@ void network_deliver_digest(SpoolControl *sc) {
 	long msglen;
 	char *recps = NULL;
 	size_t recps_len = SIZ;
+	size_t siz;
 	struct recptypes *valid;
 	namelist *nptr;
 	char bounce_to[256];
@@ -509,8 +509,8 @@ void network_deliver_digest(SpoolControl *sc) {
 
 	msg->cm_fields['M'] = malloc(msglen + 1);
 	fseek(sc->digestfp, 0L, SEEK_SET);
-	fread(msg->cm_fields['M'], (size_t)msglen, 1, sc->digestfp);
-	msg->cm_fields['M'][msglen] = 0;
+	siz = fread(msg->cm_fields['M'], (size_t)msglen, 1, sc->digestfp);
+	msg->cm_fields['M'][msglen] = '\0';
 
 	fclose(sc->digestfp);
 	sc->digestfp = NULL;
@@ -647,7 +647,19 @@ void network_spool_msg(long msgnum, void *userdata) {
 		 */
 		msg = CtdlFetchMessage(msgnum, 1);
 		if (msg != NULL) {
-
+			if (msg->cm_fields['V'] == NULL){
+				/* local message, no enVelope */
+				StrBuf *Buf;
+				Buf = NewStrBuf();
+				StrBufAppendBufPlain(Buf, msg->cm_fields['O'], -1, 0);
+				StrBufAppendBufPlain(Buf, HKEY("@"), 0);
+				StrBufAppendBufPlain(Buf, config.c_fqdn, -1, 0);
+				
+				msg->cm_fields['K'] = SmashStrBuf(&Buf);
+			}
+			else {
+				msg->cm_fields['K'] = strdup (msg->cm_fields['V']);
+			}
 			/* Set the 'List-ID' header */
 			if (msg->cm_fields['L'] != NULL) {
 				free(msg->cm_fields['L']);
@@ -715,20 +727,17 @@ void network_spool_msg(long msgnum, void *userdata) {
 				fprintf(sc->digestfp, "Subject: %s\n", msg->cm_fields['U']);
 			}
 
-			CC->redirect_buffer = malloc(SIZ);
-			CC->redirect_len = 0;
-			CC->redirect_alloc = SIZ;
-
+			CC->redirect_buffer = NewStrBufPlain(NULL, SIZ);
+			
 			safestrncpy(CC->preferred_formats, "text/plain", sizeof CC->preferred_formats);
 			CtdlOutputPreLoadedMsg(msg, MT_CITADEL, HEADERS_NONE, 0, 0, 0);
 
-			striplt(CC->redirect_buffer);
-			fprintf(sc->digestfp, "\n%s\n", CC->redirect_buffer);
+			StrBufTrim(CC->redirect_buffer);
+			fwrite(HKEY("\n"), 1, sc->digestfp);
+			fwrite(SKEY(CC->redirect_buffer), 1, sc->digestfp);
+			fwrite(HKEY("\n"), 1, sc->digestfp);
 
-			free(CC->redirect_buffer);
-			CC->redirect_buffer = NULL;
-			CC->redirect_len = 0;
-			CC->redirect_alloc = 0;
+			FreeStrBuf(&CC->redirect_buffer);
 
 			sc->num_msgs_spooled += 1;
 			free(msg);
@@ -877,10 +886,13 @@ void network_spool_msg(long msgnum, void *userdata) {
 				serialize_message(&sermsg, msg);
 				if (sermsg.len > 0) {
 
-					/* write it to the spool file */
-					snprintf(filename, sizeof filename,"%s/%s",
-						 	ctdl_netout_dir,
-						 	mptr->remote_nodename);
+					/* write it to a spool file */
+					snprintf(filename, sizeof filename,"%s/%s@%lx%x",
+					 	ctdl_netout_dir,
+					 	mptr->remote_nodename,
+						time(NULL),
+						rand()
+					);
 					CtdlLogPrintf(CTDL_DEBUG, "Appending to %s\n", filename);
 					fp = fopen(filename, "ab");
 					if (fp != NULL) {
@@ -938,31 +950,31 @@ int read_spoolcontrol_file(SpoolControl **scc, char *filename)
 		buf[strlen(buf)-1] = 0;
 
 		extract_token(instr, buf, 0, '|', sizeof instr);
-		if (!strcasecmp(instr, "lastsent")) {
+		if (!strcasecmp(instr, strof(lastsent))) {
 			sc->lastsent = extract_long(buf, 1);
 		}
-		else if (!strcasecmp(instr, "listrecp")) {
+		else if (!strcasecmp(instr, strof(listrecp))) {
 			nptr = (namelist *)
 				malloc(sizeof(namelist));
 			nptr->next = sc->listrecps;
 			extract_token(nptr->name, buf, 1, '|', sizeof nptr->name);
 			sc->listrecps = nptr;
 		}
-		else if (!strcasecmp(instr, "participate")) {
+		else if (!strcasecmp(instr, strof(participate))) {
 			nptr = (namelist *)
 				malloc(sizeof(namelist));
 			nptr->next = sc->participates;
 			extract_token(nptr->name, buf, 1, '|', sizeof nptr->name);
 			sc->participates = nptr;
 		}
-		else if (!strcasecmp(instr, "digestrecp")) {
+		else if (!strcasecmp(instr, strof(digestrecp))) {
 			nptr = (namelist *)
 				malloc(sizeof(namelist));
 			nptr->next = sc->digestrecps;
 			extract_token(nptr->name, buf, 1, '|', sizeof nptr->name);
 			sc->digestrecps = nptr;
 		}
-		else if (!strcasecmp(instr, "ignet_push_share")) {
+		else if (!strcasecmp(instr, strof(ignet_push_share))) {
 			extract_token(nodename, buf, 1, '|', sizeof nodename);
 			extract_token(roomname, buf, 2, '|', sizeof roomname);
 			mptr = (maplist *) malloc(sizeof(maplist));
@@ -977,12 +989,12 @@ int read_spoolcontrol_file(SpoolControl **scc, char *filename)
 			 * timestamps.
 			 */
 			skipthisline = 0;
-			if (!strncasecmp(buf, "subpending|", 11)) {
+			if (!strncasecmp(buf, strof(subpending)"|", 11)) {
 				if (time(NULL) - extract_long(buf, 4) > EXP) {
 					skipthisline = 1;
 				}
 			}
-			if (!strncasecmp(buf, "unsubpending|", 13)) {
+			if (!strncasecmp(buf, strof(unsubpending)"|", 13)) {
 				if (time(NULL) - extract_long(buf, 3) > EXP) {
 					skipthisline = 1;
 				}
@@ -1540,15 +1552,17 @@ void network_process_buffer(char *buffer, long size) {
 					strcpy(nexthop, msg->cm_fields['D']);
 				}
 				snprintf(filename, 
-						 sizeof filename,
-						 "%s/%s",
-						 ctdl_netout_dir,
-						 nexthop);
+					sizeof filename,
+					"%s/%s@%lx%x",
+					ctdl_netout_dir,
+					nexthop,
+					time(NULL),
+					rand()
+				);
 				CtdlLogPrintf(CTDL_DEBUG, "Appending to %s\n", filename);
 				fp = fopen(filename, "ab");
 				if (fp != NULL) {
-					fwrite(sermsg.ser,
-						sermsg.len, 1, fp);
+					fwrite(sermsg.ser, sermsg.len, 1, fp);
 					fclose(fp);
 				}
 				else {
@@ -1649,8 +1663,9 @@ void network_process_message(FILE *fp, long msgstart, long msgend) {
 	buffer = malloc(size);
 	if (buffer != NULL) {
 		fseek(fp, msgstart, SEEK_SET);
-		fread(buffer, size, 1, fp);
-		network_process_buffer(buffer, size);
+		if (fread(buffer, size, 1, fp) > 0) {
+			network_process_buffer(buffer, size);
+		}
 		free(buffer);
 	}
 
@@ -1734,10 +1749,11 @@ void network_do_spoolin(void) {
 	while (d = readdir(dp), d != NULL) {
 		if ((strcmp(d->d_name, ".")) && (strcmp(d->d_name, ".."))) {
 			snprintf(filename, 
-					 sizeof filename,
-					 "%s/%s",
-					 ctdl_netin_dir,
-					 d->d_name);
+				sizeof filename,
+				"%s/%s",
+				ctdl_netin_dir,
+				d->d_name
+			);
 			network_process_file(filename);
 		}
 	}
@@ -1746,15 +1762,60 @@ void network_do_spoolin(void) {
 }
 
 /*
- * Delete any files in the outbound queue that were intended
- * to be sent to nodes which no longer exist.
+ * Step 1: consolidate files in the outbound queue into one file per neighbor node
+ * Step 2: delete any files in the outbound queue that were for neighbors who no longer exist.
  */
-void network_purge_spoolout(void) {
+void network_consolidate_spoolout(void) {
 	DIR *dp;
 	struct dirent *d;
 	char filename[PATH_MAX];
+	char cmd[PATH_MAX];
 	char nexthop[256];
 	int i;
+	char *ptr;
+
+	/* Step 1: consolidate files in the outbound queue into one file per neighbor node */
+	dp = opendir(ctdl_netout_dir);
+	if (dp == NULL) return;
+	while (d = readdir(dp), d != NULL) {
+		if (
+			(strcmp(d->d_name, "."))
+			&& (strcmp(d->d_name, ".."))
+			&& (strchr(d->d_name, '@') != NULL)
+		) {
+			safestrncpy(nexthop, d->d_name, sizeof nexthop);
+			ptr = strchr(nexthop, '@');
+			if (ptr) *ptr = 0;
+	
+			snprintf(filename, 
+				sizeof filename,
+				"%s/%s",
+				ctdl_netout_dir,
+				d->d_name
+			);
+	
+			CtdlLogPrintf(CTDL_DEBUG, "Consolidate %s to %s\n", filename, nexthop);
+			if (network_talking_to(nexthop, NTT_CHECK)) {
+				CtdlLogPrintf(CTDL_DEBUG,
+					"Currently online with %s - skipping for now\n",
+					nexthop
+				);
+			}
+			else {
+				network_talking_to(nexthop, NTT_ADD);
+				snprintf(cmd, sizeof cmd, "/bin/cat %s >>%s/%s && /bin/rm -f %s",
+					filename,
+					ctdl_netout_dir, nexthop,
+					filename
+				);
+				system(cmd);
+				network_talking_to(nexthop, NTT_REMOVE);
+			}
+		}
+	}
+	closedir(dp);
+
+	/* Step 2: delete any files in the outbound queue that were for neighbors who no longer exist */
 
 	dp = opendir(ctdl_netout_dir);
 	if (dp == NULL) return;
@@ -1762,11 +1823,15 @@ void network_purge_spoolout(void) {
 	while (d = readdir(dp), d != NULL) {
 		if (!strcmp(d->d_name, ".") || !strcmp(d->d_name, ".."))
 			continue;
+		ptr = strchr(d->d_name, '@');
+		if (d != NULL)
+			continue;
 		snprintf(filename, 
-				 sizeof filename,
-				 "%s/%s",
-				 ctdl_netout_dir,
-				 d->d_name);
+			sizeof filename,
+			"%s/%s",
+			ctdl_netout_dir,
+			d->d_name
+		);
 
 		strcpy(nexthop, "");
 		i = is_valid_node(nexthop, NULL, d->d_name);
@@ -1784,18 +1849,34 @@ void network_purge_spoolout(void) {
 /*
  * receive network spool from the remote system
  */
-void receive_spool(int sock, char *remote_nodename) {
+void receive_spool(int *sock, char *remote_nodename) {
 	long download_len = 0L;
 	long bytes_received = 0L;
-	long bytes_copied = 0L;
 	char buf[SIZ];
 	static char pbuf[IGNET_PACKET_SIZE];
 	char tempfilename[PATH_MAX];
-	char filename[PATH_MAX];
+	char permfilename[PATH_MAX];
 	long plen;
-	FILE *fp, *newfp;
+	FILE *fp;
+
+	snprintf(tempfilename, 
+		sizeof tempfilename, 
+		"%s/%s.%lx%x",
+		ctdl_nettmp_dir,
+		remote_nodename, 
+		time(NULL),
+		rand()
+	);
+
+	snprintf(permfilename, 
+		sizeof permfilename, 
+		"%s/%s.%lx%x",
+		ctdl_netin_dir,
+		remote_nodename, 
+		time(NULL),
+		rand()
+	);
 
-	CtdlMakeTempFileName(tempfilename, sizeof tempfilename);
 	if (sock_puts(sock, "NDOP") < 0) return;
 	if (sock_getln(sock, buf, sizeof buf) < 0) return;
 	CtdlLogPrintf(CTDL_DEBUG, "<%s\n", buf);
@@ -1804,63 +1885,70 @@ void receive_spool(int sock, char *remote_nodename) {
 	}
 	download_len = extract_long(&buf[4], 0);
 
-	bytes_received = 0L;
-	fp = fopen(tempfilename, "w");
-	if (fp == NULL) {
-		CtdlLogPrintf(CTDL_CRIT, "cannot open download file locally: %s\n",
-			strerror(errno));
-		return;
-	}
+	if (download_len>0) {
+		bytes_received = 0L;
+		fp = fopen(tempfilename, "w");
+		if (fp == NULL) {
+			CtdlLogPrintf(CTDL_CRIT, "cannot open download file locally: %s\n",
+				      strerror(errno));
+			return;
+		}
 
-	while (bytes_received < download_len) {
-		/**
-		 * If shutting down we can exit here and unlink the temp file.
-		 * this shouldn't loose us any messages.
-		 */
-		if (CtdlThreadCheckStop())
-		{
-			fclose(fp);
-			unlink(tempfilename);
-			return;
-		}
-		snprintf(buf, sizeof buf, "READ %ld|%ld",
-			bytes_received,
-		     ((download_len - bytes_received > IGNET_PACKET_SIZE)
-		 ? IGNET_PACKET_SIZE : (download_len - bytes_received)));
-		if (sock_puts(sock, buf) < 0) {
-			fclose(fp);
-			unlink(tempfilename);
-			return;
-		}
-		if (sock_getln(sock, buf, sizeof buf) < 0) {
-			fclose(fp);
-			unlink(tempfilename);
-			return;
-		}
-		if (buf[0] == '6') {
-			plen = extract_long(&buf[4], 0);
-			if (sock_read(sock, pbuf, plen, 1) < 0) {
+		CtdlLogPrintf(CTDL_DEBUG, "For this download we are expecting %d bytes\n", download_len);
+		while (bytes_received < download_len) {
+			/*
+			 * If shutting down we can exit here and unlink the temp file.
+			 * this shouldn't loose us any messages.
+			 */
+			if (CtdlThreadCheckStop())
+			{
 				fclose(fp);
 				unlink(tempfilename);
 				return;
 			}
-			fwrite((char *) pbuf, plen, 1, fp);
-			bytes_received = bytes_received + plen;
+			snprintf(buf, sizeof buf, "READ %ld|%ld",
+				 bytes_received,
+				 ((download_len - bytes_received > IGNET_PACKET_SIZE)
+				  ? IGNET_PACKET_SIZE : (download_len - bytes_received)));
+			
+			if (sock_puts(sock, buf) < 0) {
+				fclose(fp);
+				unlink(tempfilename);
+				return;
+			}
+			if (sock_getln(sock, buf, sizeof buf) < 0) {
+				fclose(fp);
+				unlink(tempfilename);
+				return;
+			}
+			
+			if (buf[0] == '6') {
+				plen = extract_long(&buf[4], 0);
+				if (sock_read(sock, pbuf, plen, 1) < 0) {
+					fclose(fp);
+					unlink(tempfilename);
+					return;
+				}
+				fwrite((char *) pbuf, plen, 1, fp);
+				bytes_received = bytes_received + plen;
+			}
 		}
-	}
 
-	fclose(fp);
-	/** Last chance for shutdown exit */
+		fclose(fp);
+	}
+	/* Last chance for shutdown exit */
 	if (CtdlThreadCheckStop())
 	{
 		unlink(tempfilename);
 		return;
 	}
+
 	if (sock_puts(sock, "CLOS") < 0) {
 		unlink(tempfilename);
 		return;
 	}
-	/**
+
+	/*
 	 * From here on we must complete or messages will get lost
 	 */
 	if (sock_getln(sock, buf, sizeof buf) < 0) {
@@ -1872,36 +1960,13 @@ void receive_spool(int sock, char *remote_nodename) {
 	}
 	CtdlLogPrintf(CTDL_DEBUG, "%s\n", buf);
 	
-	/* Now copy the temp file to its permanent location.
-	 * (We copy instead of link because they may be on different filesystems)
+	/* Now move the temp file to its permanent location.
 	 */
-	begin_critical_section(S_NETSPOOL);
-	snprintf(filename, 
-			 sizeof filename, 
-			 "%s/%s.%ld",
-			 ctdl_netin_dir,
-			 remote_nodename, 
-			 (long) getpid()
-	);
-	fp = fopen(tempfilename, "r");
-	if (fp != NULL) {
-		newfp = fopen(filename, "w");
-		if (newfp != NULL) {
-			bytes_copied = 0L;
-			while (bytes_copied < download_len) {
-				plen = download_len - bytes_copied;
-				if (plen > sizeof buf) {
-					plen = sizeof buf;
-				}
-				fread(buf, plen, 1, fp);
-				fwrite(buf, plen, 1, newfp);
-				bytes_copied += plen;
-			}
-			fclose(newfp);
-		}
-		fclose(fp);
+	if (link(tempfilename, permfilename) != 0) {
+		CtdlLogPrintf(CTDL_ALERT, "Could not link %s to %s: %s\n",
+			tempfilename, permfilename, strerror(errno)
+		);
 	}
-	end_critical_section(S_NETSPOOL);
 	unlink(tempfilename);
 }
 
@@ -1910,7 +1975,7 @@ void receive_spool(int sock, char *remote_nodename) {
 /*
  * transmit network spool to the remote system
  */
-void transmit_spool(int sock, char *remote_nodename)
+void transmit_spool(int *sock, char *remote_nodename)
 {
 	char buf[SIZ];
 	char pbuf[4096];
@@ -1927,14 +1992,14 @@ void transmit_spool(int sock, char *remote_nodename)
 	}
 
 	snprintf(sfname, sizeof sfname, 
-			 "%s/%s",
-			 ctdl_netout_dir,
-			 remote_nodename);
+		"%s/%s",
+		ctdl_netout_dir,
+		remote_nodename
+	);
 	fd = open(sfname, O_RDONLY);
 	if (fd < 0) {
 		if (errno != ENOENT) {
-			CtdlLogPrintf(CTDL_CRIT, "cannot open upload file locally: %s\n",
-				strerror(errno));
+			CtdlLogPrintf(CTDL_CRIT, "cannot open %s: %s\n", sfname, strerror(errno));
 		}
 		return;
 	}
@@ -1942,7 +2007,7 @@ void transmit_spool(int sock, char *remote_nodename)
 	while (plen = (long) read(fd, pbuf, IGNET_PACKET_SIZE), plen > 0L) {
 		bytes_to_write = plen;
 		while (bytes_to_write > 0L) {
-			/** Exit if shutting down */
+			/* Exit if shutting down */
 			if (CtdlThreadCheckStop())
 			{
 				close(fd);
@@ -1960,8 +2025,7 @@ void transmit_spool(int sock, char *remote_nodename)
 			}
 			thisblock = atol(&buf[4]);
 			if (buf[0] == '7') {
-				if (sock_write(sock, pbuf,
-				   (int) thisblock) < 0) {
+				if (sock_write(sock, pbuf, (int) thisblock) < 0) {
 					close(fd);
 					return;
 				}
@@ -1975,17 +2039,18 @@ void transmit_spool(int sock, char *remote_nodename)
 
 ABORTUPL:
 	close(fd);
-	/** Last chance for shutdown exit */
+
+	/* Last chance for shutdown exit */
 	if(CtdlThreadCheckStop())
 		return;
 		
 	if (sock_puts(sock, "UCLS 1") < 0) return;
-	/**
+
+	/*
 	 * From here on we must complete or messages will get lost
 	 */
 	if (sock_getln(sock, buf, sizeof buf) < 0) return;
-	CtdlLogPrintf(CTDL_NOTICE, "Sent %ld octets to <%s>\n",
-			bytes_written, remote_nodename);
+	CtdlLogPrintf(CTDL_NOTICE, "Sent %ld octets to <%s>\n", bytes_written, remote_nodename);
 	CtdlLogPrintf(CTDL_DEBUG, "<%s\n", buf);
 	if (buf[0] == '2') {
 		CtdlLogPrintf(CTDL_DEBUG, "Removing <%s>\n", sfname);
@@ -2003,9 +2068,11 @@ void network_poll_node(char *node, char *secret, char *host, char *port) {
 	char buf[SIZ];
 	char err_buf[SIZ];
 	char connected_to[SIZ];
+	CitContext *CCC=CC;
 
 	if (network_talking_to(node, NTT_CHECK)) return;
 	network_talking_to(node, NTT_ADD);
+	CtdlLogPrintf(CTDL_DEBUG, "network: polling <%s>\n", node);
 	CtdlLogPrintf(CTDL_NOTICE, "Connecting to <%s> at %s:%s\n", node, host, port);
 
 	sock = sock_connect(host, port, "tcp");
@@ -2016,16 +2083,22 @@ void network_poll_node(char *node, char *secret, char *host, char *port) {
 	}
 	
 	CtdlLogPrintf(CTDL_DEBUG, "Connected!\n");
+	CCC->sReadBuf = NewStrBuf();
+	CCC->sMigrateBuf = NewStrBuf();
+	CCC->sPos = NULL;
 
 	/* Read the server greeting */
-	if (sock_getln(sock, buf, sizeof buf) < 0) goto bail;
+	if (sock_getln(&sock, buf, sizeof buf) < 0) goto bail;
 	CtdlLogPrintf(CTDL_DEBUG, ">%s\n", buf);
 
 	/* Check that the remote is who we think it is and warn the Aide if not */
 	extract_token (connected_to, buf, 1, ' ', sizeof connected_to);
 	if (strcmp(connected_to, node))
 	{
-		snprintf (err_buf, sizeof(err_buf), "Connected to node \"%s\" but I was expecting to connect to node \"%s\".", connected_to, node);
+		snprintf(err_buf, sizeof(err_buf),
+			"Connected to node \"%s\" but I was expecting to connect to node \"%s\".",
+			connected_to, node
+		);
 		CtdlLogPrintf(CTDL_ERR, "%s\n", err_buf);
 		CtdlAideMessage(err_buf, "Network error");
 	}
@@ -2033,20 +2106,26 @@ void network_poll_node(char *node, char *secret, char *host, char *port) {
 		/* We're talking to the correct node.  Now identify ourselves. */
 		snprintf(buf, sizeof buf, "NETP %s|%s", config.c_nodename, secret);
 		CtdlLogPrintf(CTDL_DEBUG, "<%s\n", buf);
-		if (sock_puts(sock, buf) <0) goto bail;
-		if (sock_getln(sock, buf, sizeof buf) < 0) goto bail;
+		if (sock_puts(&sock, buf) <0) goto bail;
+		if (sock_getln(&sock, buf, sizeof buf) < 0) goto bail;
 		CtdlLogPrintf(CTDL_DEBUG, ">%s\n", buf);
-		if (buf[0] != '2') goto bail;
+		if (buf[0] != '2') {
+			goto bail;
+		}
 	
 		/* At this point we are authenticated. */
 		if (!CtdlThreadCheckStop())
-			receive_spool(sock, node);
+			receive_spool(&sock, node);
 		if (!CtdlThreadCheckStop())
-			transmit_spool(sock, node);
+			transmit_spool(&sock, node);
 	}
 
-	sock_puts(sock, "QUIT");
-bail:	sock_close(sock);
+	sock_puts(&sock, "QUIT");
+bail:	
+	FreeStrBuf(&CCC->sReadBuf);
+	FreeStrBuf(&CCC->sMigrateBuf);
+	if (sock != -1)
+		sock_close(sock);
 	network_talking_to(node, NTT_REMOVE);
 }
 
@@ -2068,7 +2147,7 @@ void network_poll_other_citadel_nodes(int full_poll) {
 	char spoolfile[256];
 
 	if (working_ignetcfg == NULL) {
-		CtdlLogPrintf(CTDL_DEBUG, "No nodes defined - not polling\n");
+		CtdlLogPrintf(CTDL_DEBUG, "network: no neighbor nodes are configured - not polling.\n");
 		return;
 	}
 
@@ -2086,10 +2165,11 @@ void network_poll_other_citadel_nodes(int full_poll) {
 			poll = full_poll;
 			if (poll == 0) {
 				snprintf(spoolfile, 
-						 sizeof spoolfile,
-						 "%s/%s",
-						 ctdl_netout_dir, 
-						 node);
+					 sizeof spoolfile,
+					 "%s/%s",
+					 ctdl_netout_dir, 
+					 node
+				);
 				if (access(spoolfile, R_OK) == 0) {
 					poll = 1;
 				}
@@ -2109,12 +2189,22 @@ void network_poll_other_citadel_nodes(int full_poll) {
  * It's ok if these directories already exist.  Just fail silently.
  */
 void create_spool_dirs(void) {
-	mkdir(ctdl_spool_dir, 0700);
-	chown(ctdl_spool_dir, CTDLUID, (-1));
-	mkdir(ctdl_netin_dir, 0700);
-	chown(ctdl_netin_dir, CTDLUID, (-1));
-	mkdir(ctdl_netout_dir, 0700);
-	chown(ctdl_netout_dir, CTDLUID, (-1));
+	if ((mkdir(ctdl_spool_dir, 0700) != 0) && (errno != EEXIST))
+		CtdlLogPrintf(CTDL_EMERG, "unable to create directory [%s]: %s", ctdl_spool_dir, strerror(errno));
+	if (chown(ctdl_spool_dir, CTDLUID, (-1)) != 0)
+		CtdlLogPrintf(CTDL_EMERG, "unable to set the access rights for [%s]: %s", ctdl_spool_dir, strerror(errno));
+	if ((mkdir(ctdl_netin_dir, 0700) != 0) && (errno != EEXIST))
+		CtdlLogPrintf(CTDL_EMERG, "unable to create directory [%s]: %s", ctdl_netin_dir, strerror(errno));
+	if (chown(ctdl_netin_dir, CTDLUID, (-1)) != 0)
+		CtdlLogPrintf(CTDL_EMERG, "unable to set the access rights for [%s]: %s", ctdl_netin_dir, strerror(errno));
+	if ((mkdir(ctdl_nettmp_dir, 0700) != 0) && (errno != EEXIST))
+		CtdlLogPrintf(CTDL_EMERG, "unable to create directory [%s]: %s", ctdl_nettmp_dir, strerror(errno));
+	if (chown(ctdl_nettmp_dir, CTDLUID, (-1)) != 0)
+		CtdlLogPrintf(CTDL_EMERG, "unable to set the access rights for [%s]: %s", ctdl_nettmp_dir, strerror(errno));
+	if ((mkdir(ctdl_netout_dir, 0700) != 0) && (errno != EEXIST))
+		CtdlLogPrintf(CTDL_EMERG, "unable to create directory [%s]: %s", ctdl_netout_dir, strerror(errno));
+	if (chown(ctdl_netout_dir, CTDLUID, (-1)) != 0)
+		CtdlLogPrintf(CTDL_EMERG, "unable to set the access rights for [%s]: %s", ctdl_netout_dir, strerror(errno));
 }
 
 
@@ -2142,7 +2232,9 @@ void *network_do_queue(void *args) {
 	 */
 	if ( (time(NULL) - last_run) < config.c_net_freq ) {
 		full_processing = 0;
-		CtdlLogPrintf(CTDL_DEBUG, "Network full processing in %ld seconds.\n", config.c_net_freq - (time(NULL)- last_run));
+		CtdlLogPrintf(CTDL_DEBUG, "Network full processing in %ld seconds.\n",
+			config.c_net_freq - (time(NULL)- last_run)
+		);
 	}
 
 	/*
@@ -2151,7 +2243,10 @@ void *network_do_queue(void *args) {
 	 * don't really require extremely fine granularity here, we'll do it
 	 * with a static variable instead.
 	 */
-	if (doing_queue) return NULL;
+	if (doing_queue) {
+		CtdlClearSystemContext();
+		return NULL;
+	}
 	doing_queue = 1;
 
 	/* Load the IGnet Configuration into memory */
@@ -2205,8 +2300,9 @@ void *network_do_queue(void *args) {
 	}
 
 	/* If there is anything in the inbound queue, process it */
-	if (!CtdlThreadCheckStop())
+	if (!CtdlThreadCheckStop()) {
 		network_do_spoolin();
+	}
 
 	/* Save the network map back to disk */
 	write_network_map();
@@ -2215,7 +2311,7 @@ void *network_do_queue(void *args) {
 	free_filter_list(filterlist);
 	filterlist = NULL;
 
-	network_purge_spoolout();
+	network_consolidate_spoolout();
 
 	CtdlLogPrintf(CTDL_DEBUG, "network: queue run completed\n");
 
@@ -2224,10 +2320,19 @@ void *network_do_queue(void *args) {
 	}
 
 	doing_queue = 0;
-	if (!CtdlThreadCheckStop()) // Only reschedule if system is not stopping
-		CtdlThreadSchedule("IGnet Network", CTDLTHREAD_BIGSTACK, network_do_queue, NULL, time(NULL) + 60);
-	else
+
+	/* Reschedule this task to happen again periodically, unless the thread system indicates
+	 * that the server is shutting down.
+	 */
+	if (!CtdlThreadCheckStop()) {
+		CtdlThreadSchedule("IGnet Network", CTDLTHREAD_BIGSTACK,
+			network_do_queue, NULL, time(NULL) + 60
+		);
+	}
+	else {
 		CtdlLogPrintf(CTDL_DEBUG, "network: Task STOPPED.\n");
+	}
+	CtdlClearSystemContext();
 	return NULL;
 }
 
@@ -2250,12 +2355,6 @@ void cmd_netp(char *cmdbuf)
 	extract_token(node, cmdbuf, 0, '|', sizeof node);
 	extract_token(pass, cmdbuf, 1, '|', sizeof pass);
 
-	if (doing_queue) {
-		CtdlLogPrintf(CTDL_WARNING, "Network node <%s> refused - spooling\n", node);
-		cprintf("%d spooling - try again in a few minutes\n", ERROR + RESOURCE_BUSY);
-		return;
-	}
-
 	/* load the IGnet Configuration to check node validity */
 	load_working_ignetcfg();
 	v = is_valid_node(nexthop, secret, node);
@@ -2322,5 +2421,5 @@ CTDL_MODULE_INIT(network)
 	else
 		CtdlThreadSchedule("IGnet Network", CTDLTHREAD_BIGSTACK, network_do_queue, NULL, 0);
 	/* return our Subversion id for the Log */
-	return "$Id: serv_network.c 8056 2009-11-17 04:27:39Z ajc $";
+	return "$Id: serv_network.c 8675 2010-07-11 22:09:08Z dothebart $";
 }
