@@ -1,24 +1,24 @@
 /*
- * $Id: serv_newuser.c 8004 2009-11-05 15:11:18Z davew $
+ * $Id: serv_newuser.c 8525 2010-04-24 09:33:15Z dothebart $
  *
- * Automaticalyl copies the contents of a "New User Greetings" room to the
+ * Automatically copies the contents of a "New User Greetings" room to the
  * inbox of any new user upon account creation.
  *
- * Copyright (c) 1987-2009 by the citadel.org team
+ * Copyright (c) 1987-2010 by the citadel.org team
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
- *  (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 /*
@@ -60,7 +60,6 @@
 #include "support.h"
 #include "config.h"
 #include "user_ops.h"
-#include "policy.h"
 #include "database.h"
 #include "msgbase.h"
 
@@ -89,7 +88,7 @@ void CopyNewUserGreetings(void) {
 	 * or if it's not private.
 	 */
 	if (CtdlGetRoom(&CC->room, NEWUSERGREETINGS) != 0) return;
-	if (! CC->room.QRflags & QR_PRIVATE ) return;
+	if ((CC->room.QRflags & QR_PRIVATE) == 0) return;
 
 	cdbfr = cdb_fetch(CDB_MSGLISTS, &CC->room.QRnumber, sizeof(long));
 
@@ -101,7 +100,7 @@ void CopyNewUserGreetings(void) {
 	}
 
 	if (num_msgs > 0) {
-		CtdlSaveMsgPointersInRoom(mailboxname, msglist, num_msgs, 1, NULL);
+		CtdlSaveMsgPointersInRoom(mailboxname, msglist, num_msgs, 1, NULL, 0);
 	}
 
 	/* Now free the memory we used, and go away. */
@@ -117,5 +116,5 @@ CTDL_MODULE_INIT(newuser)
 	}
 	
 	/* return our Subversion id for the Log */
-	return "$Id: serv_newuser.c 8004 2009-11-05 15:11:18Z davew $";
+	return "$Id: serv_newuser.c 8525 2010-04-24 09:33:15Z dothebart $";
 }
