@@ -1,5 +1,5 @@
 /*
- * $Id: serv_pop3client.c 7921 2009-10-17 12:21:17Z davew $
+ * $Id: serv_pop3client.c 8617 2010-06-09 22:37:37Z dothebart $
  *
  * Consolidate mail from remote POP3 accounts.
  *
@@ -82,6 +82,7 @@ void pop3_do_fetching(char *roomname, char *pop3host, char *pop3user, char *pop3
 	char utmsgid[SIZ];
 	struct cdbdata *cdbut;
 	struct UseTable ut;
+	CitContext *CCC=CC;
 
 	CtdlLogPrintf(CTDL_DEBUG, "POP3: %s %s %s <password>\n", roomname, pop3host, pop3user);
 	CtdlLogPrintf(CTDL_NOTICE, "Connecting to <%s>\n", pop3host);
@@ -99,9 +100,12 @@ void pop3_do_fetching(char *roomname, char *pop3host, char *pop3user, char *pop3
 		goto bail;
 
 	CtdlLogPrintf(CTDL_DEBUG, "Connected!\n");
+	CCC->sReadBuf = NewStrBuf();
+	CCC->sMigrateBuf = NewStrBuf();
+	CCC->sPos = NULL;
 
 	/* Read the server greeting */
-	if (sock_getln(sock, buf, sizeof buf) < 0) goto bail;
+	if (sock_getln(&sock, buf, sizeof buf) < 0) goto bail;
 	CtdlLogPrintf(CTDL_DEBUG, ">%s\n", buf);
 	if (strncasecmp(buf, "+OK", 3)) goto bail;
 
@@ -115,8 +119,8 @@ void pop3_do_fetching(char *roomname, char *pop3host, char *pop3user, char *pop3
 	 */
 	snprintf(buf, sizeof buf, "USER %s\r", pop3user);
 	CtdlLogPrintf(CTDL_DEBUG, "<%s\n", buf);
-	if (sock_puts(sock, buf) <0) goto bail;
-	if (sock_getln(sock, buf, sizeof buf) < 0) goto bail;
+	if (sock_puts(&sock, buf) <0) goto bail;
+	if (sock_getln(&sock, buf, sizeof buf) < 0) goto bail;
 	CtdlLogPrintf(CTDL_DEBUG, ">%s\n", buf);
 	if (strncasecmp(buf, "+OK", 3)) goto bail;
 
@@ -126,8 +130,8 @@ void pop3_do_fetching(char *roomname, char *pop3host, char *pop3user, char *pop3
 	/* Password */
 	snprintf(buf, sizeof buf, "PASS %s\r", pop3pass);
 	CtdlLogPrintf(CTDL_DEBUG, "<PASS <password>\n");
-	if (sock_puts(sock, buf) <0) goto bail;
-	if (sock_getln(sock, buf, sizeof buf) < 0) goto bail;
+	if (sock_puts(&sock, buf) <0) goto bail;
+	if (sock_getln(&sock, buf, sizeof buf) < 0) goto bail;
 	CtdlLogPrintf(CTDL_DEBUG, ">%s\n", buf);
 	if (strncasecmp(buf, "+OK", 3)) goto bail;
 
@@ -137,8 +141,8 @@ void pop3_do_fetching(char *roomname, char *pop3host, char *pop3user, char *pop3
 	/* Get the list of messages */
 	snprintf(buf, sizeof buf, "LIST\r");
 	CtdlLogPrintf(CTDL_DEBUG, "<%s\n", buf);
-	if (sock_puts(sock, buf) <0) goto bail;
-	if (sock_getln(sock, buf, sizeof buf) < 0) goto bail;
+	if (sock_puts(&sock, buf) <0) goto bail;
+	if (sock_getln(&sock, buf, sizeof buf) < 0) goto bail;
 	CtdlLogPrintf(CTDL_DEBUG, ">%s\n", buf);
 	if (strncasecmp(buf, "+OK", 3)) goto bail;
 
@@ -149,7 +153,7 @@ void pop3_do_fetching(char *roomname, char *pop3host, char *pop3user, char *pop3
 		if (CtdlThreadCheckStop())
 			goto bail;
 
-		if (sock_getln(sock, buf, sizeof buf) < 0) goto bail;
+		if (sock_getln(&sock, buf, sizeof buf) < 0) goto bail;
 		CtdlLogPrintf(CTDL_DEBUG, ">%s\n", buf);
 		msg_to_fetch = atoi(buf);
 		if (msg_to_fetch > 0) {
@@ -171,8 +175,8 @@ void pop3_do_fetching(char *roomname, char *pop3host, char *pop3user, char *pop3
 		/* Find out the UIDL of the message, to determine whether we've already downloaded it */
 		snprintf(buf, sizeof buf, "UIDL %d\r", msglist[i]);
 		CtdlLogPrintf(CTDL_DEBUG, "<%s\n", buf);
-		if (sock_puts(sock, buf) <0) goto bail;
-		if (sock_getln(sock, buf, sizeof buf) < 0) goto bail;
+		if (sock_puts(&sock, buf) <0) goto bail;
+		if (sock_getln(&sock, buf, sizeof buf) < 0) goto bail;
 		CtdlLogPrintf(CTDL_DEBUG, ">%s\n", buf);
 		if (strncasecmp(buf, "+OK", 3)) goto bail;
 		extract_token(this_uidl, buf, 2, ' ', sizeof this_uidl);
@@ -197,8 +201,8 @@ void pop3_do_fetching(char *roomname, char *pop3host, char *pop3user, char *pop3
 			/* Message has not been seen. Tell the server to fetch the message... */
 			snprintf(buf, sizeof buf, "RETR %d\r", msglist[i]);
 			CtdlLogPrintf(CTDL_DEBUG, "<%s\n", buf);
-			if (sock_puts(sock, buf) <0) goto bail;
-			if (sock_getln(sock, buf, sizeof buf) < 0) goto bail;
+			if (sock_puts(&sock, buf) <0) goto bail;
+			if (sock_getln(&sock, buf, sizeof buf) < 0) goto bail;
 			CtdlLogPrintf(CTDL_DEBUG, ">%s\n", buf);
 			if (strncasecmp(buf, "+OK", 3)) goto bail;
 	
@@ -206,7 +210,7 @@ void pop3_do_fetching(char *roomname, char *pop3host, char *pop3user, char *pop3
 				goto bail;
 
 			/* If we get to this point, the message is on its way.  Read it. */
-			body = CtdlReadMessageBody(".", config.c_maxmsglen, NULL, 1, sock);
+			body = CtdlReadMessageBody(HKEY("."), config.c_maxmsglen, NULL, 1, &sock);
 			if (body == NULL) goto bail;
 	
 			CtdlLogPrintf(CTDL_DEBUG, "Converting message...\n");
@@ -221,8 +225,8 @@ void pop3_do_fetching(char *roomname, char *pop3host, char *pop3user, char *pop3
 				if (!keep) {
 					snprintf(buf, sizeof buf, "DELE %d\r", msglist[i]);
 					CtdlLogPrintf(CTDL_DEBUG, "<%s\n", buf);
-					if (sock_puts(sock, buf) <0) goto bail;
-					if (sock_getln(sock, buf, sizeof buf) < 0) goto bail;
+					if (sock_puts(&sock, buf) <0) goto bail;
+					if (sock_getln(&sock, buf, sizeof buf) < 0) goto bail;
 					CtdlLogPrintf(CTDL_DEBUG, ">%s\n", buf); /* errors here are non-fatal */
 				}
 
@@ -239,10 +243,15 @@ void pop3_do_fetching(char *roomname, char *pop3host, char *pop3user, char *pop3
 	/* Log out */
 	snprintf(buf, sizeof buf, "QUIT\r");
 	CtdlLogPrintf(CTDL_DEBUG, "<%s\n", buf);
-	if (sock_puts(sock, buf) <0) goto bail;
-	if (sock_getln(sock, buf, sizeof buf) < 0) goto bail;
+	if (sock_puts(&sock, buf) <0) goto bail;
+	if (sock_getln(&sock, buf, sizeof buf) < 0) goto bail;
 	CtdlLogPrintf(CTDL_DEBUG, ">%s\n", buf);
-bail:	sock_close(sock);
+bail:	
+	FreeStrBuf(&CCC->sReadBuf);
+	FreeStrBuf(&CCC->sMigrateBuf);
+
+	if (sock != -1)
+		sock_close(sock);
 	if (msglist) free(msglist);
 }
 
@@ -299,7 +308,12 @@ void pop3client_scan(void) {
 	static int doing_pop3client = 0;
 	struct pop3aggr *pptr;
 	time_t fastest_scan;
-	
+	CitContext popclientCC;
+
+	/* Give this thread its own private CitContext */
+	CtdlFillSystemContext(&popclientCC, "popclient");
+	citthread_setspecific(MyConKey, (void *)&popclientCC );
+
 	if (config.c_pop3_fastest < config.c_pop3_fetch)
 		fastest_scan = config.c_pop3_fastest;
 	else
@@ -337,6 +351,7 @@ void pop3client_scan(void) {
 	CtdlLogPrintf(CTDL_DEBUG, "pop3client ended\n");
 	last_run = time(NULL);
 	doing_pop3client = 0;
+	CtdlClearSystemContext();
 }
 
 
@@ -348,5 +363,5 @@ CTDL_MODULE_INIT(pop3client)
 	}
 	
 	/* return our Subversion id for the Log */
-        return "$Id: serv_pop3client.c 7921 2009-10-17 12:21:17Z davew $";
+        return "$Id: serv_pop3client.c 8617 2010-06-09 22:37:37Z dothebart $";
 }
