@@ -1,5 +1,5 @@
 /*
- * $Id: serv_rssclient.c 8162 2009-12-29 13:33:11Z dothebart $
+ * $Id: serv_rssclient.c 8658 2010-06-28 18:59:15Z dothebart $
  *
  * Bring external RSS feeds into rooms.
  *
@@ -123,7 +123,7 @@ fill_encoding_info (const char *charset, XML_Encoding * info)
 { 
   iconv_t cd = (iconv_t)(-1); 
   int flag; 
-	CtdlLogPrintf(CTDL_DEBUG, "RSS: fill encoding info ...\n");
+	CtdlLogPrintf(0, "RSS: fill encoding info ...\n");
  
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN 
   cd = iconv_open ("UCS-2LE", charset); 
@@ -201,7 +201,7 @@ iconv_convertor (void *data, const char *s)
 { 
   XML_Encoding *info = data; 
   int res; 
-	CtdlLogPrintf(CTDL_DEBUG, "RSS: **********Converting ...\n");
+	CtdlLogPrintf(0, "RSS: Converting ...\n");
 
   if (s == NULL) 
     return -1; 
@@ -234,7 +234,7 @@ handle_unknown_xml_encoding (void *encodingHandleData,
 			     XML_Encoding * info) 
 { 
   int result; 
-  CtdlLogPrintf(CTDL_DEBUG, "RSS: unknown encoding ...\n");
+  CtdlLogPrintf(0, "RSS: unknown encoding ...\n");
   result = fill_encoding_info (name, info); 
   if (result >= 0) 
     { 
@@ -289,10 +289,10 @@ void rss_save_item(rsscollection *rssc) {
 	int msglen = 0;
 	rss_item *ri = rssc->Item;
 
-	CtdlLogPrintf(CTDL_DEBUG, "RSS: saving item...\n");
 	recp = (struct recptypes *) malloc(sizeof(struct recptypes));
 	if (recp == NULL) return;
 	memset(recp, 0, sizeof(struct recptypes));
+	memset(&ut, 0, sizeof(struct UseTable));
 	recp->recp_room = strdup(ri->roomlist);
 	recp->num_room = num_tokens(ri->roomlist, '|');
 	recp->recptypes_magic = RECPTYPES_MAGIC;
@@ -321,6 +321,7 @@ void rss_save_item(rsscollection *rssc) {
 	}
 
 	/* Find out if we've already seen this item */
+
 	cdbut = cdb_fetch(CDB_USETABLE, utmsgid, strlen(utmsgid));
 	if (cdbut != NULL) {
 		/* Item has already been seen */
@@ -334,7 +335,7 @@ void rss_save_item(rsscollection *rssc) {
 	}
 	else {
 		/* Item has not been seen, so save it. */
-
+		CtdlLogPrintf(CTDL_DEBUG, "RSS: saving item...\n");
 		if (ri->description == NULL) ri->description = strdup("");
 		for (i=strlen(ri->description); i>=0; --i) {
 			if (isspace(ri->description[i])) {
@@ -348,10 +349,36 @@ void rss_save_item(rsscollection *rssc) {
 		msg->cm_anon_type = MES_NORMAL;
 		msg->cm_format_type = FMT_RFC822;
 
+		if (ri->guid != NULL) {
+			msg->cm_fields['E'] = strdup(ri->guid);
+		}
+
 		if (ri->author_or_creator != NULL) {
-			msg->cm_fields['A'] = html_to_ascii(ri->author_or_creator,
-				strlen(ri->author_or_creator), 512, 0);
-			striplt(msg->cm_fields['A']);
+			char *From;
+			StrBuf *Encoded, *QPEncoded;
+			StrBuf *UserName;
+			StrBuf *EmailAddress;
+			StrBuf *EncBuf;
+			
+			UserName = NewStrBuf();
+			EmailAddress = NewStrBuf();
+			EncBuf = NewStrBuf();
+
+			From = html_to_ascii(ri->author_or_creator,
+					     strlen(ri->author_or_creator), 
+					     512, 0);
+
+			Encoded = NewStrBufPlain(From, -1);
+			free(From);
+			StrBufTrim(Encoded);
+			QPEncoded = StrBufSanitizeEmailRecipientVector(Encoded, UserName, EmailAddress, EncBuf);
+			msg->cm_fields['A'] = SmashStrBuf(&QPEncoded);
+
+			FreeStrBuf(&Encoded);
+			FreeStrBuf(&UserName);
+			FreeStrBuf(&EmailAddress);
+			FreeStrBuf(&EncBuf);
+
 		}
 		else {
 			msg->cm_fields['A'] = strdup("rss");
@@ -359,8 +386,21 @@ void rss_save_item(rsscollection *rssc) {
 
 		msg->cm_fields['N'] = strdup(NODENAME);
 		if (ri->title != NULL) {
-			msg->cm_fields['U'] = html_to_ascii(ri->title, strlen(ri->title), 512, 0);
-			striplt(msg->cm_fields['U']);
+			long len;
+			char *Sbj;
+			StrBuf *Encoded, *QPEncoded;
+
+			QPEncoded = NULL;
+			len = strlen(ri->title);
+			Sbj = html_to_ascii(ri->title, len, 512, 0);
+			Encoded = NewStrBufPlain(Sbj, -1);
+			free(Sbj);
+
+			StrBufTrim(Encoded);
+			StrBufRFC2047encode(&QPEncoded, Encoded);
+
+			msg->cm_fields['U'] = SmashStrBuf(&QPEncoded);
+			FreeStrBuf(&Encoded);
 		}
 		msg->cm_fields['T'] = malloc(64);
 		snprintf(msg->cm_fields['T'], 64, "%ld", ri->pubdate);
@@ -452,6 +492,8 @@ void flush_rss_ite(rss_item *ri)
 	ri->link = NULL;
 	if (ri->author_or_creator != NULL) free(ri->author_or_creator);
 	ri->author_or_creator = NULL;
+	if (ri->author_url != NULL) free(ri->author_url);
+	ri->author_url = NULL;
 	if (ri->description != NULL) free(ri->description);
 	ri->description = NULL;
 	/* Throw away any existing character data */
@@ -469,7 +511,7 @@ void rss_xml_start(void *data, const char *supplied_el, const char **attr) {
 	char *sep = NULL;
 
 	/* Axe the namespace, we don't care about it */
-///	CtdlLogPrintf(CTDL_DEBUG, "RSS: supplied el %d: %s...\n", rssc->Cfg->ItemType, supplied_el);
+///	CtdlLogPrintf(0, "RSS: supplied el %d: %s...\n", rssc->Cfg->ItemType, supplied_el);
 	safestrncpy(el, supplied_el, sizeof el);
 	while (sep = strchr(el, ':'), sep) {
 		strcpy(el, ++sep);
@@ -477,17 +519,17 @@ void rss_xml_start(void *data, const char *supplied_el, const char **attr) {
 
 	if ((rssc->Cfg->ItemType == RSS_UNSET) && !strcasecmp(el, "rss")) 
 	{
-		CtdlLogPrintf(CTDL_DEBUG, "RSS: This is an RSS feed.\n");
+		CtdlLogPrintf(9, "RSS: This is an RSS feed.\n");
 		rssc->Cfg->ItemType = RSS_RSS;
 	}
 	if ((rssc->Cfg->ItemType == RSS_UNSET) && !strcasecmp(el, "rdf")) 
 	{
-		CtdlLogPrintf(CTDL_DEBUG, "RSS: This is an RDF feed.\n");
+		CtdlLogPrintf(9, "RSS: This is an RDF feed.\n");
 		rssc->Cfg->ItemType = RSS_RSS;
 	}
 	else if ((rssc->Cfg->ItemType == RSS_UNSET) && !strcasecmp(el, "feed")) 
 	{
-		CtdlLogPrintf(CTDL_DEBUG, "RSS: This is an ATOM feed.\n");
+		CtdlLogPrintf(9, "RSS: This is an ATOM feed.\n");
 		rssc->Cfg->ItemType = RSS_ATOM;
 	}
 	else if ((rssc->Cfg->ItemType == RSS_RSS) &&
@@ -535,7 +577,7 @@ void rss_xml_end(void *data, const char *supplied_el) {
 	while (sep = strchr(el, ':'), sep) {
 		strcpy(el, ++sep);
 	}
-//	CtdlLogPrintf(CTDL_DEBUG, "RSS: END %s...\n", el);
+//	CtdlLogPrintf(0, "RSS: END %s...\n", el);
 
 	if ( (!strcasecmp(el, "title")) && (ri->item_tag_nesting == 0) && (ri->chardata != NULL) ) {
 		safestrncpy(ri->channel_title, ri->chardata, sizeof ri->channel_title);
@@ -868,8 +910,6 @@ void *rssclient_scan(void *args) {
 	CtdlFillSystemContext(&rssclientCC, "rssclient");
 	citthread_setspecific(MyConKey, (void *)&rssclientCC );
 
-	CtdlThreadAllocTSD();
-
 	/*
 	 * This is a simple concurrency check to make sure only one rssclient run
 	 * is done at a time.  We could do this with a mutex, but since we
@@ -897,6 +937,7 @@ void *rssclient_scan(void *args) {
 		CtdlThreadSchedule ("RSS Client", CTDLTHREAD_BIGSTACK, rssclient_scan, NULL, last_run + config.c_net_freq);
 	else
 		CtdlLogPrintf(CTDL_DEBUG, "rssclient: Task STOPPED.\n");
+	CtdlClearSystemContext();
 	return NULL;
 }
 
@@ -909,5 +950,5 @@ CTDL_MODULE_INIT(rssclient)
 		CtdlThreadSchedule ("RSS Client", CTDLTHREAD_BIGSTACK, rssclient_scan, NULL, 0);
 	}
 	/* return our Subversion id for the Log */
-        return "$Id: serv_rssclient.c 8162 2009-12-29 13:33:11Z dothebart $";
+        return "$Id: serv_rssclient.c 8658 2010-06-28 18:59:15Z dothebart $";
 }
