@@ -1,5 +1,5 @@
 /*
- * $Id: serv_rwho.c 7921 2009-10-17 12:21:17Z davew $
+ * $Id: serv_rwho.c 8525 2010-04-24 09:33:15Z dothebart $
  *
  * This module implements server commands related to the display and
  * manipulation of the "Who's online" list.
@@ -53,7 +53,6 @@
 #include "config.h"
 #include "control.h"
 #include "user_ops.h"
-#include "policy.h"
 #include "database.h"
 #include "msgbase.h"
 
@@ -90,7 +89,7 @@ void cmd_rwho(char *argbuf) {
 		return;
 	}
 	
-	aide = CC->user.axlevel >= 6;
+	aide = CC->user.axlevel >= AxAideU;
 	cprintf("%d%c \n", LISTING_FOLLOWS, CtdlCheckExpress() );
 	
 	for (i=0; i<nContexts; i++) 
@@ -294,5 +293,5 @@ CTDL_MODULE_INIT(rwho)
 	}
 	
 	/* return our Subversion id for the Log */
-        return "$Id: serv_rwho.c 7921 2009-10-17 12:21:17Z davew $";
+        return "$Id: serv_rwho.c 8525 2010-04-24 09:33:15Z dothebart $";
 }
