@@ -1,5 +1,5 @@
 /*
- * $Id: serv_smtp.c 8050 2009-11-14 16:17:06Z davew $
+ * $Id: serv_smtp.c 8686 2010-07-19 20:56:54Z dothebart $
  *
  * This module is an SMTP and ESMTP implementation for the Citadel system.
  * It is compliant with all of the following:
@@ -76,7 +76,6 @@
 #include "config.h"
 #include "control.h"
 #include "user_ops.h"
-#include "policy.h"
 #include "database.h"
 #include "msgbase.h"
 #include "internet_addressing.h"
@@ -97,7 +96,7 @@
 
 
 
-struct citsmtp {		/* Information about the current session */
+typedef struct _citsmtp {		/* Information about the current session */
 	int command_state;
 	char helo_node[SIZ];
 	char from[SIZ];
@@ -108,7 +107,7 @@ struct citsmtp {		/* Information about the current session */
 	int is_lmtp;
 	int is_unfiltered;
 	int is_msa;
-};
+}citsmtp;
 
 enum {				/* Command states for login authentication */
 	smtp_command,
@@ -117,7 +116,7 @@ enum {				/* Command states for login authentication */
 	smtp_plain
 };
 
-#define SMTP		((struct citsmtp *)CC->session_specific_data)
+#define SMTP		((citsmtp *)CC->session_specific_data)
 
 
 int run_queue_now = 0;	/* Set to 1 to ignore SMTP send retry times */
@@ -135,19 +134,21 @@ citthread_mutex_t smtp_send_lock;
  */
 void smtp_greeting(int is_msa)
 {
+	citsmtp *sSMTP;
 	char message_to_spammer[1024];
 
 	strcpy(CC->cs_clientname, "SMTP session");
 	CC->internal_pgm = 1;
 	CC->cs_flags |= CS_STEALTH;
-	CC->session_specific_data = malloc(sizeof(struct citsmtp));
-	memset(SMTP, 0, sizeof(struct citsmtp));
-	SMTP->is_msa = is_msa;
+	CC->session_specific_data = malloc(sizeof(citsmtp));
+	memset(SMTP, 0, sizeof(citsmtp));
+	sSMTP = SMTP;
+	sSMTP->is_msa = is_msa;
 
 	/* If this config option is set, reject connections from problem
 	 * addresses immediately instead of after they execute a RCPT
 	 */
-	if ( (config.c_rbl_at_greeting) && (SMTP->is_msa == 0) ) {
+	if ( (config.c_rbl_at_greeting) && (sSMTP->is_msa == 0) ) {
 		if (rbl_check(message_to_spammer)) {
 			if (CtdlThreadCheckStop())
 				cprintf("421 %s\r\n", message_to_spammer);
@@ -201,7 +202,10 @@ void smtp_msa_greeting(void) {
  * LMTP is like SMTP but with some extra bonus footage added.
  */
 void lmtp_greeting(void) {
+	citsmtp *sSMTP;
+
 	smtp_greeting(0);
+	sSMTP = SMTP;
 	SMTP->is_lmtp = 1;
 }
 
@@ -218,9 +222,12 @@ void smtp_mta_greeting(void) {
  * We also have an unfiltered LMTP socket that bypasses spam filters.
  */
 void lmtp_unfiltered_greeting(void) {
+	citsmtp *sSMTP;
+
 	smtp_greeting(0);
-	SMTP->is_lmtp = 1;
-	SMTP->is_unfiltered = 1;
+	sSMTP = SMTP;
+	sSMTP->is_lmtp = 1;
+	sSMTP->is_unfiltered = 1;
 }
 
 
@@ -241,22 +248,23 @@ void smtp_auth_greeting(void) {
  * which_command:  0=HELO, 1=EHLO, 2=LHLO
  */
 void smtp_hello(char *argbuf, int which_command) {
+	citsmtp *sSMTP = SMTP;
 
-	safestrncpy(SMTP->helo_node, argbuf, sizeof SMTP->helo_node);
+	safestrncpy(sSMTP->helo_node, argbuf, sizeof sSMTP->helo_node);
 
-	if ( (which_command != 2) && (SMTP->is_lmtp) ) {
+	if ( (which_command != 2) && (sSMTP->is_lmtp) ) {
 		cprintf("500 Only LHLO is allowed when running LMTP\r\n");
 		return;
 	}
 
-	if ( (which_command == 2) && (SMTP->is_lmtp == 0) ) {
+	if ( (which_command == 2) && (sSMTP->is_lmtp == 0) ) {
 		cprintf("500 LHLO is only allowed when running LMTP\r\n");
 		return;
 	}
 
 	if (which_command == 0) {
 		cprintf("250 Hello %s (%s [%s])\r\n",
-			SMTP->helo_node,
+			sSMTP->helo_node,
 			CC->cs_host,
 			CC->cs_addr
 		);
@@ -264,7 +272,7 @@ void smtp_hello(char *argbuf, int which_command) {
 	else {
 		if (which_command == 1) {
 			cprintf("250-Hello %s (%s [%s])\r\n",
-				SMTP->helo_node,
+				sSMTP->helo_node,
 				CC->cs_host,
 				CC->cs_addr
 			);
@@ -282,7 +290,7 @@ void smtp_hello(char *argbuf, int which_command) {
 		 * the SMTP-MSA port, not on the SMTP-MTA port, due to
 		 * questionable reliability of TLS in certain sending MTA's.
 		 */
-		if ( (!CC->redirect_ssl) && (SMTP->is_msa) ) {
+		if ( (!CC->redirect_ssl) && (sSMTP->is_msa) ) {
 			cprintf("250-STARTTLS\r\n");
 		}
 #endif	/* HAVE_OPENSSL */
@@ -310,17 +318,18 @@ void smtp_help(void) {
 void smtp_get_user(char *argbuf) {
 	char buf[SIZ];
 	char username[SIZ];
+	citsmtp *sSMTP = SMTP;
 
 	CtdlDecodeBase64(username, argbuf, SIZ);
 	/* CtdlLogPrintf(CTDL_DEBUG, "Trying <%s>\n", username); */
 	if (CtdlLoginExistingUser(NULL, username) == login_ok) {
 		CtdlEncodeBase64(buf, "Password:", 9, 0);
 		cprintf("334 %s\r\n", buf);
-		SMTP->command_state = smtp_password;
+		sSMTP->command_state = smtp_password;
 	}
 	else {
 		cprintf("500 No such user.\r\n");
-		SMTP->command_state = smtp_command;
+		sSMTP->command_state = smtp_command;
 	}
 }
 
@@ -330,11 +339,12 @@ void smtp_get_user(char *argbuf) {
  */
 void smtp_get_pass(char *argbuf) {
 	char password[SIZ];
+	long len;
 
 	memset(password, 0, sizeof(password));	
-	CtdlDecodeBase64(password, argbuf, SIZ);
+	len = CtdlDecodeBase64(password, argbuf, SIZ);
 	/* CtdlLogPrintf(CTDL_DEBUG, "Trying <%s>\n", password); */
-	if (CtdlTryPassword(password) == pass_ok) {
+	if (CtdlTryPassword(password, len) == pass_ok) {
 		smtp_auth_greeting();
 	}
 	else {
@@ -353,11 +363,14 @@ void smtp_try_plain(char *encoded_authstring) {
 	char user[256];
 	char pass[256];
 	int result;
+	long len;
 
 	CtdlDecodeBase64(decoded_authstring, encoded_authstring, strlen(encoded_authstring) );
 	safestrncpy(ident, decoded_authstring, sizeof ident);
 	safestrncpy(user, &decoded_authstring[strlen(ident) + 1], sizeof user);
-	safestrncpy(pass, &decoded_authstring[strlen(ident) + strlen(user) + 2], sizeof pass);
+	len = safestrncpy(pass, &decoded_authstring[strlen(ident) + strlen(user) + 2], sizeof pass);
+	if (len == -1)
+		len = sizeof(pass) - 1;
 
 	SMTP->command_state = smtp_command;
 
@@ -369,7 +382,7 @@ void smtp_try_plain(char *encoded_authstring) {
 	}
 
 	if (result == login_ok) {
-		if (CtdlTryPassword(pass) == pass_ok) {
+		if (CtdlTryPassword(pass, len) == pass_ok) {
 			smtp_auth_greeting();
 			return;
 		}
@@ -437,16 +450,17 @@ void smtp_auth(char *argbuf) {
 void smtp_rset(int do_response) {
 	int is_lmtp;
 	int is_unfiltered;
+	citsmtp *sSMTP = SMTP;
 
 	/*
 	 * Our entire SMTP state is discarded when a RSET command is issued,
 	 * but we need to preserve this one little piece of information, so
 	 * we save it for later.
 	 */
-	is_lmtp = SMTP->is_lmtp;
-	is_unfiltered = SMTP->is_unfiltered;
+	is_lmtp = sSMTP->is_lmtp;
+	is_unfiltered = sSMTP->is_unfiltered;
 
-	memset(SMTP, 0, sizeof(struct citsmtp));
+	memset(sSMTP, 0, sizeof(citsmtp));
 
 	/*
 	 * It is somewhat ambiguous whether we want to log out when a RSET
@@ -462,8 +476,8 @@ void smtp_rset(int do_response) {
 	/*
 	 * Reinstate this little piece of information we saved (see above).
 	 */
-	SMTP->is_lmtp = is_lmtp;
-	SMTP->is_unfiltered = is_unfiltered;
+	sSMTP->is_lmtp = is_lmtp;
+	sSMTP->is_unfiltered = is_unfiltered;
 
 	if (do_response) {
 		cprintf("250 Zap!\r\n");
@@ -475,19 +489,22 @@ void smtp_rset(int do_response) {
  * after the DATA command finishes.
  */
 void smtp_data_clear(void) {
-	strcpy(SMTP->from, "");
-	strcpy(SMTP->recipients, "");
-	SMTP->number_of_recipients = 0;
-	SMTP->delivery_mode = 0;
-	SMTP->message_originated_locally = 0;
+	citsmtp *sSMTP = SMTP;
+
+	strcpy(sSMTP->from, "");
+	strcpy(sSMTP->recipients, "");
+	sSMTP->number_of_recipients = 0;
+	sSMTP->delivery_mode = 0;
+	sSMTP->message_originated_locally = 0;
 }
 
 const char *smtp_get_Recipients(void)
 {
-	if (SMTP == NULL)
-		return NULL;
-	else return SMTP->from;
+	citsmtp *sSMTP = SMTP;
 
+	if (sSMTP == NULL)
+		return NULL;
+	else return sSMTP->from;
 }
 
 /*
@@ -497,8 +514,9 @@ void smtp_mail(char *argbuf) {
 	char user[SIZ];
 	char node[SIZ];
 	char name[SIZ];
+	citsmtp *sSMTP = SMTP;
 
-	if (!IsEmptyStr(SMTP->from)) {
+	if (!IsEmptyStr(sSMTP->from)) {
 		cprintf("503 Only one sender permitted\r\n");
 		return;
 	}
@@ -508,10 +526,10 @@ void smtp_mail(char *argbuf) {
 		return;
 	}
 
-	strcpy(SMTP->from, &argbuf[5]);
-	striplt(SMTP->from);
-	if (haschar(SMTP->from, '<') > 0) {
-		stripallbut(SMTP->from, '<', '>');
+	strcpy(sSMTP->from, &argbuf[5]);
+	striplt(sSMTP->from);
+	if (haschar(sSMTP->from, '<') > 0) {
+		stripallbut(sSMTP->from, '<', '>');
 	}
 
 	/* We used to reject empty sender names, until it was brought to our
@@ -520,21 +538,21 @@ void smtp_mail(char *argbuf) {
 	 * address so we don't have to contend with the empty string causing
 	 * other code to fail when it's expecting something there.
 	 */
-	if (IsEmptyStr(SMTP->from)) {
-		strcpy(SMTP->from, "someone@example.com");
+	if (IsEmptyStr(sSMTP->from)) {
+		strcpy(sSMTP->from, "someone@example.com");
 	}
 
 	/* If this SMTP connection is from a logged-in user, force the 'from'
 	 * to be the user's Internet e-mail address as Citadel knows it.
 	 */
 	if (CC->logged_in) {
-		safestrncpy(SMTP->from, CC->cs_inet_email, sizeof SMTP->from);
-		cprintf("250 Sender ok <%s>\r\n", SMTP->from);
-		SMTP->message_originated_locally = 1;
+		safestrncpy(sSMTP->from, CC->cs_inet_email, sizeof sSMTP->from);
+		cprintf("250 Sender ok <%s>\r\n", sSMTP->from);
+		sSMTP->message_originated_locally = 1;
 		return;
 	}
 
-	else if (SMTP->is_lmtp) {
+	else if (sSMTP->is_lmtp) {
 		/* Bypass forgery checking for LMTP */
 	}
 
@@ -542,10 +560,10 @@ void smtp_mail(char *argbuf) {
 	 * this system (unless, of course, c_allow_spoofing is enabled)
 	 */
 	else if (config.c_allow_spoofing == 0) {
-		process_rfc822_addr(SMTP->from, user, node, name);
+		process_rfc822_addr(sSMTP->from, user, node, name);
 		if (CtdlHostAlias(node) != hostalias_nomatch) {
 			cprintf("550 You must log in to send mail from %s\r\n", node);
-			strcpy(SMTP->from, "");
+			strcpy(sSMTP->from, "");
 			return;
 		}
 	}
@@ -562,8 +580,9 @@ void smtp_rcpt(char *argbuf) {
 	char recp[1024];
 	char message_to_spammer[SIZ];
 	struct recptypes *valid = NULL;
+	citsmtp *sSMTP = SMTP;
 
-	if (IsEmptyStr(SMTP->from)) {
+	if (IsEmptyStr(sSMTP->from)) {
 		cprintf("503 Need MAIL before RCPT\r\n");
 		return;
 	}
@@ -573,9 +592,9 @@ void smtp_rcpt(char *argbuf) {
 		return;
 	}
 
-	if ( (SMTP->is_msa) && (!CC->logged_in) ) {
+	if ( (sSMTP->is_msa) && (!CC->logged_in) ) {
 		cprintf("550 You must log in to send mail on this port.\r\n");
-		strcpy(SMTP->from, "");
+		strcpy(sSMTP->from, "");
 		return;
 	}
 
@@ -583,14 +602,14 @@ void smtp_rcpt(char *argbuf) {
 	striplt(recp);
 	stripallbut(recp, '<', '>');
 
-	if ( (strlen(recp) + strlen(SMTP->recipients) + 1 ) >= SIZ) {
+	if ( (strlen(recp) + strlen(sSMTP->recipients) + 1 ) >= SIZ) {
 		cprintf("452 Too many recipients\r\n");
 		return;
 	}
 
 	/* RBL check */
 	if ( (!CC->logged_in)	/* Don't RBL authenticated users */
-	   && (!SMTP->is_lmtp) ) {	/* Don't RBL LMTP clients */
+	   && (!sSMTP->is_lmtp) ) {	/* Don't RBL LMTP clients */
 		if (config.c_rbl_at_greeting == 0) {	/* Don't RBL again if we already did it */
 			if (rbl_check(message_to_spammer)) {
 				if (CtdlThreadCheckStop())
@@ -605,7 +624,7 @@ void smtp_rcpt(char *argbuf) {
 
 	valid = validate_recipients(recp, 
 				    smtp_get_Recipients (),
-				    (SMTP->is_lmtp)? POST_LMTP:
+				    (sSMTP->is_lmtp)? POST_LMTP:
 				       (CC->logged_in)? POST_LOGGED_IN:
 				                        POST_EXTERNAL);
 	if (valid->num_error != 0) {
@@ -625,8 +644,8 @@ void smtp_rcpt(char *argbuf) {
 	}
 
 	if (valid->num_internet > 0) {
-		if ( (SMTP->message_originated_locally == 0)
-		   && (SMTP->is_lmtp == 0) ) {
+		if ( (sSMTP->message_originated_locally == 0)
+		   && (sSMTP->is_lmtp == 0) ) {
 			cprintf("551 <%s> - relaying denied\r\n", recp);
 			free_recipients(valid);
 			return;
@@ -634,11 +653,11 @@ void smtp_rcpt(char *argbuf) {
 	}
 
 	cprintf("250 RCPT ok <%s>\r\n", recp);
-	if (!IsEmptyStr(SMTP->recipients)) {
-		strcat(SMTP->recipients, ",");
+	if (!IsEmptyStr(sSMTP->recipients)) {
+		strcat(sSMTP->recipients, ",");
 	}
-	strcat(SMTP->recipients, recp);
-	SMTP->number_of_recipients += 1;
+	strcat(sSMTP->recipients, recp);
+	sSMTP->number_of_recipients += 1;
 	if (valid != NULL)  {
 		free_recipients(valid);
 	}
@@ -651,7 +670,8 @@ void smtp_rcpt(char *argbuf) {
  * Implements the DATA command
  */
 void smtp_data(void) {
-	char *body;
+	StrBuf *body;
+	char *defbody; //TODO: remove me
 	struct CtdlMessage *msg = NULL;
 	long msgnum = (-1L);
 	char nowstamp[SIZ];
@@ -659,13 +679,14 @@ void smtp_data(void) {
 	int scan_errors;
 	int i;
 	char result[SIZ];
+	citsmtp *sSMTP = SMTP;
 
-	if (IsEmptyStr(SMTP->from)) {
+	if (IsEmptyStr(sSMTP->from)) {
 		cprintf("503 Need MAIL command first.\r\n");
 		return;
 	}
 
-	if (SMTP->number_of_recipients < 1) {
+	if (sSMTP->number_of_recipients < 1) {
 		cprintf("503 Need RCPT command first.\r\n");
 		return;
 	}
@@ -673,37 +694,37 @@ void smtp_data(void) {
 	cprintf("354 Transmit message now - terminate with '.' by itself\r\n");
 	
 	datestring(nowstamp, sizeof nowstamp, time(NULL), DATESTRING_RFC822);
-	body = malloc(4096);
+	defbody = malloc(4096);
 
-	if (body != NULL) {
-		if (SMTP->is_lmtp && (CC->cs_UDSclientUID != -1)) {
-			snprintf(body, 4096,
-				 "Received: from %s (Citadel from userid %ld)\n"
-				 "	by %s; %s\n",
-				 SMTP->helo_node,
-				 (long int) CC->cs_UDSclientUID,
-				 config.c_fqdn,
-				 nowstamp);
+	if (defbody != NULL) {
+		if (sSMTP->is_lmtp && (CC->cs_UDSclientUID != -1)) {
+			snprintf(defbody, 4096,
+			       "Received: from %s (Citadel from userid %ld)\n"
+			       "	by %s; %s\n",
+			       sSMTP->helo_node,
+			       (long int) CC->cs_UDSclientUID,
+			       config.c_fqdn,
+			       nowstamp);
 		}
 		else {
-			snprintf(body, 4096,
+			snprintf(defbody, 4096,
 				 "Received: from %s (%s [%s])\n"
 				 "	by %s; %s\n",
-				 SMTP->helo_node,
+				 sSMTP->helo_node,
 				 CC->cs_host,
 				 CC->cs_addr,
 				 config.c_fqdn,
 				 nowstamp);
 		}
 	}
-	body = CtdlReadMessageBody(".", config.c_maxmsglen, body, 1, 0);
+	body = CtdlReadMessageBodyBuf(HKEY("."), config.c_maxmsglen, defbody, 1, NULL);
 	if (body == NULL) {
 		cprintf("550 Unable to save message: internal error.\r\n");
 		return;
 	}
 
 	CtdlLogPrintf(CTDL_DEBUG, "Converting message...\n");
-	msg = convert_internet_message(body);
+	msg = convert_internet_message_buf(&body);
 
 	/* If the user is locally authenticated, FORCE the From: header to
 	 * show up as the real sender.  Yes, this violates the RFC standard,
@@ -732,18 +753,18 @@ void smtp_data(void) {
 	if (msg->cm_fields['P'] != NULL) {
 		free(msg->cm_fields['P']);
 	}
-	msg->cm_fields['P'] = strdup(SMTP->from);
+	msg->cm_fields['P'] = strdup(sSMTP->from);
 
 	/* Set the "envelope to" address */
 	if (msg->cm_fields['V'] != NULL) {
 		free(msg->cm_fields['V']);
 	}
-	msg->cm_fields['V'] = strdup(SMTP->recipients);
+	msg->cm_fields['V'] = strdup(sSMTP->recipients);
 
 	/* Submit the message into the Citadel system. */
-	valid = validate_recipients(SMTP->recipients, 
+	valid = validate_recipients(sSMTP->recipients, 
 				    smtp_get_Recipients (),
-				    (SMTP->is_lmtp)? POST_LMTP:
+				    (sSMTP->is_lmtp)? POST_LMTP:
 				       (CC->logged_in)? POST_LOGGED_IN:
 				                        POST_EXTERNAL);
 
@@ -751,7 +772,7 @@ void smtp_data(void) {
 	 * submission (such as virus checkers or spam filters), call them now
 	 * and give them an opportunity to reject the message.
 	 */
-	if (SMTP->is_unfiltered) {
+	if (sSMTP->is_unfiltered) {
 		scan_errors = 0;
 	}
 	else {
@@ -783,8 +804,8 @@ void smtp_data(void) {
 	 * have different results, we can get away with just spitting out the
 	 * same message once for each recipient.
 	 */
-	if (SMTP->is_lmtp) {
-		for (i=0; i<SMTP->number_of_recipients; ++i) {
+	if (sSMTP->is_lmtp) {
+		for (i=0; i<sSMTP->number_of_recipients; ++i) {
 			cprintf("%s", result);
 		}
 	}
@@ -799,8 +820,8 @@ void smtp_data(void) {
 		syslog((LOG_MAIL | LOG_INFO),
 			"%ld: from=<%s>, nrcpts=%d, relay=%s [%s], stat=%s",
 			msgnum,
-			SMTP->from,
-			SMTP->number_of_recipients,
+			sSMTP->from,
+			sSMTP->number_of_recipients,
 			CC->cs_host,
 			CC->cs_addr,
 			result
@@ -840,6 +861,11 @@ void smtp_starttls(void)
  */
 void smtp_command_loop(void) {
 	char cmdbuf[SIZ];
+	citsmtp *sSMTP = SMTP;
+
+	if (sSMTP == NULL) {
+		CtdlLogPrintf(CTDL_EMERG, "Session SMTP data is null.  WTF?  We will crash now.\n");
+	}
 
 	time(&CC->lastcmd);
 	memset(cmdbuf, 0, sizeof cmdbuf); /* Clear it, just in case */
@@ -851,15 +877,15 @@ void smtp_command_loop(void) {
 	CtdlLogPrintf(CTDL_INFO, "SMTP server: %s\n", cmdbuf);
 	while (strlen(cmdbuf) < 5) strcat(cmdbuf, " ");
 
-	if (SMTP->command_state == smtp_user) {
+	if (sSMTP->command_state == smtp_user) {
 		smtp_get_user(cmdbuf);
 	}
 
-	else if (SMTP->command_state == smtp_password) {
+	else if (sSMTP->command_state == smtp_password) {
 		smtp_get_pass(cmdbuf);
 	}
 
-	else if (SMTP->command_state == smtp_plain) {
+	else if (sSMTP->command_state == smtp_plain) {
 		smtp_try_plain(cmdbuf);
 	}
 
@@ -952,9 +978,10 @@ void smtp_try(const char *key, const char *addr, int *status,
 	char mx_port[256];
 	int lp, rp;
 	char *msgtext;
-	char *ptr;
+	const char *ptr;
 	size_t msg_size;
 	int scan_done;
+	CitContext *CCC=CC;
 	
 	
 	/* Parse out the host portion of the recipient address */
@@ -964,15 +991,10 @@ void smtp_try(const char *key, const char *addr, int *status,
 		user, node, name);
 
 	/* Load the message out of the database */
-	CC->redirect_buffer = malloc(SIZ);
-	CC->redirect_len = 0;
-	CC->redirect_alloc = SIZ;
+	CCC->redirect_buffer = NewStrBufPlain(NULL, SIZ);
 	CtdlOutputMsg(msgnum, MT_RFC822, HEADERS_ALL, 0, 1, NULL, ESC_DOT);
-	msgtext = CC->redirect_buffer;
-	msg_size = CC->redirect_len;
-	CC->redirect_buffer = NULL;
-	CC->redirect_len = 0;
-	CC->redirect_alloc = 0;
+	msg_size = StrLength(CC->redirect_buffer);
+	msgtext = SmashStrBuf(&CC->redirect_buffer);
 
 	/* If no envelope_from is supplied, extract one from the message */
 	if ( (envelope_from == NULL) || (IsEmptyStr(envelope_from)) ) {
@@ -980,7 +1002,7 @@ void smtp_try(const char *key, const char *addr, int *status,
 		scan_done = 0;
 		ptr = msgtext;
 		do {
-			if (ptr = memreadline(ptr, buf, sizeof buf), *ptr == 0) {
+			if (ptr = cmemreadline(ptr, buf, sizeof buf), *ptr == 0) {
 				scan_done = 1;
 			}
 			if (!strncasecmp(buf, "From:", 5)) {
@@ -1079,8 +1101,12 @@ void smtp_try(const char *key, const char *addr, int *status,
 		return;
 	}
 
+	CCC->sReadBuf = NewStrBuf();
+	CCC->sMigrateBuf = NewStrBuf();
+	CCC->sPos = NULL;
+
 	/* Process the SMTP greeting from the server */
-	if (ml_sock_gets(sock, buf) < 0) {
+	if (ml_sock_gets(&sock, buf) < 0) {
 		*status = 4;
 		strcpy(dsn, "Connection broken during SMTP conversation");
 		goto bail;
@@ -1104,8 +1130,8 @@ void smtp_try(const char *key, const char *addr, int *status,
 	/* Do a EHLO command.  If it fails, try the HELO command. */
 	snprintf(buf, sizeof buf, "EHLO %s\r\n", config.c_fqdn);
 	CtdlLogPrintf(CTDL_DEBUG, ">%s", buf);
-	sock_write(sock, buf, strlen(buf));
-	if (ml_sock_gets(sock, buf) < 0) {
+	sock_write(&sock, buf, strlen(buf));
+	if (ml_sock_gets(&sock, buf) < 0) {
 		*status = 4;
 		strcpy(dsn, "Connection broken during SMTP HELO");
 		goto bail;
@@ -1114,8 +1140,8 @@ void smtp_try(const char *key, const char *addr, int *status,
 	if (buf[0] != '2') {
 		snprintf(buf, sizeof buf, "HELO %s\r\n", config.c_fqdn);
 		CtdlLogPrintf(CTDL_DEBUG, ">%s", buf);
-		sock_write(sock, buf, strlen(buf));
-		if (ml_sock_gets(sock, buf) < 0) {
+		sock_write(&sock, buf, strlen(buf));
+		if (ml_sock_gets(&sock, buf) < 0) {
 			*status = 4;
 			strcpy(dsn, "Connection broken during SMTP HELO");
 			goto bail;
@@ -1141,8 +1167,8 @@ void smtp_try(const char *key, const char *addr, int *status,
 		CtdlEncodeBase64(encoded, buf, strlen(mx_user) + strlen(mx_user) + strlen(mx_pass) + 2, 0);
 		snprintf(buf, sizeof buf, "AUTH PLAIN %s\r\n", encoded);
 		CtdlLogPrintf(CTDL_DEBUG, ">%s", buf);
-		sock_write(sock, buf, strlen(buf));
-		if (ml_sock_gets(sock, buf) < 0) {
+		sock_write(&sock, buf, strlen(buf));
+		if (ml_sock_gets(&sock, buf) < 0) {
 			*status = 4;
 			strcpy(dsn, "Connection broken during SMTP AUTH");
 			goto bail;
@@ -1165,8 +1191,8 @@ void smtp_try(const char *key, const char *addr, int *status,
 	/* previous command succeeded, now try the MAIL FROM: command */
 	snprintf(buf, sizeof buf, "MAIL FROM:<%s>\r\n", envelope_from);
 	CtdlLogPrintf(CTDL_DEBUG, ">%s", buf);
-	sock_write(sock, buf, strlen(buf));
-	if (ml_sock_gets(sock, buf) < 0) {
+	sock_write(&sock, buf, strlen(buf));
+	if (ml_sock_gets(&sock, buf) < 0) {
 		*status = 4;
 		strcpy(dsn, "Connection broken during SMTP MAIL");
 		goto bail;
@@ -1188,8 +1214,8 @@ void smtp_try(const char *key, const char *addr, int *status,
 	/* MAIL succeeded, now try the RCPT To: command */
 	snprintf(buf, sizeof buf, "RCPT TO:<%s@%s>\r\n", user, node);
 	CtdlLogPrintf(CTDL_DEBUG, ">%s", buf);
-	sock_write(sock, buf, strlen(buf));
-	if (ml_sock_gets(sock, buf) < 0) {
+	sock_write(&sock, buf, strlen(buf));
+	if (ml_sock_gets(&sock, buf) < 0) {
 		*status = 4;
 		strcpy(dsn, "Connection broken during SMTP RCPT");
 		goto bail;
@@ -1210,8 +1236,8 @@ void smtp_try(const char *key, const char *addr, int *status,
 
 	/* RCPT succeeded, now try the DATA command */
 	CtdlLogPrintf(CTDL_DEBUG, ">DATA\n");
-	sock_write(sock, "DATA\r\n", 6);
-	if (ml_sock_gets(sock, buf) < 0) {
+	sock_write(&sock, "DATA\r\n", 6);
+	if (ml_sock_gets(&sock, buf) < 0) {
 		*status = 4;
 		strcpy(dsn, "Connection broken during SMTP DATA");
 		goto bail;
@@ -1231,15 +1257,16 @@ void smtp_try(const char *key, const char *addr, int *status,
 	}
 
 	/* If we reach this point, the server is expecting data.*/
-	sock_write(sock, msgtext, msg_size);
+	sock_write(&sock, msgtext, msg_size);
 	if (msgtext[msg_size-1] != 10) {
 		CtdlLogPrintf(CTDL_WARNING, "Possible problem: message did not "
 			"correctly terminate. (expecting 0x10, got 0x%02x)\n",
 				buf[msg_size-1]);
+		sock_write(&sock, "\r\n", 2);
 	}
 
-	sock_write(sock, ".\r\n", 3);
-	if (ml_sock_gets(sock, buf) < 0) {
+	sock_write(&sock, ".\r\n", 3);
+	if (ml_sock_gets(&sock, buf) < 0) {
 		*status = 4;
 		strcpy(dsn, "Connection broken during SMTP message transmit");
 		goto bail;
@@ -1263,14 +1290,17 @@ void smtp_try(const char *key, const char *addr, int *status,
 	*status = 2;
 
 	CtdlLogPrintf(CTDL_DEBUG, ">QUIT\n");
-	sock_write(sock, "QUIT\r\n", 6);
-	ml_sock_gets(sock, buf);
+	sock_write(&sock, "QUIT\r\n", 6);
+	ml_sock_gets(&sock, buf);
 	CtdlLogPrintf(CTDL_DEBUG, "<%s\n", buf);
 	CtdlLogPrintf(CTDL_INFO, "SMTP client: delivery to <%s> @ <%s> (%s) succeeded\n",
 		user, node, name);
 
 bail:	free(msgtext);
-	sock_close(sock);
+	FreeStrBuf(&CCC->sReadBuf);
+	FreeStrBuf(&CCC->sMigrateBuf);
+	if (sock != -1)
+		sock_close(sock);
 
 	/* Write something to the syslog (which may or may not be where the
 	 * rest of the Citadel logs are going; some sysadmins want LOG_MAIL).
@@ -1304,7 +1334,7 @@ void smtp_do_bounce(char *instr) {
 	char addr[1024];
 	char dsn[1024];
 	char bounceto[1024];
-	char boundary[64];
+	StrBuf *boundary;
 	int num_bounces = 0;
 	int bounce_this = 0;
 	long bounce_msgid = (-1);
@@ -1314,13 +1344,13 @@ void smtp_do_bounce(char *instr) {
 	struct recptypes *valid;
 	int successful_bounce = 0;
 	static int seq = 0;
-	char *omsgtext;
-	size_t omsgsize;
+	StrBuf *BounceMB;
 	long omsgid = (-1);
 
 	CtdlLogPrintf(CTDL_DEBUG, "smtp_do_bounce() called\n");
 	strcpy(bounceto, "");
-	sprintf(boundary, "=_Citadel_Multipart_%s_%04x%04x", config.c_fqdn, getpid(), ++seq);
+	boundary = NewStrBufPlain(HKEY("=_Citadel_Multipart_"));
+	StrBufAppendPrintf(boundary, "%s_%04x%04x", config.c_fqdn, getpid(), ++seq);
 	lines = num_tokens(instr, '\n');
 
 	/* See if it's time to give up on delivery of this message */
@@ -1342,6 +1372,7 @@ void smtp_do_bounce(char *instr) {
 	bmsg = (struct CtdlMessage *) malloc(sizeof(struct CtdlMessage));
 	if (bmsg == NULL) return;
 	memset(bmsg, 0, sizeof(struct CtdlMessage));
+	BounceMB = NewStrBufPlain(NULL, 1024);
 
         bmsg->cm_magic = CTDLMESSAGE_MAGIC;
         bmsg->cm_anon_type = MES_NORMAL;
@@ -1350,39 +1381,39 @@ void smtp_do_bounce(char *instr) {
         bmsg->cm_fields['O'] = strdup(MAILROOM);
         bmsg->cm_fields['N'] = strdup(config.c_nodename);
         bmsg->cm_fields['U'] = strdup("Delivery Status Notification (Failure)");
-	bmsg->cm_fields['M'] = malloc(1024);
+	StrBufAppendBufPlain(BounceMB, HKEY("Content-type: multipart/mixed; boundary=\""), 0);
+	StrBufAppendBuf(BounceMB, boundary, 0);
+        StrBufAppendBufPlain(BounceMB, HKEY("\"\r\n"), 0);
+	StrBufAppendBufPlain(BounceMB, HKEY("MIME-Version: 1.0\r\n"), 0);
+	StrBufAppendBufPlain(BounceMB, HKEY("X-Mailer: " CITADEL "\r\n"), 0);
+        StrBufAppendBufPlain(BounceMB, HKEY("\r\nThis is a multipart message in MIME format.\r\n\r\n"), 0);
+        StrBufAppendBufPlain(BounceMB, HKEY("--"), 0);
+        StrBufAppendBuf(BounceMB, boundary, 0);
+	StrBufAppendBufPlain(BounceMB, HKEY("\r\n"), 0);
+        StrBufAppendBufPlain(BounceMB, HKEY("Content-type: text/plain\r\n\r\n"), 0);
 
-        strcpy(bmsg->cm_fields['M'], "Content-type: multipart/mixed; boundary=\"");
-        strcat(bmsg->cm_fields['M'], boundary);
-        strcat(bmsg->cm_fields['M'], "\"\r\n");
-        strcat(bmsg->cm_fields['M'], "MIME-Version: 1.0\r\n");
-        strcat(bmsg->cm_fields['M'], "X-Mailer: " CITADEL "\r\n");
-        strcat(bmsg->cm_fields['M'], "\r\nThis is a multipart message in MIME format.\r\n\r\n");
-        strcat(bmsg->cm_fields['M'], "--");
-        strcat(bmsg->cm_fields['M'], boundary);
-        strcat(bmsg->cm_fields['M'], "\r\n");
-        strcat(bmsg->cm_fields['M'], "Content-type: text/plain\r\n\r\n");
-
-	if (give_up) strcat(bmsg->cm_fields['M'],
+	if (give_up) StrBufAppendBufPlain(BounceMB, HKEY(
 "A message you sent could not be delivered to some or all of its recipients\n"
 "due to prolonged unavailability of its destination(s).\n"
 "Giving up on the following addresses:\n\n"
-);
+						  ), 0);
 
-        else strcat(bmsg->cm_fields['M'],
+        else StrBufAppendBufPlain(BounceMB, HKEY(
 "A message you sent could not be delivered to some or all of its recipients.\n"
 "The following addresses were undeliverable:\n\n"
-);
+					  ), 0);
 
 	/*
 	 * Now go through the instructions checking for stuff.
 	 */
 	for (i=0; i<lines; ++i) {
+		long addrlen;
+		long dsnlen;
 		extract_token(buf, instr, i, '\n', sizeof buf);
 		extract_token(key, buf, 0, '|', sizeof key);
-		extract_token(addr, buf, 1, '|', sizeof addr);
+		addrlen = extract_token(addr, buf, 1, '|', sizeof addr);
 		status = extract_int(buf, 2);
-		extract_token(dsn, buf, 3, '|', sizeof dsn);
+		dsnlen = extract_token(dsn, buf, 3, '|', sizeof dsn);
 		bounce_this = 0;
 
 		CtdlLogPrintf(CTDL_DEBUG, "key=<%s> addr=<%s> status=%d dsn=<%s>\n",
@@ -1404,17 +1435,10 @@ void smtp_do_bounce(char *instr) {
 		if (bounce_this) {
 			++num_bounces;
 
-			if (bmsg->cm_fields['M'] == NULL) {
-				CtdlLogPrintf(CTDL_ERR, "ERROR ... M field is null "
-					"(%s:%d)\n", __FILE__, __LINE__);
-			}
-
-			bmsg->cm_fields['M'] = realloc(bmsg->cm_fields['M'],
-				strlen(bmsg->cm_fields['M']) + 1024 );
-			strcat(bmsg->cm_fields['M'], addr);
-			strcat(bmsg->cm_fields['M'], ": ");
-			strcat(bmsg->cm_fields['M'], dsn);
-			strcat(bmsg->cm_fields['M'], "\r\n");
+			StrBufAppendBufPlain(BounceMB, addr, addrlen, 0);
+			StrBufAppendBufPlain(BounceMB, HKEY(": "), 0);
+			StrBufAppendBufPlain(BounceMB, dsn, dsnlen, 0);
+			StrBufAppendBufPlain(BounceMB, HKEY("\r\n"), 0);
 
 			remove_token(instr, i, '\n');
 			--i;
@@ -1424,34 +1448,25 @@ void smtp_do_bounce(char *instr) {
 
 	/* Attach the original message */
 	if (omsgid >= 0) {
-        	strcat(bmsg->cm_fields['M'], "--");
-        	strcat(bmsg->cm_fields['M'], boundary);
-        	strcat(bmsg->cm_fields['M'], "\r\n");
-        	strcat(bmsg->cm_fields['M'], "Content-type: message/rfc822\r\n");
-        	strcat(bmsg->cm_fields['M'], "Content-Transfer-Encoding: 7bit\r\n");
-        	strcat(bmsg->cm_fields['M'], "Content-Disposition: inline\r\n");
-        	strcat(bmsg->cm_fields['M'], "\r\n");
+        	StrBufAppendBufPlain(BounceMB, HKEY("--"), 0);
+		StrBufAppendBuf(BounceMB, boundary, 0);
+        	StrBufAppendBufPlain(BounceMB, HKEY("\r\n"), 0);
+		StrBufAppendBufPlain(BounceMB, HKEY("Content-type: message/rfc822\r\n"), 0);
+        	StrBufAppendBufPlain(BounceMB, HKEY("Content-Transfer-Encoding: 7bit\r\n"), 0);
+        	StrBufAppendBufPlain(BounceMB, HKEY("Content-Disposition: inline\r\n"), 0);
+        	StrBufAppendBufPlain(BounceMB, HKEY("\r\n"), 0);
 	
-		CC->redirect_buffer = malloc(SIZ);
-		CC->redirect_len = 0;
-		CC->redirect_alloc = SIZ;
+		CC->redirect_buffer = NewStrBufPlain(NULL, SIZ);
 		CtdlOutputMsg(omsgid, MT_RFC822, HEADERS_ALL, 0, 1, NULL, 0);
-		omsgtext = CC->redirect_buffer;
-		omsgsize = CC->redirect_len;
-		CC->redirect_buffer = NULL;
-		CC->redirect_len = 0;
-		CC->redirect_alloc = 0;
-		bmsg->cm_fields['M'] = realloc(bmsg->cm_fields['M'],
-				(strlen(bmsg->cm_fields['M']) + omsgsize + 1024) );
-		strcat(bmsg->cm_fields['M'], omsgtext);
-		free(omsgtext);
+		StrBufAppendBuf(BounceMB, CC->redirect_buffer, 0);
+		FreeStrBuf(&CC->redirect_buffer);
 	}
 
 	/* Close the multipart MIME scope */
-        strcat(bmsg->cm_fields['M'], "--");
-        strcat(bmsg->cm_fields['M'], boundary);
-        strcat(bmsg->cm_fields['M'], "--\r\n");
-
+        StrBufAppendBufPlain(BounceMB, HKEY("--"), 0);
+	StrBufAppendBuf(BounceMB, boundary, 0);
+	StrBufAppendBufPlain(BounceMB, HKEY("--\r\n"), 0);
+	bmsg->cm_fields['A'] = SmashStrBuf(&BounceMB);
 	/* Deliver the bounce if there's anything worth mentioning */
 	CtdlLogPrintf(CTDL_DEBUG, "num_bounces = %d\n", num_bounces);
 	if (num_bounces > 0) {
@@ -1482,7 +1497,7 @@ void smtp_do_bounce(char *instr) {
 			free_recipients(valid);
 		}
 	}
-
+	FreeStrBuf(&boundary);
 	CtdlFreeMessage(bmsg);
 	CtdlLogPrintf(CTDL_DEBUG, "Done processing bounces\n");
 }
@@ -1727,10 +1742,9 @@ void *smtp_do_queue(void *arg) {
 	int num_processed = 0;
 	struct CitContext smtp_queue_CC;
 
-	CtdlLogPrintf(CTDL_INFO, "SMTP client: processing outbound queue\n");
-
 	CtdlFillSystemContext(&smtp_queue_CC, "SMTP Send");
 	citthread_setspecific(MyConKey, (void *)&smtp_queue_CC );
+	CtdlLogPrintf(CTDL_INFO, "SMTP client: processing outbound queue\n");
 
 	if (CtdlGetRoom(&CC->room, SMTP_SPOOLOUT_ROOM) != 0) {
 		CtdlLogPrintf(CTDL_ERR, "Cannot find room <%s>\n", SMTP_SPOOLOUT_ROOM);
@@ -1741,6 +1755,8 @@ void *smtp_do_queue(void *arg) {
 
 	citthread_mutex_unlock (&smtp_send_lock);
 	CtdlLogPrintf(CTDL_INFO, "SMTP client: queue run completed; %d messages processed\n", num_processed);
+
+	CtdlClearSystemContext();
 	return(NULL);
 }
 
@@ -1918,5 +1934,5 @@ CTDL_MODULE_INIT(smtp)
 	}
 	
 	/* return our Subversion id for the Log */
-	return "$Id: serv_smtp.c 8050 2009-11-14 16:17:06Z davew $";
+	return "$Id: serv_smtp.c 8686 2010-07-19 20:56:54Z dothebart $";
 }
