@@ -1,5 +1,5 @@
 /*
- * $Id: serv_spam.c 7921 2009-10-17 12:21:17Z davew $
+ * $Id: serv_spam.c 8525 2010-04-24 09:33:15Z dothebart $
  *
  * This module allows Citadel to use SpamAssassin to filter incoming messages
  * arriving via SMTP.  For more information on SpamAssassin, visit
@@ -58,7 +58,6 @@
 #include "config.h"
 #include "control.h"
 #include "user_ops.h"
-#include "policy.h"
 #include "database.h"
 #include "msgbase.h"
 #include "internet_addressing.h"
@@ -80,8 +79,8 @@ int spam_assassin(struct CtdlMessage *msg) {
 	char buf[SIZ];
 	int is_spam = 0;
 	int sa;
-	char *msgtext;
-	size_t msglen;
+	StrBuf *msgtext;
+	CitContext *CCC=CC;
 
 	/* For users who have authenticated to this server we never want to
 	 * apply spam filtering, because presumably they're trustworthy.
@@ -107,40 +106,40 @@ int spam_assassin(struct CtdlMessage *msg) {
 		return(0);
 	}
 
+	CCC->sReadBuf = NewStrBuf();
+	CCC->sMigrateBuf = NewStrBuf();
+	CCC->sPos = NULL;
+
 	/* Command */
 	CtdlLogPrintf(CTDL_DEBUG, "Transmitting command\n");
 	sprintf(buf, "CHECK SPAMC/1.2\r\n\r\n");
-	sock_write(sock, buf, strlen(buf));
+	sock_write(&sock, buf, strlen(buf));
 
 	/* Message */
-	CC->redirect_buffer = malloc(SIZ);
-	CC->redirect_len = 0;
-	CC->redirect_alloc = SIZ;
+	CCC->redirect_buffer = NewStrBufPlain(NULL, SIZ);
 	CtdlOutputPreLoadedMsg(msg, MT_RFC822, HEADERS_ALL, 0, 1, 0);
 	msgtext = CC->redirect_buffer;
-	msglen = CC->redirect_len;
 	CC->redirect_buffer = NULL;
-	CC->redirect_len = 0;
-	CC->redirect_alloc = 0;
 
-	sock_write(sock, msgtext, msglen);
-	free(msgtext);
+	sock_write(&sock, SKEY(msgtext));
+	FreeStrBuf(&msgtext);
 
 	/* Close one end of the socket connection; this tells SpamAssassin
 	 * that we're done.
 	 */
-	sock_shutdown(sock, SHUT_WR);
+	if (sock != -1)
+		sock_shutdown(sock, SHUT_WR);
 	
 	/* Response */
 	CtdlLogPrintf(CTDL_DEBUG, "Awaiting response\n");
-        if (sock_getln(sock, buf, sizeof buf) < 0) {
+        if (sock_getln(&sock, buf, sizeof buf) < 0) {
                 goto bail;
         }
         CtdlLogPrintf(CTDL_DEBUG, "<%s\n", buf);
 	if (strncasecmp(buf, "SPAMD", 5)) {
 		goto bail;
 	}
-        if (sock_getln(sock, buf, sizeof buf) < 0) {
+        if (sock_getln(&sock, buf, sizeof buf) < 0) {
                 goto bail;
         }
         CtdlLogPrintf(CTDL_DEBUG, "<%s\n", buf);
@@ -191,6 +190,8 @@ int spam_assassin(struct CtdlMessage *msg) {
 	}
 
 bail:	close(sock);
+	FreeStrBuf(&CCC->sReadBuf);
+	FreeStrBuf(&CCC->sMigrateBuf);
 	return(is_spam);
 }
 
@@ -204,5 +205,5 @@ CTDL_MODULE_INIT(spam)
 	}
 	
 	/* return our Subversion id for the Log */
-        return "$Id: serv_spam.c 7921 2009-10-17 12:21:17Z davew $";
+        return "$Id: serv_spam.c 8525 2010-04-24 09:33:15Z dothebart $";
 }
