@@ -1,5 +1,5 @@
 /*
- * $Id: serv_upgrade.c 8294 2010-02-09 21:09:19Z ajc $
+ * $Id: serv_upgrade.c 8574 2010-05-18 21:34:56Z dothebart $
  *
  * Transparently handle the upgrading of server data formats.
  *
@@ -93,7 +93,9 @@ void fix_sys_user_name(void)
 	{
 		if (strncmp(usbuf.fullname, "SYS_", 4))
 		{	/** Delete any user 0 that doesn't start with SYS_ */
-			makeuserkey(usernamekey, usbuf.fullname);
+			makeuserkey(usernamekey, 
+				    usbuf.fullname, 
+				    cutuserkey(usbuf.fullname));
 			cdb_delete(CDB_USERS, usernamekey, strlen(usernamekey));
 		}
 		else
@@ -324,5 +326,5 @@ CTDL_MODULE_UPGRADE(upgrade)
 	check_server_upgrades();
 	
 	/* return our Subversion id for the Log */
-	return "$Id: serv_upgrade.c 8294 2010-02-09 21:09:19Z ajc $";
+	return "$Id: serv_upgrade.c 8574 2010-05-18 21:34:56Z dothebart $";
 }
