@@ -1,5 +1,5 @@
 /*
- * $Id: serv_wiki.c 8289 2010-02-09 19:42:25Z dothebart $
+ * $Id: serv_wiki.c 8686 2010-07-19 20:56:54Z dothebart $
  *
  * Server-side module for Wiki rooms.  This handles things like version control. 
  * 
@@ -53,7 +53,6 @@
 #include "config.h"
 #include "control.h"
 #include "user_ops.h"
-#include "policy.h"
 #include "database.h"
 #include "msgbase.h"
 #include "euidindex.h"
@@ -575,7 +574,7 @@ void wiki_rev(char *pagename, char *rev, char *operation)
 			fseek(fp, 0L, SEEK_SET);
 			msg->cm_fields['M'] = malloc(len + 1);
 			rv = fread(msg->cm_fields['M'], len, 1, fp);
-			CtdlLogPrintf(CTDL_DEBUG, "\033[32mdid %d blocks of %d bytes\033[0m\n", rv, len);
+			CtdlLogPrintf(CTDL_DEBUG, "did %d blocks of %d bytes\n", rv, len);
 			msg->cm_fields['M'][len] = 0;
 			fclose(fp);
 		}
@@ -663,5 +662,5 @@ CTDL_MODULE_INIT(wiki)
 	}
 
 	/* return our Subversion id for the Log */
-	return "$Id: serv_wiki.c 8289 2010-02-09 19:42:25Z dothebart $";
+	return "$Id: serv_wiki.c 8686 2010-07-19 20:56:54Z dothebart $";
 }
