@@ -1,4 +1,4 @@
-/* $Id: msgbase.h 7968 2009-10-25 22:05:03Z davew $ */
+/* $Id: msgbase.h 8430 2010-03-05 21:19:07Z ajc $ */
 
 #ifndef MSGBASE_H
 #define MSGBASE_H
@@ -12,7 +12,8 @@ enum {
 	MSGS_LAST,
 	MSGS_GT,
 	MSGS_EQ,
-	MSGS_SEARCH
+	MSGS_SEARCH,
+	MSGS_LT
 };
 
 enum {
@@ -95,7 +96,7 @@ void cmd_msgs (char *cmdbuf);
 void cmd_isme (char *cmdbuf);
 void help_subst (char *strbuf, char *source, char *dest);
 void do_help_subst (char *buffer);
-void memfmout (char *mptr, char subst, char *nl);
+void memfmout (char *mptr, const char *nl);
 void output_mime_parts(char *);
 void cmd_msg0 (char *cmdbuf);
 void cmd_msg2 (char *cmdbuf);
@@ -143,9 +144,18 @@ void dump_message(struct CtdlMessage *msg, long Siz);
 int is_valid_message(struct CtdlMessage *);
 void ReplicationChecks(struct CtdlMessage *);
 int CtdlSaveMsgPointersInRoom(char *roomname, long newmsgidlist[], int num_newmsgs,
-				int do_repl_check, struct CtdlMessage *supplied_msg);
+			int do_repl_check, struct CtdlMessage *supplied_msg, int suppress_refcount_adj);
 int CtdlSaveMsgPointerInRoom(char *roomname, long msgid, int do_repl_check, struct CtdlMessage *msg);
-char *CtdlReadMessageBody(char *terminator, size_t maxlen, char *exist, int crlf, int sock);
+char *CtdlReadMessageBody(char *terminator, long tlen, size_t maxlen, char *exist, int crlf, int *sock);
+StrBuf *CtdlReadMessageBodyBuf(char *terminator,	/* token signalling EOT */
+			       long tlen,
+			       size_t maxlen,		/* maximum message length */
+			       char *exist,		/* if non-null, append to it;
+							   exist is ALWAYS freed  */
+			       int crlf,		/* CRLF newlines instead of LF */
+			       int *sock		/* socket handle or 0 for this session's client socket */
+	);
+
 int CtdlOutputMsg(long msg_num,		/* message number (local) to fetch */
 		  int mode,		/* how would you like that message? */
 		  int headers_only,	/* eschew the message body? */
@@ -193,7 +203,7 @@ void CtdlSetSeen(long *target_msgnums, int num_target_msgnums,
 		struct ctdluser *which_user, struct ctdlroom *which_room);
 void CtdlGetSeen(char *buf, int which_set);
 
-struct recptypes *validate_recipients(char *recipients,
+struct recptypes *validate_recipients(const char *recipients,
  				      const char *RemoteIdentifier, 
 				      int Flags);
 
