@@ -1,5 +1,5 @@
 /*
- * $Id: sysdep.c 8017 2009-11-08 15:53:19Z davew $
+ * $Id: sysdep.c 8634 2010-06-20 22:28:28Z dothebart $
  *
  * Citadel "system dependent" stuff.
  * See COPYING for copyright information.
@@ -49,6 +49,7 @@
 #include <errno.h>
 #include <stdarg.h>
 #include <grp.h>
+#define SHOW_ME_VAPPEND_PRINTF
 #include <libcitadel.h>
 #include "citadel.h"
 #include "server.h"
@@ -92,7 +93,7 @@ int verbosity = DEFAULT_VERBOSITY;		/* Logging level */
 
 int syslog_facility = LOG_DAEMON;
 int enable_syslog = 0;
-
+int print_to_logfile = 1;
 
 /*
  * CtdlLogPrintf()  ...   Write logging information
@@ -106,44 +107,91 @@ void CtdlLogPrintf(enum LogLevel loglevel, const char *format, ...) {
 
 void vCtdlLogPrintf(enum LogLevel loglevel, const char *format, va_list arg_ptr)
 {
-	char buf[SIZ], buf2[SIZ];
 
 	if (enable_syslog) {
 		vsyslog((syslog_facility | loglevel), format, arg_ptr);
 	}
 
 	/* stderr output code */
-	if (enable_syslog || running_as_daemon) return;
+	if (enable_syslog || !print_to_logfile) return;
 
 	/* if we run in forground and syslog is disabled, log to terminal */
 	if (loglevel <= verbosity) { 
 		struct timeval tv;
 		struct tm tim;
 		time_t unixtime;
+		StrBuf *lBuf;
 		CitContext *CCC = CC;
+		ThreadTSD *cTSD = CTP;
+		CtdlThreadNode *node = NULL;
+		long lwpid = 0;
+
+		if (cTSD != NULL) 
+			node = cTSD->self;
+		if ((node != NULL) && (node->reltid != 0))
+		{
+			lwpid = node->reltid;
+		}
 
 		gettimeofday(&tv, NULL);
 		/* Promote to time_t; types differ on some OSes (like darwin) */
 		unixtime = tv.tv_sec;
 		localtime_r(&unixtime, &tim);
-		if ((CCC != NULL) && (CCC->cs_pid != 0)) {
-			sprintf(buf,
-				"%04d/%02d/%02d %2d:%02d:%02d.%06ld [%3d] ",
-				tim.tm_year + 1900, tim.tm_mon + 1,
-				tim.tm_mday, tim.tm_hour, tim.tm_min,
-				tim.tm_sec, (long)tv.tv_usec,
-				CCC->cs_pid);
-		} else {
-			sprintf(buf,
-				"%04d/%02d/%02d %2d:%02d:%02d.%06ld ",
-				tim.tm_year + 1900, tim.tm_mon + 1,
-				tim.tm_mday, tim.tm_hour, tim.tm_min,
-				tim.tm_sec, (long)tv.tv_usec);
-		}
-		vsnprintf(buf2, SIZ, format, arg_ptr);   
 
-		fprintf(stderr, "%s%s", buf, buf2);
+		if ((CCC != NULL) && (CCC != &masterCC))
+			lBuf = CCC->lBuf;
+		else 
+			lBuf = NewStrBuf();
+		if (lBuf == NULL) {
+			char buf[SIZ], buf2[SIZ];
+			
+			if ((CCC != NULL) && (CCC->cs_pid != 0)) {
+				sprintf(buf,
+					"%04d/%02d/%02d %2d:%02d:%02d.%06ld xx [%3d] ",
+					tim.tm_year + 1900, tim.tm_mon + 1,
+					tim.tm_mday, tim.tm_hour, tim.tm_min,
+					tim.tm_sec, (long)tv.tv_usec,
+					CCC->cs_pid);
+			} else {
+				sprintf(buf,
+					"%04d/%02d/%02d %2d:%02d:%02d.%06ld xx ",
+					tim.tm_year + 1900, tim.tm_mon + 1,
+					tim.tm_mday, tim.tm_hour, tim.tm_min,
+					tim.tm_sec, (long)tv.tv_usec);
+			}
+			vsnprintf(buf2, SIZ, format, arg_ptr);   
+
+			fprintf(stderr, ":%s%s", buf, buf2);
+		}
+		else {
+			StrBufPrintf(lBuf,
+				     "%04d/%02d/%02d %2d:%02d:%02d.%06ld ",
+				     tim.tm_year + 1900, tim.tm_mon + 1,
+				     tim.tm_mday, tim.tm_hour, tim.tm_min,
+				     tim.tm_sec, (long)tv.tv_usec);
+
+			
+			if (lwpid != 0)
+				StrBufAppendPrintf(lBuf,
+						   "[LWP:%d] ",
+						   lwpid);
+				
+			if (CCC != NULL) {
+				if (CCC->cs_pid != 0)
+					StrBufAppendPrintf(lBuf,
+							   "[%3d] ",
+							   CCC->cs_pid);
+				else if (CCC->user.usernum != 0)
+					StrBufAppendPrintf(lBuf,
+							   "[:%d] ",
+							   CCC->user.usernum);
+			}
+			StrBufVAppendPrintf(lBuf, format, arg_ptr);   
+			fwrite(ChrPtr(lBuf), 1, StrLength(lBuf), stderr);
+		}
 		fflush(stderr);
+		if (CCC == NULL) FreeStrBuf(&lBuf);
+
 	}
 }   
 
@@ -159,17 +207,20 @@ volatile int restart_server = 0;
 volatile int running_as_daemon = 0;
 
 static RETSIGTYPE signal_cleanup(int signum) {
-#ifdef THREADS_USESIGNALS
+
 	if (CT)
 		CT->signal = signum;
 	else
-#endif
 	{
 		CtdlLogPrintf(CTDL_DEBUG, "Caught signal %d; shutting down.\n", signum);
 		exit_signal = signum;
 	}
 }
 
+static RETSIGTYPE signal_exit(int signum) {
+	exit(1);
+}
+
 
 
 /*
@@ -210,21 +261,22 @@ void init_sysdep(void) {
 	 * call signal_cleanup() to gracefully shut down the server.
 	 */
 	sigemptyset(&set);
-	sigaddset(&set, SIGINT);
-	sigaddset(&set, SIGQUIT);
+	sigaddset(&set, SIGINT);		// intr = shutdown
+	// sigaddset(&set, SIGQUIT);		// quit = force quit
 	sigaddset(&set, SIGHUP);
 	sigaddset(&set, SIGTERM);
-	// sigaddset(&set, SIGSEGV);	commented out because
-	// sigaddset(&set, SIGILL);	we want core dumps
+	// sigaddset(&set, SIGSEGV);		// we want core dumps
+	// sigaddset(&set, SIGILL);		// we want core dumps
 	// sigaddset(&set, SIGBUS);
 	sigprocmask(SIG_UNBLOCK, &set, NULL);
 
-	signal(SIGINT, signal_cleanup);
-	signal(SIGQUIT, signal_cleanup);
+	signal(SIGINT, signal_cleanup);		// intr = shutdown
+	// signal(SIGQUIT, signal_cleanup);	// quit = force quit
 	signal(SIGHUP, signal_cleanup);
 	signal(SIGTERM, signal_cleanup);
-	// signal(SIGSEGV, signal_cleanup);	commented out because
-	// signal(SIGILL, signal_cleanup);	we want core dumps
+	signal(SIGUSR2, signal_exit);
+	// signal(SIGSEGV, signal_cleanup);	// we want coredumps
+	// signal(SIGILL, signal_cleanup);	// we want core dumps
 	// signal(SIGBUS, signal_cleanup);
 
 	/*
@@ -437,12 +489,21 @@ void flush_output(void) {
 #endif
 }
 
+/*
+static void flush_client_inbuf(void)
+{
+	CitContext *CCC=CC;
 
+	FlushStrBuf(CCC->ReadBuf);
+	CCC->Pos = NULL;
+
+}
+*/
 
 /*
  * client_write()   ...    Send binary data to the client.
  */
-int client_write(char *buf, int nbytes)
+int client_write(const char *buf, int nbytes)
 {
 	int bytes_written = 0;
 	int retval;
@@ -453,16 +514,13 @@ int client_write(char *buf, int nbytes)
 	CitContext *Ctx;
 	int fdflags;
 
+	if (nbytes < 1) return(0);
+
+//	flush_client_inbuf();
 	Ctx = CC;
 	if (Ctx->redirect_buffer != NULL) {
-		if ((Ctx->redirect_len + nbytes + 2) >= Ctx->redirect_alloc) {
-			Ctx->redirect_alloc = (Ctx->redirect_alloc * 2) + nbytes;
-			Ctx->redirect_buffer = realloc(Ctx->redirect_buffer,
-						Ctx->redirect_alloc);
-		}
-		memcpy(&Ctx->redirect_buffer[Ctx->redirect_len], buf, nbytes);
-		Ctx->redirect_len += nbytes;
-		Ctx->redirect_buffer[Ctx->redirect_len] = 0;
+		StrBufAppendBufPlain(Ctx->redirect_buffer,
+				     buf, nbytes, 0);
 		return 0;
 	}
 
@@ -472,21 +530,34 @@ int client_write(char *buf, int nbytes)
 		return 0;
 	}
 #endif
+	if (Ctx->client_socket == -1) return -1;
 
 	fdflags = fcntl(Ctx->client_socket, F_GETFL);
 
-	while (bytes_written < nbytes) {
+	while ((bytes_written < nbytes) && (Ctx->client_socket != -1)){
 		if ((fdflags & O_NONBLOCK) == O_NONBLOCK) {
 			FD_ZERO(&wset);
 			FD_SET(Ctx->client_socket, &wset);
 			if (select(1, NULL, &wset, NULL, NULL) == -1) {
-				CtdlLogPrintf(CTDL_ERR,
-					"client_write(%d bytes) select failed: %s (%d)\n",
-					nbytes - bytes_written,
-					strerror(errno), errno);
-				cit_backtrace();
-				Ctx->kill_me = 1;
-				return -1;
+				if (errno == EINTR)
+				{
+					CtdlLogPrintf(CTDL_DEBUG, "client_write(%d bytes) select() interrupted.\n", nbytes-bytes_written);
+					if (CtdlThreadCheckStop()) {
+						CC->kill_me = 1;
+						return (-1);
+					} else {
+						/* can't trust fd's and stuff so we need to re-create them */
+						continue;
+					}
+				} else {
+					CtdlLogPrintf(CTDL_ERR,
+						"client_write(%d bytes) select failed: %s (%d)\n",
+						nbytes - bytes_written,
+						strerror(errno), errno);
+					cit_backtrace();
+					Ctx->kill_me = 1;
+					return -1;
+				}
 			}
 		}
 
@@ -507,6 +578,10 @@ int client_write(char *buf, int nbytes)
 	return 0;
 }
 
+void cputbuf(const StrBuf *Buf) {   
+	client_write(ChrPtr(Buf), StrLength(Buf)); 
+}   
+
 
 /*
  * cprintf()	Send formatted printable data to the client.
@@ -526,60 +601,169 @@ void cprintf(const char *format, ...) {
 
 /*
  * Read data from the client socket.
- * Return values are:
- *	1	Requested number of bytes has been read.
- *	0	Request timed out.
- *	-1	The socket is broken.
- * If the socket breaks, the session will be terminated.
+ *
+ * sock		socket fd to read from
+ * buf		buffer to read into 
+ * bytes	number of bytes to read
+ * timeout	Number of seconds to wait before timing out
+ *
+ * Possible return values:
+ *      1       Requested number of bytes has been read.
+ *      0       Request timed out.
+ *	-1   	Connection is broken, or other error.
  */
-int client_read_to(char *buf, int bytes, int timeout)
+int client_read_blob(StrBuf *Target, int bytes, int timeout)
 {
-	int len,rlen;
-	fd_set rfds;
-	int fd;
-	struct timeval tv;
-	int retval;
+	CitContext *CCC=CC;
+	const char *Error;
+	int retval = 0;
 
 #ifdef HAVE_OPENSSL
-	if (CC->redirect_ssl) {
-		return (client_read_ssl(buf, bytes, timeout));
+	if (CCC->redirect_ssl) {
+		retval = client_read_sslblob(Target, bytes, timeout);
+		if (retval < 0) {
+			CtdlLogPrintf(CTDL_CRIT, 
+				      "%s failed\n",
+				      __FUNCTION__);
+		}
 	}
+	else 
 #endif
-	len = 0;
-	fd = CC->client_socket;
-	while(len<bytes) {
-		FD_ZERO(&rfds);
-		FD_SET(fd, &rfds);
-		tv.tv_sec = timeout;
-		tv.tv_usec = 0;
-
-		retval = select( (fd)+1, 
-				 &rfds, NULL, NULL, &tv);
-		if (retval < 0)
+	{
+		retval = StrBufReadBLOBBuffered(Target, 
+						CCC->ReadBuf,
+						&CCC->Pos,
+						&CCC->client_socket,
+						1, 
+						bytes,
+						O_TERM,
+						&Error);
+		if (retval < 0) {
+			CtdlLogPrintf(CTDL_CRIT, 
+				      "%s failed: %s\n",
+				      __FUNCTION__, 
+				      Error);
+			return retval;
+		}
+		else
 		{
-			if (errno == EINTR)
-			{
-				CtdlLogPrintf(CTDL_DEBUG, "Interrupted select().\n");
-				CC->kill_me = 1;
-				return (-1);
-			}
-		}
+#ifdef BIGBAD_IODBG
+			int rv = 0;
+			char fn [SIZ];
+			FILE *fd;
+			
+			snprintf(fn, SIZ, "/tmp/foolog_%s.%d", CCC->ServiceName, CCC->cs_pid);
+			
+			fd = fopen(fn, "a+");
+			fprintf(fd, "Read: BufSize: %d BufContent: [",
+				StrLength(Target));
+			rv = fwrite(ChrPtr(Target), StrLength(Target), 1, fd);
+			fprintf(fd, "]\n");
+			
+			
+			fclose(fd);
+#endif
 
-		if (FD_ISSET(fd, &rfds) == 0) {
-			return(0);
 		}
-
-		rlen = read(fd, &buf[len], bytes-len);
-		if (rlen<1) {
-			/* The socket has been disconnected! */
-			CC->kill_me = 1;
-			return(-1);
-		}
-		len = len + rlen;
 	}
-	return(1);
+	return retval;
 }
 
+
+/*
+ * to make client_read_random_blob() more efficient, increase buffer size.
+ * just use in greeting function, else your buffer may be flushed
+ */
+void client_set_inbound_buf(long N)
+{
+	FlushStrBuf(CC->ReadBuf);
+	ReAdjustEmptyBuf(CC->ReadBuf, N * SIZ, N * SIZ);
+}
+
+int client_read_random_blob(StrBuf *Target, int timeout)
+{
+	CitContext *CCC=CC;
+	int rc;
+
+	rc =  client_read_blob(Target, 1, timeout);
+	if (rc > 0)
+	{
+		long len;
+		const char *pch;
+		
+		len = StrLength(CCC->ReadBuf);
+		pch = ChrPtr(CCC->ReadBuf);
+
+		if (len > 0)
+		{
+			if (CCC->Pos != NULL) {
+				len -= CCC->Pos - pch;
+				pch = CCC->Pos;
+			}
+			StrBufAppendBufPlain(Target, pch, len, 0);
+			FlushStrBuf(CCC->ReadBuf);
+			CCC->Pos = NULL;
+#ifdef BIGBAD_IODBG
+			{
+				int rv = 0;
+				char fn [SIZ];
+				FILE *fd;
+			
+				snprintf(fn, SIZ, "/tmp/foolog_%s.%d", CCC->ServiceName, CCC->cs_pid);
+			
+				fd = fopen(fn, "a+");
+				fprintf(fd, "Read: BufSize: %d BufContent: [",
+					StrLength(Target));
+				rv = fwrite(ChrPtr(Target), StrLength(Target), 1, fd);
+				fprintf(fd, "]\n");
+			
+			
+				fclose(fd);
+			}
+#endif
+	
+			return StrLength(Target);
+		}
+		return rc;
+	}
+	else
+		return rc;
+}
+
+int client_read_to(char *buf, int bytes, int timeout)
+{
+	CitContext *CCC=CC;
+	int rc;
+
+	rc = client_read_blob(CCC->MigrateBuf, bytes, timeout);
+	if (rc < 0)
+	{
+		*buf = '\0';
+		return rc;
+	}
+	else
+	{
+		memcpy(buf, 
+		       ChrPtr(CCC->MigrateBuf),
+		       StrLength(CCC->MigrateBuf) + 1);
+		FlushStrBuf(CCC->MigrateBuf);
+		return rc;
+	}
+}
+
+
+int HaveMoreLinesWaiting(CitContext *CCC)
+{
+	if ((CCC->kill_me == 1) || (
+	    (CCC->Pos == NULL) && 
+	    (StrLength(CCC->ReadBuf) == 0) && 
+	    (CCC->client_socket != -1)) )
+		return 0;
+	else
+		return 1;
+}
+
+
 /*
  * Read data from the client socket with default timeout.
  * (This is implemented in terms of client_read_to() and could be
@@ -590,6 +774,130 @@ INLINE int client_read(char *buf, int bytes)
 	return(client_read_to(buf, bytes, config.c_sleeping));
 }
 
+int CtdlClientGetLine(StrBuf *Target)
+{
+	CitContext *CCC=CC;
+	const char *Error;
+	int rc;
+
+	FlushStrBuf(Target);
+#ifdef HAVE_OPENSSL
+	if (CCC->redirect_ssl) {
+#ifdef BIGBAD_IODBG
+		char fn [SIZ];
+		FILE *fd;
+		int len = 0;
+		int rlen = 0;
+		int  nlen = 0;
+		int nrlen = 0;
+		const char *pch;
+
+		snprintf(fn, SIZ, "/tmp/foolog_%s.%d", CCC->ServiceName, CCC->cs_pid);
+
+		fd = fopen(fn, "a+");
+		pch = ChrPtr(CCC->ReadBuf);
+		len = StrLength(CCC->ReadBuf);
+		if (CCC->Pos != NULL)
+			rlen = CC->Pos - pch;
+		else
+			rlen = 0;
+
+/*		fprintf(fd, "\n\n\nBufSize: %d BufPos: %d \nBufContent: [%s]\n\n_____________________\n",
+			len, rlen, pch);
+*/
+		fprintf(fd, "\n\n\nSSL1: BufSize: %d BufPos: %d \n_____________________\n",
+			len, rlen);
+#endif
+		rc = client_readline_sslbuffer(Target,
+					       CCC->ReadBuf,
+					       &CCC->Pos,
+					       1);
+#ifdef BIGBAD_IODBG
+                pch = ChrPtr(CCC->ReadBuf);
+                nlen = StrLength(CCC->ReadBuf);
+                if (CCC->Pos != NULL)
+                        nrlen = CC->Pos - pch;
+                else
+                        nrlen = 0;
+/*
+                fprintf(fd, "\n\n\nBufSize: was: %d is: %d BufPos: was: %d is: %d \nBufContent: [%s]\n\n_____________________\n",
+                        len, nlen, rlen, nrlen, pch);
+*/
+                fprintf(fd, "\n\n\nSSL2: BufSize: was: %d is: %d BufPos: was: %d is: %d \n",
+                        len, nlen, rlen, nrlen);
+
+                fprintf(fd, "SSL3: Read: BufSize: %d BufContent: [%s]\n\n*************\n",
+                        StrLength(Target), ChrPtr(Target));
+                fclose(fd);
+
+		if (rc < 0)
+			CtdlLogPrintf(CTDL_CRIT, 
+				      "%s failed\n",
+				      __FUNCTION__);
+#endif
+		return rc;
+	}
+	else 
+#endif
+	{
+#ifdef BIGBAD_IODBG
+		char fn [SIZ];
+		FILE *fd;
+		int len, rlen, nlen, nrlen;
+		const char *pch;
+
+		snprintf(fn, SIZ, "/tmp/foolog_%s.%d", CCC->ServiceName, CCC->cs_pid);
+
+		fd = fopen(fn, "a+");
+		pch = ChrPtr(CCC->ReadBuf);
+		len = StrLength(CCC->ReadBuf);
+		if (CCC->Pos != NULL)
+			rlen = CC->Pos - pch;
+		else
+			rlen = 0;
+
+/*		fprintf(fd, "\n\n\nBufSize: %d BufPos: %d \nBufContent: [%s]\n\n_____________________\n",
+			len, rlen, pch);
+*/
+		fprintf(fd, "\n\n\nBufSize: %d BufPos: %d \n_____________________\n",
+			len, rlen);
+#endif
+		rc = StrBufTCP_read_buffered_line_fast(Target, 
+						       CCC->ReadBuf,
+						       &CCC->Pos,
+						       &CCC->client_socket,
+						       5,
+						       1,
+						       &Error);
+
+#ifdef BIGBAD_IODBG
+                pch = ChrPtr(CCC->ReadBuf);
+                nlen = StrLength(CCC->ReadBuf);
+                if (CCC->Pos != NULL)
+                        nrlen = CC->Pos - pch;
+                else
+                        nrlen = 0;
+/*
+                fprintf(fd, "\n\n\nBufSize: was: %d is: %d BufPos: was: %d is: %d \nBufContent: [%s]\n\n_____________________\n",
+                        len, nlen, rlen, nrlen, pch);
+*/
+                fprintf(fd, "\n\n\nBufSize: was: %d is: %d BufPos: was: %d is: %d \n",
+                        len, nlen, rlen, nrlen);
+
+                fprintf(fd, "Read: BufSize: %d BufContent: [%s]\n\n*************\n",
+                        StrLength(Target), ChrPtr(Target));
+                fclose(fd);
+
+		if ((rc < 0) && (Error != NULL))
+			CtdlLogPrintf(CTDL_CRIT, 
+				      "%s failed: %s\n",
+				      __FUNCTION__,
+				      Error);
+#endif
+		return rc;
+	}
+}
+
 
 /*
  * client_getln()   ...   Get a LF-terminated line of text from the client.
@@ -599,32 +907,33 @@ INLINE int client_read(char *buf, int bytes)
 int client_getln(char *buf, int bufsize)
 {
 	int i, retval;
+	CitContext *CCC=CC;
+	const char *pCh;
 
-	/* Read one character at a time.
-	 */
-	for (i = 0;;i++) {
-		retval = client_read(&buf[i], 1);
-		if (retval != 1 || buf[i] == '\n' || i == (bufsize-1))
-			break;
-	}
+	retval = CtdlClientGetLine(CCC->MigrateBuf);
+	if (retval < 0)
+	  return(retval >= 0);
 
-	/* If we got a long line, discard characters until the newline.
-	 */
-	if (i == (bufsize-1))
-		while (buf[i] != '\n' && retval == 1)
-			retval = client_read(&buf[i], 1);
 
+	i = StrLength(CCC->MigrateBuf);
+	pCh = ChrPtr(CCC->MigrateBuf);
 	/* Strip the trailing LF, and the trailing CR if present.
 	 */
-	buf[i] = 0;
+	if (bufsize <= i)
+		i = bufsize - 1;
 	while ( (i > 0)
-		&& ( (buf[i - 1]==13)
-		     || ( buf[i - 1]==10)) ) {
+		&& ( (pCh[i - 1]==13)
+		     || ( pCh[i - 1]==10)) ) {
 		i--;
-		buf[i] = 0;
 	}
-	if (retval < 0) safestrncpy(&buf[i], "000", bufsize - i);
-	return(retval);
+	memcpy(buf, pCh, i);
+	buf[i] = 0;
+
+	FlushStrBuf(CCC->MigrateBuf);
+	if (retval < 0) {
+		safestrncpy(&buf[i], "000", bufsize - i);
+	}
+	return(retval >= 0);
 }
 
 
@@ -688,7 +997,7 @@ void sysdep_master_cleanup(void) {
 	CtdlDestroyServiceHook();
 	CtdlDestroyRoomHooks();
 	#ifdef HAVE_BACKTRACE
-	eCrash_Uninit();
+///	eCrash_Uninit();
 	#endif
 }
 
@@ -720,7 +1029,10 @@ void start_daemon(int unused) {
 	 * We don't just call close() because we don't want these fd's
 	 * to be reused for other files.
 	 */
-	chdir(ctdl_run_dir);
+	if (chdir(ctdl_run_dir) != 0)
+		CtdlLogPrintf(CTDL_EMERG, 
+			      "unable to change into directory [%s]: %s", 
+			      ctdl_run_dir, strerror(errno));
 
 	child = fork();
 	if (child != 0) {
@@ -733,9 +1045,13 @@ void start_daemon(int unused) {
 
 	setsid();
 	umask(0);
-        freopen("/dev/null", "r", stdin);
-        freopen("/dev/null", "w", stdout);
-        freopen("/dev/null", "w", stderr);
+        if ((freopen("/dev/null", "r", stdin) != stdin) || 
+	    (freopen("/dev/null", "w", stdout) != stdout) || 
+	    (freopen("/dev/null", "w", stderr) != stderr))
+		CtdlLogPrintf(CTDL_EMERG, 
+			      "unable to reopen stdin/out/err %s", 
+			      strerror(errno));
+		
 
 	do {
 		current_child = fork();
@@ -805,6 +1121,7 @@ void checkcrash(void)
 			" For more information, please see:\n \n"
 			" http://citadel.org/doku.php/faq:mastering_your_os:gdb#how.do.i.make.my.system.produce.core-files"
 			"\n \n"
+
 			" If you have already done this, the core dump is likely to be found at %score.%d\n"
 			,
 			ctdl_run_dir, ForkedPid);
@@ -866,18 +1183,13 @@ int convert_login(char NameToConvert[]) {
  */
  
 void *worker_thread(void *arg) {
-	int i;
 	int highest;
 	CitContext *ptr;
 	CitContext *bind_me = NULL;
 	fd_set readfds;
 	int retval = 0;
-	CitContext *con= NULL;	/* Temporary context pointer */
-	struct ServiceFunctionHook *serviceptr;
-	int ssock;			/* Descriptor for client socket */
 	struct timeval tv;
 	int force_purge = 0;
-	int m;
 	
 
 	while (!CtdlThreadCheckStop()) {
@@ -895,15 +1207,26 @@ do_select:	force_purge = 0;
 
 		begin_critical_section(S_SESSION_TABLE);
 		for (ptr = ContextList; ptr != NULL; ptr = ptr->next) {
+			int client_socket;
+			client_socket = ptr->client_socket;
 			/* Dont select on dead sessions only truly idle ones */
-			if ((ptr->state == CON_IDLE)) {
-				FD_SET(ptr->client_socket, &readfds);
-				if (ptr->client_socket > highest)
-					highest = ptr->client_socket;
+			if ((ptr->state == CON_IDLE) && 
+			    (CC->kill_me == 0) &&
+			    (client_socket != -1))
+			{
+				FD_SET(client_socket, &readfds);
+				if (client_socket > highest)
+					highest = client_socket;
 			}
 			if ((bind_me == NULL) && (ptr->state == CON_READY)) {
 				bind_me = ptr;
 				ptr->state = CON_EXECUTING;
+				break;
+			}
+			if ((bind_me == NULL) && (ptr->state == CON_GREETING)) {
+				bind_me = ptr;
+				ptr->state = CON_STARTING;
+				break;
 			}
 		}
 		end_critical_section(S_SESSION_TABLE);
@@ -917,16 +1240,6 @@ do_select:	force_purge = 0;
 		 * ahead and get ready to select().
 		 */
 
-		/* First, add the various master sockets to the fdset. */
-		for (serviceptr = ServiceHookTable; serviceptr != NULL;
-	    	serviceptr = serviceptr->next ) {
-			m = serviceptr->msock;
-			FD_SET(m, &readfds);
-			if (m > highest) {
-				highest = m;
-			}
-		}
-
 		if (!CtdlThreadCheckStop()) {
 			tv.tv_sec = 1;		/* wake up every second if no input */
 			tv.tv_usec = 0;
@@ -947,6 +1260,7 @@ do_select:	force_purge = 0;
 			if (errno != EINTR) {
 				CtdlLogPrintf(CTDL_EMERG, "Exiting (%s)\n", strerror(errno));
 				CtdlThreadStopAll();
+				continue;
 			} else {
 				CtdlLogPrintf(CTDL_DEBUG, "Interrupted CtdlThreadSelect.\n");
 				if (CtdlThreadCheckStop()) return(NULL);
@@ -955,7 +1269,148 @@ do_select:	force_purge = 0;
 		}
 		else if(retval == 0) {
 			if (CtdlThreadCheckStop()) return(NULL);
-			goto SKIP_SELECT;
+		}
+
+		/* It must be a client socket.  Find a context that has data
+		 * waiting on its socket *and* is in the CON_IDLE state.  Any
+		 * active sockets other than our chosen one are marked as
+		 * CON_READY so the next thread that comes around can just bind
+		 * to one without having to select() again.
+		 */
+		begin_critical_section(S_SESSION_TABLE);
+		for (ptr = ContextList; ptr != NULL; ptr = ptr->next) {
+			int checkfd = ptr->client_socket;
+			if ((checkfd != -1) && (ptr->state == CON_IDLE) ){
+				if (FD_ISSET(checkfd, &readfds)) {
+					ptr->input_waiting = 1;
+					if (!bind_me) {
+						bind_me = ptr;	/* I choose you! */
+						bind_me->state = CON_EXECUTING;
+					}
+					else {
+						ptr->state = CON_READY;
+					}
+				} else if ((ptr->is_async) && (ptr->async_waiting) && (ptr->h_async_function)) {
+					if (!bind_me) {
+						bind_me = ptr;	/* I choose you! */
+						bind_me->state = CON_EXECUTING;
+					}
+					else {
+						ptr->state = CON_READY;
+					}
+				}
+			}
+		}
+		end_critical_section(S_SESSION_TABLE);
+
+SKIP_SELECT:
+		/* We're bound to a session */
+		if (bind_me != NULL) {
+			become_session(bind_me);
+
+			if (bind_me->state == CON_STARTING) {
+				bind_me->state = CON_EXECUTING;
+				begin_session(bind_me);
+				bind_me->h_greeting_function();
+			}
+			/* If the client has sent a command, execute it. */
+			if (CC->input_waiting) {
+				CC->h_command_function();
+
+				while (HaveMoreLinesWaiting(CC))
+				       CC->h_command_function();
+
+				CC->input_waiting = 0;
+			}
+
+			/* If there are asynchronous messages waiting and the
+			 * client supports it, do those now */
+			if ((CC->is_async) && (CC->async_waiting)
+			   && (CC->h_async_function != NULL)) {
+				CC->h_async_function();
+				CC->async_waiting = 0;
+			}
+			
+			force_purge = CC->kill_me;
+			become_session(NULL);
+			bind_me->state = CON_IDLE;
+		}
+
+		dead_session_purge(force_purge);
+		do_housekeeping();
+	}
+	/* If control reaches this point, the server is shutting down */	
+	return(NULL);
+}
+
+
+
+
+/*
+ * A function to handle selecting on master sockets.
+ * In other words it handles new connections.
+ * It is a thread.
+ */
+void *select_on_master (void *arg)
+{
+	struct ServiceFunctionHook *serviceptr;
+	fd_set master_fds;
+	int highest;
+	struct timeval tv;
+	int ssock;			/* Descriptor for client socket */
+	CitContext *con= NULL;	/* Temporary context pointer */
+	int m;
+	int i;
+	int retval;
+	struct CitContext select_on_master_CC;
+
+	CtdlFillSystemContext(&select_on_master_CC, "select_on_master");
+	citthread_setspecific(MyConKey, (void *)&select_on_master_CC);
+
+	while (!CtdlThreadCheckStop()) {
+		/* Initialize the fdset. */
+		FD_ZERO(&master_fds);
+		highest = 0;
+
+		/* First, add the various master sockets to the fdset. */
+		for (serviceptr = ServiceHookTable; serviceptr != NULL;
+	    	serviceptr = serviceptr->next ) {
+			m = serviceptr->msock;
+			FD_SET(m, &master_fds);
+			if (m > highest) {
+				highest = m;
+			}
+		}
+
+		if (!CtdlThreadCheckStop()) {
+			tv.tv_sec = 60;		/* wake up every second if no input */
+			tv.tv_usec = 0;
+			retval = CtdlThreadSelect(highest + 1, &master_fds, NULL, NULL, &tv);
+		}
+		else
+			return NULL;
+
+		/* Now figure out who made this select() unblock.
+		 * First, check for an error or exit condition.
+		 */
+		if (retval < 0) {
+			if (errno == EBADF) {
+				CtdlLogPrintf(CTDL_NOTICE, "select() failed: (%s)\n",
+					strerror(errno));
+				continue;
+			}
+			if (errno != EINTR) {
+				CtdlLogPrintf(CTDL_EMERG, "Exiting (%s)\n", strerror(errno));
+				CtdlThreadStopAll();
+			} else {
+				CtdlLogPrintf(CTDL_DEBUG, "Interrupted CtdlThreadSelect.\n");
+				if (CtdlThreadCheckStop()) return(NULL);
+				continue;
+			}
+		}
+		else if(retval == 0) {
+			if (CtdlThreadCheckStop()) return(NULL);
+			continue;
 		}
 		/* Next, check to see if it's a new client connecting
 		 * on a master socket.
@@ -963,7 +1418,7 @@ do_select:	force_purge = 0;
 		else for (serviceptr = ServiceHookTable; serviceptr != NULL;
 		     serviceptr = serviceptr->next ) {
 
-			if (FD_ISSET(serviceptr->msock, &readfds)) {
+			if (FD_ISSET(serviceptr->msock, &master_fds)) {
 				ssock = accept(serviceptr->msock, NULL, 0);
 				if (ssock >= 0) {
 					CtdlLogPrintf(CTDL_DEBUG,
@@ -991,6 +1446,7 @@ do_select:	force_purge = 0;
 						serviceptr->h_command_function;
 					con->h_async_function =
 						serviceptr->h_async_function;
+					con->h_greeting_function = serviceptr->h_greeting_function;
 					con->ServiceName =
 						serviceptr->ServiceName;
 					
@@ -1004,72 +1460,19 @@ do_select:	force_purge = 0;
 						SO_REUSEADDR,
 						&i, sizeof(i));
 
-					become_session(con);
-					begin_session(con);
-					serviceptr->h_greeting_function();
-					become_session(NULL);
-					con->state = CON_IDLE;
+					con->state = CON_GREETING;
+
 					retval--;
-					if (retval)
-						CtdlLogPrintf (CTDL_DEBUG, "Select said more than 1 fd to handle but we only handle one\n");
-					goto do_select;
+					if (retval == 0)
+						break;
 				}
 			}
 		}
-
-		/* It must be a client socket.  Find a context that has data
-		 * waiting on its socket *and* is in the CON_IDLE state.  Any
-		 * active sockets other than our chosen one are marked as
-		 * CON_READY so the next thread that comes around can just bind
-		 * to one without having to select() again.
-		 */
-		begin_critical_section(S_SESSION_TABLE);
-		for (ptr = ContextList; ptr != NULL; ptr = ptr->next) {
-			if ( (FD_ISSET(ptr->client_socket, &readfds))
-			   && (ptr->state != CON_EXECUTING) ) {
-				ptr->input_waiting = 1;
-				if (!bind_me) {
-					bind_me = ptr;	/* I choose you! */
-					bind_me->state = CON_EXECUTING;
-				}
-				else {
-					ptr->state = CON_READY;
-				}
-			}
-		}
-		end_critical_section(S_SESSION_TABLE);
-
-SKIP_SELECT:
-		/* We're bound to a session */
-		if (bind_me != NULL) {
-			become_session(bind_me);
-
-			/* If the client has sent a command, execute it. */
-			if (CC->input_waiting) {
-				CC->h_command_function();
-				CC->input_waiting = 0;
-			}
-
-			/* If there are asynchronous messages waiting and the
-			 * client supports it, do those now */
-			if ((CC->is_async) && (CC->async_waiting)
-			   && (CC->h_async_function != NULL)) {
-				CC->h_async_function();
-				CC->async_waiting = 0;
-			}
-			
-			force_purge = CC->kill_me;
-			become_session(NULL);
-			bind_me->state = CON_IDLE;
-		}
-
-		dead_session_purge(force_purge);
-		do_housekeeping();
 	}
-	/* If control reaches this point, the server is shutting down */	
-	return(NULL);
-}
+	CtdlClearSystemContext();
 
+	return NULL;
+}
 
 
 
