@@ -1,4 +1,4 @@
-/* $Id: messages.h 7944 2009-10-24 15:35:16Z davew $ */
+/* $Id: messages.h 8426 2010-03-04 15:09:52Z ajc $ */
 
 #define MAXURLS		50	/* Max embedded URL's per message */
 extern int num_urls;
@@ -19,6 +19,6 @@ int client_make_message(CtdlIPC *ipc,
 						int mode,
 						char *subject,
 						int subject_required);
-void citedit(CtdlIPC *ipc, FILE *);
+void citedit(FILE *);
 char *load_message_from_file(FILE *src);
 int file_checksum(char *filename);
