@@ -1,9 +1,23 @@
 /*
- * $Id: rooms.c 7944 2009-10-24 15:35:16Z davew $
+ * $Id: rooms.c 8428 2010-03-04 20:02:46Z ajc $
  *
- * 
  * Client-side functions which perform room operations
  *
+ * Copyright (c) 1987-2010 by the citadel.org team
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "sysdep.h"
@@ -1242,10 +1256,9 @@ void do_edit(CtdlIPC *ipc,
 		stty_ctdl(0);
 		screen_set();
 	} else {
-		scr_printf("Entering %s.  "
-			"Press return twice when finished.\n", desc);
+		scr_printf("Entering %s.  Press return twice when finished.\n", desc);
 		fp = fopen(temp, "r+");
-		citedit(ipc, fp);
+		citedit(fp);
 		fclose(fp);
 	}
 
