@@ -1,5 +1,5 @@
 /* 
- * $Id: user_ops.c 8032 2009-11-10 19:39:19Z davew $
+ * $Id: user_ops.c 8591 2010-05-21 22:38:09Z dothebart $
  *
  * Server functions which perform operations on user objects.
  *
@@ -38,7 +38,6 @@
 #include "citadel.h"
 #include "server.h"
 #include "database.h"
-#include "user_ops.h"
 #include "sysdep_decls.h"
 #include "support.h"
 #include "room_ops.h"
@@ -55,29 +54,13 @@
 
 #include "ctdl_module.h"
 
+////#define CTDL_INLINE_USR INLINE
+#include "user_ops.h"
+
 /* These pipes are used to talk to the chkpwd daemon, which is forked during startup */
 int chkpwd_write_pipe[2];
 int chkpwd_read_pipe[2];
 
-/*
- * makeuserkey() - convert a username into the format used as a database key
- *		 (it's just the username converted into lower case)
- */
-INLINE void makeuserkey(char *key, char *username) {
-	int i, len;
-
-	len = strlen(username);
-	if (len >= USERNAME_SIZE)
-	{
-		CtdlLogPrintf (CTDL_EMERG, "Username to long: %s", username);
-		cit_backtrace ();
-		len = USERNAME_SIZE - 1; 
-		username[USERNAME_SIZE - 1]='\0';
-	}
-	for (i=0; i<=len; ++i) {
-		key[i] = tolower(username[i]);
-	}
-}
 
 
 /*
@@ -94,7 +77,7 @@ int getuser(struct ctdluser *usbuf, char name[])
  * CtdlGetUser()  -  retrieve named user into supplied buffer.
  *	       returns 0 on success
  */
-int CtdlGetUser(struct ctdluser *usbuf, char name[])
+int CtdlGetUserLen(struct ctdluser *usbuf, const char *name, long len)
 {
 
 	char usernamekey[USERNAME_SIZE];
@@ -104,7 +87,7 @@ int CtdlGetUser(struct ctdluser *usbuf, char name[])
 		memset(usbuf, 0, sizeof(struct ctdluser));
 	}
 
-	makeuserkey(usernamekey, name);
+	makeuserkey(usernamekey, name, len);
 	cdbus = cdb_fetch(CDB_USERS, usernamekey, strlen(usernamekey));
 
 	if (cdbus == NULL) {	/* user not found */
@@ -121,6 +104,12 @@ int CtdlGetUser(struct ctdluser *usbuf, char name[])
 }
 
 
+int CtdlGetUser(struct ctdluser *usbuf, char *name)
+{
+	return CtdlGetUserLen(usbuf, name, cutuserkey(name));
+}
+
+
 /*
  * CtdlGetUserLock()  -  same as getuser() but locks the record
  */
@@ -152,7 +141,9 @@ void CtdlPutUser(struct ctdluser *usbuf)
 {
 	char usernamekey[USERNAME_SIZE];
 
-	makeuserkey(usernamekey, usbuf->fullname);
+	makeuserkey(usernamekey, 
+		    usbuf->fullname, 
+		    cutuserkey(usbuf->fullname));
 
 	usbuf->version = REV_LEVEL;
 	cdb_store(CDB_USERS,
@@ -204,8 +195,8 @@ int rename_user(char *oldname, char *newname) {
 	char newnamekey[USERNAME_SIZE];
 
 	/* Create the database keys... */
-	makeuserkey(oldnamekey, oldname);
-	makeuserkey(newnamekey, newname);
+	makeuserkey(oldnamekey, oldname, cutuserkey(oldname));
+	makeuserkey(newnamekey, newname, cutuserkey(newname));
 
 	/* Lock up and get going */
 	begin_critical_section(S_USERS);
@@ -377,7 +368,7 @@ void MailboxName(char *buf, size_t n, const struct ctdluser *who, const char *pr
  */
 int is_aide(void)
 {
-	if (CC->user.axlevel >= 6)
+	if (CC->user.axlevel >= AxAideU)
 		return (1);
 	else
 		return (0);
@@ -394,7 +385,7 @@ int is_room_aide(void)
 		return (0);
 	}
 
-	if ((CC->user.axlevel >= 6)
+	if ((CC->user.axlevel >= AxAideU)
 	    || (CC->room.QRroomaide == CC->user.usernum)) {
 		return (1);
 	} else {
@@ -523,10 +514,11 @@ int getuserbyuid(struct ctdluser *usbuf, uid_t number)
  *
  * NOTE: "authname" should only be used if we are attempting to use the "master user" feature
  */
-int CtdlLoginExistingUser(char *authname, char *trythisname)
+int CtdlLoginExistingUser(char *authname, const char *trythisname)
 {
 	char username[SIZ];
 	int found_user;
+	long len;
 
 	CtdlLogPrintf(9, "CtdlLoginExistingUser(%s, %s)\n", authname, trythisname);
 
@@ -551,8 +543,9 @@ int CtdlLoginExistingUser(char *authname, char *trythisname)
 	}
 
 	/* Continue attempting user validation... */
-	safestrncpy(username, trythisname, USERNAME_SIZE);
+	safestrncpy(username, trythisname, sizeof (username));
 	striplt(username);
+	len = cutuserkey(username);
 
 	if (IsEmptyStr(username)) {
 		return login_not_found;
@@ -591,7 +584,8 @@ int CtdlLoginExistingUser(char *authname, char *trythisname)
 		CtdlLogPrintf(CTDL_DEBUG, "found it: uid=%ld, gecos=%s here: %d\n",
 			(long)pd.pw_uid, pd.pw_gecos, found_user);
 		if (found_user != 0) {
-			create_user(username, 0);
+			len = cutuserkey(username);
+			create_user(username, len, 0);
 			found_user = getuserbyuid(&CC->user, pd.pw_uid);
 		}
 
@@ -613,7 +607,7 @@ int CtdlLoginExistingUser(char *authname, char *trythisname)
 
 		found_user = getuserbyuid(&CC->user, ldap_uid);
 		if (found_user != 0) {
-			create_user(trythisname, 0);
+			create_user(username, len, 0);
 			found_user = getuserbyuid(&CC->user, ldap_uid);
 		}
 
@@ -649,7 +643,7 @@ int CtdlLoginExistingUser(char *authname, char *trythisname)
 
 	/* Did we find something? */
 	if (found_user == 0) {
-		if (((CC->nologin)) && (CC->user.axlevel < 6)) {
+		if (((CC->nologin)) && (CC->user.axlevel < AxAideU)) {
 			return login_too_many_users;
 		} else {
 			safestrncpy(CC->curr_user, CC->user.fullname,
@@ -719,7 +713,7 @@ void do_login(void)
 	 * (as specified in setup), automatically assign access level 6.
 	 */
 	if (!strcasecmp(CC->user.fullname, config.c_sysadm)) {
-		CC->user.axlevel = 6;
+		CC->user.axlevel = AxAideU;
 	}
 
 	/* If we're authenticating off the host system, automatically give
@@ -727,7 +721,7 @@ void do_login(void)
 	 */
 	if (config.c_auth_mode == AUTHMODE_HOST) {
 		if (CC->user.uid == 0) {
-			CC->user.axlevel = 6;
+			CC->user.axlevel = AxAideU;
 		}
 	}
 
@@ -818,7 +812,7 @@ void CtdlUserLogout(void)
 	CCC->logged_in = 0;
 
 	/* Check to see if the user was deleted whilst logged in and purge them if necessary */
-	if ((CCC->user.axlevel == 0) && (CCC->user.usernum))
+	if ((CCC->user.axlevel == AxDeleted) && (CCC->user.usernum))
 		purge_user(CCC->user.fullname);
 
 	/* Free any output buffers */
@@ -900,7 +894,7 @@ void start_chkpwd_daemon(void) {
 }
 
 
-int CtdlTryPassword(char *password)
+int CtdlTryPassword(const char *password, long len)
 {
 	int code;
 
@@ -972,13 +966,17 @@ int CtdlTryPassword(char *password)
 	else {
 
 		/* native auth mode */
+		char *pw;
 
-		strproc(password);
+		pw = (char*) malloc(len + 1);
+		memcpy(pw, password, len + 1);
+		strproc(pw);
 		strproc(CC->user.password);
-		code = strcasecmp(CC->user.password, password);
-		strproc(password);
+		code = strcasecmp(CC->user.password, pw);
+		strproc(pw);
 		strproc(CC->user.password);
-		code = strcasecmp(CC->user.password, password);
+		code = strcasecmp(CC->user.password, pw);
+		free (pw);
 	}
 
 	if (!code) {
@@ -993,12 +991,13 @@ int CtdlTryPassword(char *password)
 
 void cmd_pass(char *buf)
 {
-	char password[256];
+	char password[SIZ];
 	int a;
+	long len;
 
 	memset(password, 0, sizeof(password));
-	extract_token(password, buf, 0, '|', sizeof password);
-	a = CtdlTryPassword(password);
+	len = extract_token(password, buf, 0, '|', sizeof password);
+	a = CtdlTryPassword(password, len);
 
 	switch (a) {
 	case pass_already_logged_in:
@@ -1028,7 +1027,7 @@ int purge_user(char pname[])
 	struct ctdluser usbuf;
 	char usernamekey[USERNAME_SIZE];
 
-	makeuserkey(usernamekey, pname);
+	makeuserkey(usernamekey, pname, cutuserkey(pname));
 
 	/* If the name is empty we can't find them in the DB any way so just return */
 	if (IsEmptyStr(pname))
@@ -1044,7 +1043,7 @@ int purge_user(char pname[])
 	 */
 	if (CtdlIsUserLoggedInByNum(usbuf.usernum)) {
 		CtdlLogPrintf(CTDL_WARNING, "User <%s> is logged in; not deleting.\n", pname);
-		usbuf.axlevel = 0;
+		usbuf.axlevel = AxDeleted;
 		CtdlPutUser(&usbuf);
 		return (1);
 	}
@@ -1092,9 +1091,9 @@ int purge_user(char pname[])
 }
 
 
-int internal_create_user (char *username, struct ctdluser *usbuf, uid_t uid)
+int internal_create_user (const char *username, long len, struct ctdluser *usbuf, uid_t uid)
 {
-	if (!CtdlGetUser(usbuf, username)) {
+	if (!CtdlGetUserLen(usbuf, username, len)) {
 		return (ERROR + ALREADY_EXISTS);
 	}
 
@@ -1133,7 +1132,7 @@ int internal_create_user (char *username, struct ctdluser *usbuf, uid_t uid)
  * Set 'become_user' to nonzero if this is self-service account creation and we want
  * to actually log in as the user we just created, otherwise set it to 0.
  */
-int create_user(char *newusername, int become_user)
+int create_user(const char *newusername, long len, int become_user)
 {
 	struct ctdluser usbuf;
 	struct ctdlroom qrbuf;
@@ -1154,7 +1153,7 @@ int create_user(char *newusername, int become_user)
 
 		struct passwd pd;
 		struct passwd *tempPwdPtr;
-		char pwdbuffer[256];
+		char pwdbuffer[SIZ];
 	
 #ifdef HAVE_GETPWNAM_R
 #ifdef SOLARIS_GETPWUID
@@ -1171,6 +1170,7 @@ int create_user(char *newusername, int become_user)
 			if (IsEmptyStr (username))
 			{
 				safestrncpy(username, pd.pw_name, sizeof username);
+				len = cutuserkey(username);
 			}
 		}
 		else {
@@ -1186,7 +1186,7 @@ int create_user(char *newusername, int become_user)
 	}
 #endif /* HAVE_LDAP */
 	
-	if ((retval = internal_create_user(username, &usbuf, uid)) != 0)
+	if ((retval = internal_create_user(username, len, &usbuf, uid)) != 0)
 		return retval;
 	
 	/*
@@ -1241,6 +1241,7 @@ int create_user(char *newusername, int become_user)
 void cmd_newu(char *cmdbuf)
 {
 	int a;
+	long len;
 	char username[26];
 
 	if (config.c_auth_mode != AUTHMODE_NATIVE) {
@@ -1265,8 +1266,8 @@ void cmd_newu(char *cmdbuf)
 			config.c_nodename, config.c_maxsessions);
 	}
 	extract_token(username, cmdbuf, 0, '|', sizeof username);
-	username[25] = 0;
 	strproc(username);
+	len = cutuserkey(username);
 
 	if (IsEmptyStr(username)) {
 		cprintf("%d You must supply a user name.\n", ERROR + USERNAME_REQUIRED);
@@ -1280,7 +1281,7 @@ void cmd_newu(char *cmdbuf)
 		return;
 	}
 
-	a = create_user(username, 1);
+	a = create_user(username, len, 1);
 
 	if (a == 0) {
 		logged_in_response();
@@ -1356,8 +1357,9 @@ void cmd_setp(char *new_pw)
 void cmd_creu(char *cmdbuf)
 {
 	int a;
-	char username[26];
-	char password[32];
+	long len;
+	char username[SIZ];
+	char password[SIZ];
 	struct ctdluser tmp;
 
 	if (CtdlAccessCheck(ac_aide)) {
@@ -1366,17 +1368,18 @@ void cmd_creu(char *cmdbuf)
 
 	extract_token(username, cmdbuf, 0, '|', sizeof username);
 	extract_token(password, cmdbuf, 1, '|', sizeof password);
-	username[25] = 0;
-	password[31] = 0;
+	////username[25] = 0;
+	//password[31] = 0;
 	strproc(username);
 	strproc(password);
+	len = cutuserkey(username);
 
 	if (IsEmptyStr(username)) {
 		cprintf("%d You must supply a user name.\n", ERROR + USERNAME_REQUIRED);
 		return;
 	}
 
-	a = create_user(username, 0);
+	a = create_user(username, len, 0);
 
 	if (a == 0) {
 		if (!IsEmptyStr(password)) {
@@ -1668,7 +1671,7 @@ void cmd_gnur(char *argbuf)
 			sizeof(struct ctdluser) : cdbus->len));
 		cdb_free(cdbus);
 		if ((usbuf.flags & US_NEEDVALID)
-		    && (usbuf.axlevel > 0)) {
+		    && (usbuf.axlevel > AxDeleted)) {
 			cprintf("%d %s\n", MORE_DATA, usbuf.fullname);
 			cdb_close_cursor(CDB_USERS);
 			return;
@@ -1702,7 +1705,9 @@ void cmd_vali(char *v_args)
 	extract_token(user, v_args, 0, '|', sizeof user);
 	newax = extract_int(v_args, 1);
 
-	if (CtdlAccessCheck(ac_aide)) {
+	if (CtdlAccessCheck(ac_aide) || 
+	    (newax > AxAideU) ||
+	    (newax < AxDeleted)) {
 		return;
 	}
 
@@ -1762,8 +1767,8 @@ void ListThisUser(struct ctdluser *usbuf, void *data)
 		return;
 	}
 
-	if (usbuf->axlevel > 0) {
-		if ((CC->user.axlevel >= 6)
+	if (usbuf->axlevel > AxDeleted) {
+		if ((CC->user.axlevel >= AxAideU)
 		    || ((usbuf->flags & US_UNLISTED) == 0)
 		    || ((CC->internal_pgm))) {
 			cprintf("%s|%d|%ld|%ld|%ld|%ld||\n",
@@ -1810,7 +1815,7 @@ void cmd_chek(char *argbuf)
 	if ((REGISCALL != 0) && ((CC->user.flags & US_REGIS) == 0))
 		regis = 1;
 
-	if (CC->user.axlevel >= 6) {
+	if (CC->user.axlevel >= AxAideU) {
 		get_control();
 		if (CitControl.MMflags & MM_VALID)
 			vali = 1;
@@ -1901,8 +1906,8 @@ void cmd_asup(char *cmdbuf)
 		usbuf.posted = extract_int(cmdbuf, 4);
 	if (np > 5) {
 		newax = extract_int(cmdbuf, 5);
-		if ((newax >= 0) && (newax <= 6)) {
-			usbuf.axlevel = extract_int(cmdbuf, 5);
+		if ((newax >= AxDeleted) && (newax <= AxAideU)) {
+			usbuf.axlevel = newax;
 		}
 	}
 	if (np > 7) {
@@ -1912,7 +1917,7 @@ void cmd_asup(char *cmdbuf)
 		usbuf.USuserpurge = extract_int(cmdbuf, 8);
 	}
 	CtdlPutUserLock(&usbuf);
-	if (usbuf.axlevel == 0) {
+	if (usbuf.axlevel == AxDeleted) {
 		if (purge_user(requested_user) == 0) {
 			deleted = 1;
 		}
@@ -2081,5 +2086,5 @@ CTDL_MODULE_INIT(user_ops)
 		CtdlRegisterProtoHook(cmd_newu, "NEWU", "Autoconverted. TODO: document me.");
 	}
 	/* return our Subversion id for the Log */
-	return "$Id: user_ops.c 8032 2009-11-10 19:39:19Z davew $";
+	return "$Id: user_ops.c 8591 2010-05-21 22:38:09Z dothebart $";
 }
