@@ -1,4 +1,4 @@
-/* $Id: citadel_ipc.c 8289 2010-02-09 19:42:25Z dothebart $ 
+/* $Id: citadel_ipc.c 8639 2010-06-22 21:11:59Z dothebart $ 
  *
  * Copyright (c) 1987-2009 by the citadel.org team
  *
@@ -1108,7 +1108,7 @@ int CtdlIPCValidateUser(CtdlIPC *ipc, const char *username, int axlevel, char *c
 
 	if (!cret) return -2;
 	if (!username) return -2;
-	if (axlevel < 0 || axlevel > 7) return -2;
+	if (axlevel < AxDeleted || axlevel > AxAideU) return -2;
 
 	aaa = (char *)malloc(strlen(username) + 17);
 	if (!aaa) return -1;
@@ -1860,10 +1860,15 @@ int CtdlIPCAideSetUserParameters(CtdlIPC *ipc, const struct ctdluser *uret, char
 /* GPEX */
 /* which is 0 = room, 1 = floor, 2 = site, 3 = default for mailboxes */
 /* caller must free the struct ExpirePolicy */
-int CtdlIPCGetMessageExpirationPolicy(CtdlIPC *ipc, int which,
+int CtdlIPCGetMessageExpirationPolicy(CtdlIPC *ipc, GPEXWhichPolicy which,
 		struct ExpirePolicy **policy, char *cret)
 {
-	static char *proto[] = {"room", "floor", "site", "mailboxes" };
+	static char *proto[] = {
+		strof(roompolicy),
+		strof(floorpolicy),
+		strof(sitepolicy),
+		strof(mailboxespolicy)
+	};
 	char cmd[256];
 	register int ret;
 
@@ -2043,7 +2048,7 @@ int CtdlIPCStartEncryption(CtdlIPC *ipc, char *cret)
 	}
 	/* Pointless flag waving */
 #if SSLEAY_VERSION_NUMBER >= 0x0922
-	SSL_set_session_id_context(temp_ssl, "Citadel SID", 14);
+	SSL_set_session_id_context(temp_ssl, (const unsigned char*) "Citadel SID", 14);
 #endif
 
 	if (!access(EGD_POOL, F_OK))
@@ -2090,7 +2095,7 @@ int CtdlIPCStartEncryption(CtdlIPC *ipc, char *cret)
 	}
 	ipc->ssl = temp_ssl;
 
-	BIO_set_close(ipc->ssl->rbio, BIO_NOCLOSE);
+	if (BIO_set_close(ipc->ssl->rbio, BIO_NOCLOSE))
 	{
 		int bits, alg_bits;
 
@@ -2183,24 +2188,6 @@ int CtdlIPCMessageBaseCheck(CtdlIPC *ipc, char **mret, char *cret)
 /* ************************************************************************** */
 
 
-INLINE void CtdlIPC_lock(CtdlIPC *ipc)
-{
-	if (ipc->network_status_cb) ipc->network_status_cb(1);
-#ifdef THREADED_CLIENT
-	pthread_mutex_lock(&(ipc->mutex));
-#endif
-}
-
-
-INLINE void CtdlIPC_unlock(CtdlIPC *ipc)
-{
-#ifdef THREADED_CLIENT
-	pthread_mutex_unlock(&(ipc->mutex));
-#endif
-	if (ipc->network_status_cb) ipc->network_status_cb(0);
-}
-
-
 /* Read a listing from the server up to 000.  Append to dest if it exists */
 char *CtdlIPCReadListing(CtdlIPC *ipc, char *dest)
 {
